%% 3D diffusion with mixed terms
%
% This example calculates 3D radiation belt simulation in (L*, V, K) 
% coordinates. 
% Two plots are plotted at the end:
% 1. Flux as a function of (L*, time) for constant energy and pitch-angle 
% 2. PSD as a function of L*, time for constant mu and K
% Difusion due to Chorus day and night-side waves is considered outside of
% the plasmasphere and hiss diffusion due to hiss waves is considered
% inside of the plasmasphere

% Include VERB4D_Library into the path
addpath('../../VERB4D_Library');



clear all
disp('3D diffusion')

% Here we specify folders, where we save data
% VERB4D_input and VERB4D_output are expected by VERB4D_Solver and can not
% be changed. VERB4D_prep is a temporary folder for Matlab files
VERB4D_input_folder = 'VERB4D_input/'; % input folder for VERB4D_Solver
VERB4D_output_folder = 'VERB4D_output/'; % output folder for VERB4D_Solver
VERB4D_prep_folder = 'VERB4D_prep/'; % folder for temporary Matbal files
% Diffusion coefficient folders:
DiffCoeff_input_folder = '../../'; % Where to read diffusion coefficients
DiffCoeff_output_folder = 'VERB4D_input/DiffCoeff_Q/'; % Where to save

mkdir(VERB4D_input_folder);
mkdir(VERB4D_output_folder);
mkdir(VERB4D_prep_folder)
mkdir(DiffCoeff_output_folder);

% Computational time
start_date = datenum('12-Nov-2012');
total_time = 1; %days
end_date = start_date + total_time;
time_step = 1/(24/1); %days
output_time = 1/(24/1); %days
time = [start_date:time_step:start_date+total_time]';

Kp0 = 1; % Kp for initial PSD
DxxKp = 4; % Kp for which diffusion coefficients were calculated. Used for scaling.


% Setting this to true will make all input and output files in .mat format
% as well as calling the .mat plot functions
% 
% if false will save all inputs and outputs into .plt/.dat form which is slower
use_matlab = true;


% Save all folder and parameters into a temporary file, so other
% subroutines can easily access them if run alone
save(['VERB4D_prep'], 'VERB4D_input_folder', 'VERB4D_output_folder', 'VERB4D_prep_folder', 'DiffCoeff_output_folder', 'start_date', 'total_time', 'end_date', 'Kp0', 'DxxKp', 'time');

%%
%system(['rm ', VERB4D_output_folder, '*']); % delete previous results
system(['del /Q ', strrep(VERB4D_output_folder,'/','\'), '*']); % delete previous results
if (true) % This needs to be recalculated only if grid changes
    %% Delete previous results
    %system(['rm ', VERB4D_input_folder, '*']);
    system(['del /Q ', strrep(VERB4D_input_folder,'/','\'), '*']); % delete previous results
    %% Load/create 'satellite' data, e.g. Kp, Dst, etc
    create_sat_data
    
    %% Create (P, L, V, K) grid, with size(P)==1
    create_grid
    
    %% Calculate L*, energy, pitch-angle, and Jacobians
    create_LEA_J_Dip
    
    %% Create initial PSD
    create_initial_PSD

    %% Create boundary conditions
    create_boundary_conditions

    %% Load/calculate diffusion coefficients
    create_diffusion_coefficients

    %% Create losses (loss cone)
    create_sources_and_losses
end


%% Create scaling and updates of coefficients
create_scaling

%% Parameters.ini
fid = fopen(['parameters.ini'], 'w');
fprintf(fid, ['time_total = ', num2str(total_time), '\n']); %
fprintf(fid, ['time_step = ', num2str(time_step), '\n']); %
fprintf(fid, ['time_output = ', num2str(output_time), '\n']); %
fprintf(fid, ['max_threads = 8 \n inversion_method = Lapack \n']); % Lapack with multithread can be only used for small grids
%fprintf(fid, ['max_threads = 8 \n inversion_method = ADI \n']); %
if (use_matlab)
    fprintf(fid, ['use_matlab = true \n']); % must be lower case "true" or "false"
else
    fprintf(fid, ['use_matlab = false \n']); % must be lower case "true" or "false"
end
fprintf(fid, ['include_boundary = true \n']); % must be lower case "true" or "false"
fprintf(fid, ['time_first = ', num2str(start_date), '\n']); %
fclose(fid);


%% RUN
'RUN VERB4D'
%tic; system('./VERB4D_Solver'); toc
        %tic; system('VERB-4D.exe'); toc
tic; system('./VERB4D_Solver'); toc
%system('./bgr_run.sh VERB4D_Solver')
%system('tail -n 50 screen.log')

%% Plot
plot_results 

