function [P_arr, R_arr, V_arr, K_arr, mu_arr] = calc_PRVK_grid2(P, R, V, K)
    %%
    % CALC_PRVK_GRID2 calculates (P,R,V,K) grid using grid paramters as size and limits
    %
    % [P_arr, R_arr, V_arr, K_arr, mu_arr] = calc_LVK_grid2(P, R, V, K)
    %
    % Input:
    %  P - magnetic local time, in rad, P=0 is noon!
    %  R - distance from the Earth, in Re
    %  V=(K + g_shift)^2 * mu - invariant (read paper)
    %  K - invariant, MeV/G
    %
    % Input parameters are the following structures. For P and R:
    % P.size - number of points
    % P.useLog - if to use log grid
    % P.min - min value
    % P.max - max value
    % (optional) P.val - a value to use if grid size == 1
    % same for R
    %
    % For K:
    % K.size - number of points
    % K.useLog - if to use Log grid
    % K.al_min - minimum pitch-angle value at L=K.L_val
    % K.al_max - maximum pitch-angle value at L=K.L_val
    % K.L_val - L-value to calculate K.min and K.max from K.al_min and K.al_max
    % (optional) K.val - a value to use if grid size == 1
    % 
    % For V:
    % V.size - number of points
    % V.useLog - if to use Log grid
    % V.en_min - minimum energy value at L=V.L_val
    % V.en_max - maximum energy value at L=V.L_val
    % V.L_val - L-value to calculate V.min and V.max from V.en_min and V.en_max
    % (optional) V.val - a value to use if grid size == 1
    %
    % Output:
    %  P_arr - MLT array in rad with P=0 at noon
    %  R_arr - radial distance array in Re
    %  V_arr - V new adiabatic invariant
    %  K_arr - K adiabatic invariant
    %  mu_arr - mu values on the gird

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plasma.Dip.Lalpha2K
    import plasma.pfunc
    import plasma.pc2mu
    

    % Create the new orthogonal grid in P, R, V, K)
    % P - local time
    % R - radial distance
    % V - invariant = M / (K + const) ^ 2, where M is invariant
    % K - invariant
    P_arr = zeros(P.size, R.size, V.size, K.size);
    R_arr = zeros(P.size, R.size, V.size, K.size);
    V_arr = zeros(P.size, R.size, V.size, K.size);
    K_arr = zeros(P.size, R.size, V.size, K.size);

    RE = 6.387e6; % meters

    
    % MLT grid, regular
    
    if (P.size > 1)
        P.min = 0;
        P.max = 360; % deg
        % the least point and the first point are the same by construction of the grid
        P_1d = [P.min:(P.max-P.min)/(P.size-1):P.max]*pi/180;
        %P_1d = [P.min:(P.max-P.min)/(P.size):P.max - (P.max-P.min)/(P.size)]*pi/180;    
    else 
        P_1d = P.val;
    end
    
    if (R.size > 1)
        R_1d = [R.min:(R.max-R.min)/(R.size-1):R.max];
    else 
        R_1d = R.val;
    end

    if (K.size > 1)
        % Grid in pitch-angle is created and converted to K grid
        % Log is more accurate for losses
        % The simulation is much more stable is there is no loss-cone, so
        % using NotLog can eliminate loss-cone if resolution is low and
        % this will increase stability
        try 
            K.useLog;
        catch
            K.useLog = false;
        end
        % create grid
        % create grid in pitch-angle and then convert to K
        % This works better for K:
        a1d_min = K.al_min;%0.7*pi/180;
        %a1d_min = alc(L_val);
        a1d_max = K.al_max; %89.3*pi/180;
        %a1d = [a1d_min:(a1d_max-a1d_min)/(K_size-1):a1d_max];
        L_val = K.L_val;
        
        if (false) %% dense grid for near loss-cone
            dense_pa = 5*pi/180;
            dense_gp = 2 + round(K.size * 0.1);
            dense_pa2 = a1d_max;%85*pi/180;
            dense_gp2 = 1;%2 + round(K.size * 0.1);
            rest_gp = K.size - dense_gp - dense_gp2 + 2;
            al_1d_dense = [a1d_min:(dense_pa-a1d_min)/(dense_gp-1):dense_pa];
            al_1d_dense2 = [dense_pa2:(a1d_max-dense_pa2)/(dense_gp2-1):a1d_max];
            al_1d_sparce = [dense_pa:(dense_pa2-dense_pa)/(rest_gp-1):dense_pa2];
            al_1d = [al_1d_dense(1:end-1), al_1d_sparce, al_1d_dense2(2:end)];
            K_1d = Lalpha2K(repmat(L_val, [1 K.size]), al_1d);
        elseif (false) %% dense grid for near loss-cone
            dense_pa = 5*pi/180;
            dense_gp = 2 + round(K.size * 0.1);
            rest_gp = K.size - dense_gp + 1;
            al_1d_dense = [a1d_min:(dense_pa-a1d_min)/(dense_gp-1):dense_pa];
            al_1d_sparce = [dense_pa:(a1d_max-dense_pa)/(rest_gp-1):a1d_max];
            al_1d = [al_1d_dense(1:end-1), al_1d_sparce];
            K_1d = Lalpha2K(repmat(L_val, [1 K.size]), al_1d);
        elseif (K.useLog)
            %create alpha grid and copy convert to K-grid
            al_1d = 10.^[log10(a1d_min):(log10(a1d_max)-log10(a1d_min))/(K.size-1):log10(a1d_max)];
            %al_1d = 10.^[log10(a1d_max):-(log10(a1d_max)-log10(a1d_min))/(K.size-1):log10(a1d_min)];
            K_1d = Lalpha2K(repmat(L_val, [1 K.size]), al_1d);
        else
            al_1d = [a1d_min:(a1d_max-a1d_min)/(K.size-1):a1d_max];
            %al_1d = [a1d_max:-(a1d_max-a1d_min)/(K.size-1):a1d_min];
            K_1d = Lalpha2K(repmat(L_val, [1 K.size]), al_1d);
        end
    else
        try
            K_1d = K.val;
        catch
            al_1d = K.al_val;
            K_1d = Lalpha2K(K.L_val, K.al_val);
        end
    end

    if (V.size > 1)

        % default - use log
        try 
            V.useLog;
        catch
            V.useLog = true;
        end
        
        g_shift = V.g_shift; %0.5; % <-- works good for L=5.5 with MT
        L_val = V.L_val; %6.6; % <-- this is an L-value, at which the grid will be from 0.01 to 10 MeV
        en_max = V.en_max;
        en_min = V.en_min;
        %V.max = (min(K_1d) + g_shift)^2 * pc2mu(L_val, pfunc(en_max), pi/2);
        %V.min = (min(K_1d) + g_shift)^2 * pc2mu(L_val, pfunc(en_min), pi/2);    
        V.max = (min(K_1d) + g_shift)^2 * pc2mu(L_val, pfunc(en_max), max(al_1d));
        V.min = (min(K_1d) + g_shift)^2 * pc2mu(L_val, pfunc(en_min), max(al_1d));    
        
        if (V.useLog)
            % create log grid
            V_1d = 10.^([log10(V.min):(log10(V.max)-log10(V.min))/(V.size-1):log10(V.max)]);
        else
            % create not-log grid
            V_1d = [V.min:(V.max-V.min)/(V.size-1):V.max];
        end
    else 
        % one point
        %V_1d = V.g_shift^2 * pc2mu(V.L_val, pfunc(V.en_max), pi/2);
        %V_1d =   (min(K_1d) + V.g_shift)^2 * pc2mu(V.L_val, pfunc(V.en_val), K.al_val);
        try
            V.mu_val;
        catch
            V.mu_val = pc2mu(V.L_val, pfunc(V.en_val), pi/2)
        end
        V_1d =   (min(K_1d) + V.g_shift)^2 * V.mu_val;
    end

    % create 4D grid from 1D grid
    %for iP = 1:P.size
    %    for iR = 1:R.size
    %        for iV = 1:V.size
    %            for iK = 1:K.size
    %                P_arr(iP, iR, iV, iK) = P_1d(iP);
    %                R_arr(iP, iR, iV, iK) = R_1d(iR);
    %                V_arr(iP, iR, iV, iK) = V_1d(iV);
    %                K_arr(iP, iR, iV, iK) = K_1d(iK);
    %            end
    %        end
    %    end
    %end
    
    P_arr = permute(repmat(P_1d', [1, R.size, V.size, K.size]), [1, 2, 3, 4]);
    R_arr = permute(repmat(R_1d', [1, P.size, V.size, K.size]), [2, 1, 3, 4]);
    V_arr = permute(repmat(V_1d', [1, P.size, R.size, K.size]), [2, 3, 1, 4]);
    K_arr = permute(repmat(K_1d', [1, P.size, R.size, V.size]), [2, 3, 4, 1]);
    
    mu_arr = V_arr ./ (K_arr + V.g_shift).^2;
    
end
