function [V_ExB] = calc_drift_V_ExB(P, R, Bf, Ef)
    %%
    % CALC_DRIFT_V_EXB calculates ExB drift velocities
    % V = ExB/B^2
    %
    % [V_ExB] = calc_drift_V_ExB(P, R, Bf, Ef)
    %
    % Input:
    %   P - MLT in rad, P==0 at noon
    %   R - radial distance in Re
    %%  Bf.z - magnetic field at bmin-surface, in Tesla
    %   Ef.p - azimuthal (tangential) electric field, in V/m
    %   Ef.r - radial electric field, in V/m
    %
    % Output:
    %   V_ExBf.p - ExB drift velocity component in 1/days
    %   V_ExBf.r - ExB drift velocity component in Re/days

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import read_write.load_4d
    import plasma.*

    % Check that B is in Tesla :(
    if (max(Bf.z(:)) - plasma.Dip.B_Dip(min(R.arr(:))) > 1e-3)
       error('B should be in Tesla, it seems like it is not in Tesla'); 
    end
    
    %% V_ExB drifts, energy and pitch-angle independent 
    % B is in Tesla!
    V_ExB.r = zeros(size(P.arr));
    V_ExB.p = zeros(size(P.arr));
    V_ExB.z = zeros(size(P.arr));

    for iP = 1:P.size
        for iR = 1:R.size
            % V = E x B / B^2 in m/s
            % B is in Tesla!
            % E is in V/m...
            
            %% V_tmp = cross([Ef.r(iP, iR), Ef.p(iP, iR), Ef.z(iP, iR)], [Bf.r(iP, iR), Bf.p(iP, iR), Bf.z(iP, iR)]) / dot([Bf.r(iP, iR), Bf.p(iP, iR), Bf.z(iP, iR)], [Bf.r(iP, iR), Bf.p(iP, iR), Bf.z(iP, iR)]);
            % V is in m/s
            V_tmp = cross([Ef.r(iP, iR), Ef.p(iP, iR), Ef.z(iP, iR)], [Bf.r(iP, iR), Bf.p(iP, iR), Bf.z(iP, iR)]) / dot([Bf.r(iP, iR), Bf.p(iP, iR), Bf.z(iP, iR)], [Bf.r(iP, iR), Bf.p(iP, iR), Bf.z(iP, iR)]);
                        
            V_ExB.r(iP, iR, :, :) = V_tmp(1);
            V_ExB.p(iP, iR, :, :) = V_tmp(2);
            V_ExB.z(iP, iR, :, :) = V_tmp(3);
        end
    end

    % Convert into 1/days and Re/days
    Re = 6.387e6; % meters
    V_ExB.p = V_ExB.p ./ R.arr * (60*60*24) / Re; % 1/days
    V_ExB.r = V_ExB.r * (60*60*24) / Re; % Re/days
    V_ExB.z = V_ExB.z * (60*60*24) / Re; % Re/days

end

