function [V_GC] = calc_drift_V_GC_Dip(P, R, pc, al, tau_b)
    %%
    % CALC_DRIFT_V_GC_DIP calculates grdient and curvature drifts for dipole field
    %
    % V_GC = calc_drift_V_GC_Dip(P, R, pc, al, tau_b)
    %
    % Input
    %   P - MLT in rad, P==0 at noon
    %   R - Radial distance in Re
    %   pc - p*c, momentum * speed of light
    %   al - pitch-angle in deg
    %   tau_b - bounce time (not dimentionless!!!), optional parameter
    %
    % Output:
    %   V_GC.r - radial velocity in Re/days
    %   V_GC.p - azimuthal velocity in 1/days

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plasma.Kfunc
    import plasma.pc2mu
    import plasma.K2Jc
    import plasma.Dip.Lalpha2K
    import plasma.Dip.Lmuy2Jc
    
    import plasma.Dip.B_Dip

    % Turn off DIPOLE warnings - everything is dipole here
    warning('off', 'VERB:DIPOLE')

    try
        % check if we have tau_b - bounce time - as a parameter
        tau_b;
    catch
        % calculate tau_b in case it wasn't a parameter
        Re = 6.387e6; % meters
        mc2 = 0.511;  % MeV
        c_si = 299792458;   % m/sec

        sin_al_c = sin(al.arr);
        T_alpha = (1.38 + 0.055 * pow(sin_al_c, 1.0/3) - 0.32 * pow(sin_al_c, 1.0/2) - 0.037 * pow(sin_al_c, 2.0/3) - 0.394 * sin_al_c + 0.056 * pow(sin_al_c, 4.0/3)); % Ksenya, dipole?; h(y) in Vanya's thesises
        Bounce_time_dimentionless.arr = R.arr .* (4.0 .* T_alpha);% ./ 60 ./ 60 ./ 24; % Bounce period, dipole!, sec. 
        
        use_relativistic = true;
        if (use_relativistic)
            en.arr = Kfunc(pc.arr);
            Velocity        = c_si .* sqrt(en.arr .* (en.arr + 2*mc2)) ./ (en.arr + mc2);
        else
            Velocity        = c_si .* pc.arr ./ mc2; % non-relativistic
        end

        tau_b.arr = Re ./ Velocity .* Bounce_time_dimentionless.arr;        
    end
    
    warning('off', 'VERB4D:DIPOLE')
    K.arr = plasma.Dip.Lalpha2K(R.arr, al.arr);
    mu.arr = plasma.pc2mu(R.arr, pc.arr, al.arr);
    warning('on', 'VERB4D:DIPOLE')
        
    V_GC.p = zeros(size(P.arr));
    V_GC.r = zeros(size(P.arr));
    V_GC.z = zeros(size(P.arr));
    
    %for i = 1:length(P.arr(:))
    for iP = 1:P.size
        for iR = 1:R.size

            % *_i index is just a shortcut for .arr(i)
            P_c = P.arr(iP,iR,1,1);
            R_c = R.arr(iP,iR,1,1);
            
            % constants
            mc2 = 0.511;  % MeV
            Re = 6.387e6; % meters
            c_si = 299792458;   % m/sec
            MeV2J = 1.60217646e-13;
            q = -1.6e-19; % C

            % calculate derivatives
            % P_p means +1 in P-direction
            % P_m means -1 in P-direction
            % R_p means +1 in R-direction
            % R_m means -1 in R-direction
            delta_P = pi/12;
            P_P_p = P_c + delta_P;
            P_P_m = P_c - delta_P;
            P_R_p = P_c;
            P_R_m = P_c;

            delta_R = 0.5;
            R_P_p = R_c;
            R_P_m = R_c;
            R_R_p = R_c + delta_R;
            R_R_m = R_c - delta_R;

            % B is in Tesla here, any magnetic field can be used here
            B_c = B_Dip(R_c) * 1e-4; % Tesla
            
            for iK = 1:size(P.arr, 4)
                K_c = K.arr(iP,iR,1,iK);
                al_c = unique(al.arr(iP,iR,:,iK)); % we can do this if al is the same on V-grid
                
                %sin_al_P_p = sin(al_c) / sqrt(B_c*1e4) * sqrt(B_Dip(R_P_p));
                %sin_al_P_m = sin(al_c) / sqrt(B_c*1e4) * sqrt(B_Dip(R_P_m));
                sin_al_R_p = sin(al_c) / sqrt(B_c*1e4) * sqrt(B_Dip(R_R_p));
                %sin_al_R_m = sin(al_c) / sqrt(B_c*1e4) * sqrt(B_Dip(R_R_m));

                %K_P_p = plasma.Dip.Y_sina(sin_al_P_p)./sin_al_P_p .* R_P_p .* sqrt(B_Dip(R_P_p)) ;
                %K_P_m = plasma.Dip.Y_sina(sin_al_P_m)./sin_al_P_m .* R_P_m .* sqrt(B_Dip(R_P_m)) ;
                K_R_p = plasma.Dip.Y_sina(sin_al_R_p)./sin_al_R_p .* R_R_p .* sqrt(B_Dip(R_R_p)) ;
                %K_R_m = plasma.Dip.Y_sina(sin_al_R_m)./sin_al_R_m .* R_R_m .* sqrt(B_Dip(R_R_m)) ;
                
                Grad_K.p = 0; %(K_P_p - K_c) ./ (P_P_p - P_c) ./ R_c ./ Re;          
                Grad_K.r = (K_R_p - K_c) ./ (R_R_p - R_c) ./ Re; % !!!          
                Grad_K.z = 0;

                for iV = 1:size(P.arr, 3)
                    
                    %pc_c = pc.arr(iP,iR,iV,iK);
                    tau_b_c = tau_b.arr(iP,iR,iV,iK);

                    mu_c = mu.arr(iP,iR,iV,iK); % can be modified for any magnetic field
                    %Jc_i = K2Jc(Lalpha2K(R_i, al_i), mu_i); % Valid for any magnetic field!

                    % From conservation of mu, find sin(alpha) at the P+-1 and R+-1 points
                    %sin_al_P_p = sqrt( 2.0 .* mu_c .* mc2 .* B_Dip(R_P_p) ) ./ pc_c;
                    %sin_al_P_m = sqrt( 2.0 .* mu_c .* mc2 .* B_Dip(R_P_m) ) ./ pc_c;
                    %sin_al_R_p = sqrt( 2.0 .* mu_c .* mc2 .* B_Dip(R_R_p) ) ./ pc_c;
                    %sin_al_R_m = sqrt( 2.0 .* mu_c .* mc2 .* B_Dip(R_R_m) ) ./ pc_c;
                    
                    % Find J*c and P+-1 and R+-1 points
                    %Jc_P_p= Lmuy2Jc(R_P_p, mu_i, sin_al_P_p);
                    %Jc_P_m = Lmuy2Jc(R_P_m, mu_i, sin_al_P_m);
                    %Jc_R_p = Lmuy2Jc(R_R_p, mu_i, sin_al_R_p);
                    %Jc_R_m = Lmuy2Jc(R_R_m, mu_i, sin_al_R_m);

                    % central differences
                    %Grad_Jc.p = (Jc_P_p - Jc_P_m) ./ (P_P_p - P_P_m) ./ R_center ./ Re;          
                    %Grad_Jc.r = (Jc_R_p - Jc_R_m) ./ (R_R_p - R_R_m) ./ Re; % !!!                      

                    % right-side differences
                    %Grad_Jc.p = (Jc_P_p - Jc_i) ./ (P_P_p - P_i) ./ R_i ./ Re;          
                    %Grad_Jc.r = (Jc_R_p - Jc_i) ./ (R_R_p - R_i) ./ Re; % !!!          

                    % left-side differences
                    %Grad_Jc.p = (Jc_center - Jc_P_m) ./ (P_center - P_P_m) ./ R_center ./ Re;          
                    %Grad_Jc.r = (Jc_center - Jc_R_m) ./ (R_center - R_R_m) ./ Re; % !!!

                    %Grad_Jc.z = 0;                    
                    
                    % Bounce time for real field should be used in real magnetic field
                    %sin_alpha_center = sin(al_center);
                    %T_alpha = (1.38 + 0.055 * pow(sin_alpha_center, 1.0/3) - 0.32 * pow(sin_alpha_center, 1.0/2) - 0.037 * pow(sin_alpha_center, 2.0/3) - 0.394 * sin_alpha_center + 0.056 * pow(sin_alpha_center, 4.0/3)); % Ksenya, dipole?; h(y) in Vanya's thesises
                    %tau_b = 4.0 .* (R_center .* Re) .* mc2 ./ pc_center ./ c_si .* T_alpha;% ./ 60 ./ 60 ./ 24; % Bounce period, dipole!, sec

                    % Eq 2.35, Roederer [1970]
                    %V_tmp = MeV2J ./ c_si ./ q ./ tau_b_c ./ (B_c) ./ (B_c) .* cross([Grad_Jc.r, Grad_Jc.p, Grad_Jc.z], [0, 0, B_c]);
                    V_tmp = sqrt(8.*mc2.*mu_c) .* MeV2J ./ c_si ./ q ./ tau_b_c ./ (B_c) ./ (B_c) .* cross([Grad_K.r, Grad_K.p, Grad_K.z], [0, 0, B_c]);

                    if (~isreal(V_tmp))
                        error('V_tmp is complex')
                    end

                    V_GC.p(iP,iR,iV,iK) = V_tmp(2) ./ R_c * (60*60*24); % 1/days   
                    V_GC.r(iP,iR,iV,iK) = V_tmp(1) * (60*60*24); % Re/Days
                end
            end
        end
    end
    
    warning('on', 'VERB:DIPOLE')

    %% Test-output
    MLT_test = 12;
    R_test = 4.5;
    en_test = 1;
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - MLT_test));
    [tmp, L_rot] = min(abs(R.arr(P_idx, :, 1, 1) - R_test));
    [tmp, En1] = min(abs(en.arr(P_idx, L_rot, :, 1) - en_test));
    %drift_time = (2.*pi.*R.arr(1, R_5, En1, end)) ./ V_GC.p(1, R_5, En1, end);
    drift_time = (2.*pi) ./ V_GC.p(P_idx, L_rot, En1, end) * 24 * 60;
    fprintf(1, 'Rotation time (sec) at L=%f of %f MeV electron: %f min\n', R.arr(1,L_rot,En1,1), en.arr(P_idx, L_rot, En1, end), drift_time)
    fprintf(1, 'B x grad(B) drift velocity at L=%f of %f MeV electron: %e rad/sec\n', R.arr(P_idx,L_rot,En1,1), en.arr(P_idx, L_rot, En1, end), V_GC.p(P_idx, L_rot, En1, end) / 24 / 60 / 60)
    
end
