function [V_GC] = calc_drift_V_GC_Tsy(P, R, K, en, al, Bf, Bounce_time_dimentionless, ... // All following  parameters are for K recalculation 
                            matlabd, Bmin_surface, Field_line_arr, kext, internal_field, maginput, varargin)
    %%
    % CALC_DRIFT_V_GC_TSY calculates GC drift velocities in Tsyganenco field for any pitch-angle
    %
    % calc_drift_V_GC_Tsy(P, R, K, en, al, Bf, Bounce_time_dimentionless,
    % ... // All following  parameters are for K recalculation in K.size=1
    %                        matlabd, Bmin_surface, Field_line_arr, kext, internal_field, maginput, varargin)
    %
    % Input:
    %   P - MLT in rad, P=0 at noon
    %   R - Radial distance in Re
    %   K - adiabatic invariant in G^{0.5} Re
    %   en - energy, MeV
    %   al - pitch-angle
    %%  Bf.z - magnetic field at bmin-surface, in Tesla
    %   Bounce_time_dimentionless - bounce time, dimentionless: Bounce_time_dimentionless = Bounce_time / ( Re ./ Velocity )
    %
    %  All following  parameters are for K recalculation:
    %   matlabd - current date
    %   Bmin_surface - surface of b-minimum
    %   Field_line_arr - array or traced magnetic field lines, each point has:
    %     Field_line_arr(1,1).R - radial distance
    %     Field_line_arr(1,1).lat - latitude
    %     Field_line_arr(1,1).phi - azumuthal angle
    %     Field_line_arr(1,1).B - magnetic field value in Tesla (I think)
    %   internal_field - internal magnetic field for Tsyganenko
    %   maginput - Onera parameter, has Kp and solar wind parameters, check Onera description
    %
    % Optional arguments, varargin:
    %    'use_relativistic', [true]|false - if to use relativistically correct velocities (usually true, unless for comparison)
    %    'use_dipole_approx', true|[false] - we can use dipole approximation to find K
    %    'show_warnings', true|[false] - show warnings, true/false
    %    
    % Output:
    %   V_GC.p - azimuthal gradient-curvature drift velocity component in 1/days
    %   V_GC.r - radial gradient-curvature drift vlocity component in Re/days

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import read_write.load_4d
    import plasma.Dip.Y
    import plasma.Tsy.calc_K_Onera_Tsy

    import plasma.pfunc
    import plasma.pc2mu
    
    % Check that B is in Tesla :(
    if (max(Bf.z(:)) > 1e-4)
       error('B should be in Tesla!'); 
    end
    
    % flag, if use relativistic equations
    use_relativistic = true;
    use_dipole_approx = false;
    show_warnings  = false;
        
    % Optional arguments
    it = 1;
    while it <= length(varargin)
        if strcmp(varargin(it), 'use_relativistic')
            use_relativistic = varargin{it+1};
            it = it + 1;
        elseif strcmp(varargin(it), 'use_dipole_approx')
            use_dipole_approx = varargin{it+1};
            it = it + 1;
        elseif strcmp(varargin(it), 'show_warnings')
            show_warnings = varargin{it+1};
            it = it + 1;
        else
            error(['Unknown optional argument: ', num2str(varargin{it})])
        end
        it = it + 1;
    end
    
    %options = [0, 0, 0, 0, internal_field];
    sysaxes_geo = 1;
 
    % X, Y coordinates inside the minb surface
    X.arr = zeros(size(P.arr(:,:,1,1)));
    Y.arr = zeros(size(P.arr(:,:,1,1)));
    [X.arr, Y.arr] = pol2cart(P.arr(:, :, 1, 1), R.arr(:, :, 1, 1));
    
    %% V_GC drifts, energy and pitch-angle dependent 

    V_GC.r = zeros(size(P.arr));
    V_GC.p = zeros(size(P.arr));
    V_GC.z = zeros(size(P.arr));
    
    pc.arr = pfunc(en.arr);
    mu.arr = pc2mu(R.arr, pc.arr, al.arr, repmat(Bf.z, [1 1 size(R.arr,3) size(R.arr,4)]).*1e4);
    V_size = size(en.arr, 3);
    for iP = 1:P.size
        for iR = 1:R.size

            % Magnetic field in current point
            B_c = Bf.z(iP,iR); % Tesla

            % Take derivative toward direction of lower magnetic field value
            % otherwise we can't find the same Bm on the next field line for ~90 electrons
            if (iP < P.size && Bf.z(iP+1,iR) < Bf.z(iP,iR))
                iP_dP_idx = iP + 1;
            elseif (iP == P.size && Bf.z(1+1,iR) < Bf.z(iP,iR))
                iP_dP_idx = 1 + 1; % P(P.size) == P(1), periodic boundary
            elseif (iP == 1)
                iP_dP_idx = P.size - 1; % P(P.size) == P(1), periodic boundary
            elseif (iP ~= 1)
                iP_dP_idx = iP - 1;
            end

            % Assume that magnetic field always decreases with distance
            if (iR == 1 || (iR < R.size && Bf.z(iP,iR+1) < Bf.z(iP,iR)))
                iR_dR_idx = iR + 1;
            else
                iR_dR_idx = iR - 1;
            end

            % R and P and the current point
            P_c = P.arr(iP,iR,1,1);
            R_c = R.arr(iP,iR,1,1);

            % for each pitch-angle...
            for iK = 1:K.size

                % alpha at the current point
                %warning('I think in general al changes along V-grid!!!!');
                al_c = unique(al.arr(iP,iR,:,iK)); % we can do this if al is the same on V-grid

                % sin(al) at the points we choose for derivatives
                sin_al_dP = sin(al_c) / sqrt(B_c) * sqrt(Bf.z(iP_dP_idx, iR));
                sin_al_dR = sin(al_c) / sqrt(B_c) * sqrt(Bf.z(iP, iR_dR_idx));

                % for sin, shound't be >1. If it is it's actually an error
                if (sin_al_dP > 1)
                    %warning(['fix sin_al_dP: ', num2str(iP), ',', num2str(iR)])
                    sin_al_dP = 1;
                end                             
                if (sin_al_dR > 1)
                    %if (iR ~= R.size)
                        %warning(['fix sin_al_dR: ', num2str(iP), ',', num2str(iR)])
                    %end
                    sin_al_dR = 1;
                end

                % Calculate grad J:
                % The goal is to calculate K or Jc at neighbor points for the same mu and Bm
                if (K.size == 1)
                    % if we have one point in pitch-angle,
                    % we need to calculate K at neighbour points tracing field line

                    if (~use_dipole_approx)
                        % K at the current point
                        K_c = K.arr(iP,iR,1,iK);

                        CoordVec_eq_geo_dP = [Bmin_surface.X_VERB_geo.arr(iP_dP_idx, iR), Bmin_surface.Y_VERB_geo.arr(iP_dP_idx, iR), Bmin_surface.Z_VERB_geo.arr(iP_dP_idx, iR)];
                        CoordVec_eq_geo_dR = [Bmin_surface.X_VERB_geo.arr(iP, iR_dR_idx), Bmin_surface.Y_VERB_geo.arr(iP, iR_dR_idx), Bmin_surface.Z_VERB_geo.arr(iP, iR_dR_idx)];

                        %Bmin should be in NanoTesla
                        [~, K_dP] = calc_K_Onera_Tsy(Field_line_arr(iP_dP_idx, iR), kext, internal_field, sysaxes_geo, matlabd, maginput, Bf.z(iP_dP_idx, iR), CoordVec_eq_geo_dP, asin(sin_al_dP)); 
                        [~, K_dR] = calc_K_Onera_Tsy(Field_line_arr(iP, iR_dR_idx), kext, internal_field, sysaxes_geo, matlabd, maginput, Bf.z(iP, iR_dR_idx), CoordVec_eq_geo_dR, asin(sin_al_dR)); 
                        
                    else                               
                        % we can use dipole field firmula for K

                        % K at the current point
                        K_c = plasma.Dip.Y_sina(sin(al_c))./sin_al_dP .* R.arr(iP, iR, 1, 1) .* sqrt(Bf.z(iP, iR)*1e4) ;
                        
                        %K_dP = Y(asin(sin_al_dP))./sin_al_dP .* R.arr(iP_dP_idx, iR, 1, 1) .* sqrt(Bf.z(iP_dP_idx, iR)*1e4) ;
                        %K_dR = Y(asin(sin_al_dR))./sin_al_dR .* R.arr(iP, iR_dR_idx, 1, 1) .* sqrt(Bf.z(iP, iR_dR_idx)*1e4) ;
                        K_dP = plasma.Dip.Y_sina(sin_al_dP)./sin_al_dP .* R.arr(iP_dP_idx, iR, 1, 1) .* sqrt(Bf.z(iP_dP_idx, iR)*1e4) ;
                        K_dR = plasma.Dip.Y_sina(sin_al_dR)./sin_al_dR .* R.arr(iP, iR_dR_idx, 1, 1) .* sqrt(Bf.z(iP, iR_dR_idx)*1e4) ;
                        
                    end  
                    
                else
                    % if we have all pitch-angles,
                    % we can find our K_dP by interpolation
                    interp_method = 'PCHIP';%'cubic';

                    % interpolate sin(al) instead of al, not sure why
                    sin_al_tmp = squeeze(sin(al.arr(iP_dP_idx, iR, 1, :)));
                    notnan = find(~isnan(sin_al_tmp));
                    if (length(notnan) > 1)
                        try
                            K_dP = interp1(sin_al_tmp(notnan), squeeze(K.arr(iP_dP_idx, iR, 1, notnan)), sin_al_dP, interp_method);
                        catch
                            K_dp = NaN;
                        end
                    else
                        K_dP = NaN;
                    end                                
                    if (show_warnings && isnan(K_dP))
                        warning(['K_dP is NaN: al=', num2str(asin(sin_al_dR))]);
                        %CoordVec_eq_geo_dP = onera_desp_lib_rotate([X_eq_gsm.arr(iP_dP_idx, iR), Y_eq_gsm.arr(iP_dP_idx, iR), Z_eq_gsm.arr(iP_dP_idx, iR)], 'gsm2geo', current_date);
                        %[tmp, K_dP] = calc_K_Onera(kext, options, sysaxes_geo, matlabd, maginput, Bf.z(iP_dP_idx, iR) * 1e9, CoordVec_eq_geo_dP, asin(sin_al_dP)); 
                    end

                    sin_al_tmp = squeeze(sin(al.arr(iP, iR_dR_idx, 1, :)));
                    notnan = find(~isnan(sin_al_tmp));
                    if (length(notnan) > 1)
                        try
                            K_dR = interp1(sin_al_tmp(notnan), squeeze(K.arr(iP, iR_dR_idx, 1, notnan)), sin_al_dR, interp_method);
                        catch
                            K_dR = NaN;
                        end
                    else
                        K_dR = NaN;
                    end
                    if (show_warnings && isnan(K_dR))
                        warning(['K_dR is NaN: al=', num2str(asin(sin_al_dP))])
                        %CoordVec_eq_geo_dR = onera_desp_lib_rotate([X_eq_gsm.arr(iP, iR_dR_idx), Y_eq_gsm.arr(iP, iR_dR_idx), Z_eq_gsm.arr(iP, iR_dR_idx)], 'gsm2geo', current_date);
                        %[tmp, K_dR] = calc_K_Onera(kext, options, sysaxes_geo, matlabd, maginput, Bf.z(iP, iR_dR_idx) * 1e9, CoordVec_eq_geo_dR, asin(sin_al_dR)); 
                    end

                    %if (K_dP == NaN || K_dR == NaN)
                    %    error('K_d* error');
                    %end
                end

                % P and R at the neighboar points for derivative calculation
                P_dP = P.arr(iP_dP_idx, iR, 1, iK);
                R_dR = R.arr(iP, iR_dR_idx, 1, iK);

                K_c = K.arr(iP,iR,1,iK);

                % for each energy...
                for iV = 1:V_size

                    % mu, pc, and en at the current point
                    mu_c = mu.arr(iP,iR,iV,iK);
                    en_c = en.arr(iP,iR,iV,iK);
                    pc_c = pc.arr(iP,iR,iV,iK);

                    % constants
                    B0 = 0.312e-4; % Tesla
                    q = -1.6e-19; % C 
                    mc2 = 0.511;  % MeV
                    MeV2J = 1.60217646e-13; % Conversion from MeV to Joules
                    c_si = 299792458;   % m/sec
                    Re = 6.387e6; % meters

                    %% For better accuracy
                    c_si = 1; %% !! Because we multiply and divide by it ; 299792458;   % m/sec
                    %%
                    
                    % Grad(K)
                    Grad_K.p = (K_dP - K_c) ./ (P_dP - P_c) ./ R_c ./ Re;          
                    Grad_K.r = (K_dR - K_c) ./ (R_dR - R_c) ./ Re; % !!! 
                    Grad_K.z = 0;                    
                    
                    if (use_relativistic)
                        Velocity        = c_si .* sqrt(en_c * (en_c + 2*mc2)) ./ (en_c + mc2);
                        %Velocity        = pc_c ./ en_c .* c_si; ????????????
                    else
                        Velocity        = c_si .* pc_c ./ mc2; % non-relativistic
                    end

                    Bounce_time = Re ./ Velocity * Bounce_time_dimentionless.arr(iP,iR,iV,iK);

                    % B is in Tesla here, any magnetic field can be used here
                    % Eq 2.35, Roederer [1970]
                    %% V_tmp = (sqrt(8.*mc2.*mu_c) .* MeV2J ./ c_si ./ q ./ B_c ./ B_c ./ Bounce_time ) .* cross([Grad_K.r, Grad_K.p, Grad_K.z], [0, 0, B_c]);
                    %% V_GC.r(iP, iR, iV, iK) = V_tmp(1);
                    %% V_GC.p(iP, iR, iV, iK) = V_tmp(2);
                    %% V_GC.z(iP, iR, iV, iK) = V_tmp(3);

                    % For better accuracy / speed:
                    
                    % r  p  z
                    % Gr Gp Gz
                    % 0  0  Bz
                    %V_GC.r(iP, iR, iV, iK) = (sqrt(8.*mc2.*mu_c) .* MeV2J ./ c_si ./ q ./ B_c ./ B_c ./ Bounce_time ) .*    Grad_K.p .* B_c;
                    %V_GC.p(iP, iR, iV, iK) = (sqrt(8.*mc2.*mu_c) .* MeV2J ./ c_si ./ q ./ B_c ./ B_c ./ Bounce_time ) .*  (-Grad_K.r .* B_c);
                    %V_GC.z(iP, iR, iV, iK) = 0;

                    V_GC.r(iP, iR, iV, iK) = (sqrt(8.*mc2.*mu_c) .* MeV2J ./ c_si ./ q ./ B_c ./ 1 ./ Bounce_time ) .*    Grad_K.p .* 1;
                    V_GC.p(iP, iR, iV, iK) = (sqrt(8.*mc2.*mu_c) .* MeV2J ./ c_si ./ q ./ B_c ./ 1 ./ Bounce_time ) .*  (-Grad_K.r .* 1);
                    V_GC.z(iP, iR, iV, iK) = 0;
                    
                end
            end
        end

    end

    if (false)
        %% test-plot
        [X_VERB.arr, Y_VERB.arr] = pol2cart(P.arr(:, :, 1, 1), R.arr(:, :, 1, 1));

        al_idx = 15;%K.size;

        figure
        pcolor(X_VERB.arr, Y_VERB.arr, log10(squeeze(Bounce_time_dimentionless.arr(:,:,1,al_idx))));
        %caxis([0 90])
        colorbar 
        set(gca, 'xDir', 'reverse', 'yDir', 'reverse')
        title(['Pitch-angle, deg. K = ', num2str(unique(K.arr(:,:,1,al_idx)))]);
    end
    
    % Convert into Re/days and 1/days
    V_GC.p = V_GC.p ./ R.arr * (60*60*24); % 1/day   
    V_GC.r = V_GC.r * (60*60*24); % Re/day
        
    
    %% Test-output
    MLT_test = 12;
    R_test = 4.5;
    en_test = 1;
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - MLT_test));
    [tmp, L_rot] = min(abs(R.arr(P_idx, :, 1, 1) - R_test));
    [tmp, En1] = min(abs(en.arr(P_idx, L_rot, :, 1) - en_test));
    %drift_time = (2.*pi.*R.arr(1, R_5, En1, end)) ./ V_GC.p(1, R_5, En1, end);
    drift_time = (2.*pi) ./ V_GC.p(P_idx, L_rot, En1, end) * 24 * 60;
    fprintf(1, 'Rotation time (sec) at L=%f of %f MeV electron: %f min\n', R.arr(1,L_rot,En1,1), en.arr(P_idx, L_rot, En1, end), drift_time)
    fprintf(1, 'B x grad(B) drift velocity at L=%f of %f MeV electron: %e rad/sec\n', R.arr(P_idx,L_rot,En1,1), en.arr(P_idx, L_rot, En1, end), V_GC.p(P_idx, L_rot, En1, end) / 24 / 60 / 60)


end
