function [V_GC] = calc_drift_V_GC_Tsy_Eq(P, R, K, en, al, Bf, varargin)
    %%
    % CALC_DRIFT_V_GC_TSY_EQ calculates GC drift velocities in Tsyganenko field for equatorially mirroring particles
    %
    % V_GC = calc_drift_V_GC_Tsy_Eq(P, R, K, en, al, Bf, varargin)
    %
    % Input:
    %   P - MLT in rad, P=0 at noon
    %   R - Radial distance in Re
    %   K - adiabatic invariant in G^{0.5} Re
    %   en - energy, MeV
    %   al - pitch-angle
    %%  Bf.z - magnetic field at bmin-surface, in Tesla
    %   Optional parameters (varargin _pairs_):
    %    'use_relativistic', [true]|false - (optional) if to use relativistically correct velocities
    %    'check_90deg', [true]|false - check if all pitch-angles=90deg
    %
    % Output:
    %   V_GC.p - azimuthal gradient-curvature drift velocity component, in 1/days
    %   V_GC.r - radial gradient-curvature drift velocity component, in Re/days

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import read_write.load_4d
    import plasma.Dip.Y
    import plasma.Tsy.calc_K_Onera_Tsy
    import plasma.pfunc

    % Check that B is in Tesla :(
    if (max(Bf.z(:)) > 1e-4)
       error('B should be in Tesla!'); 
    end
    
    use_relativistic = true;
    check_90eg = true;

    % Optional arguments
    it = 1;
    while it <= length(varargin)
        if strcmp(varargin(it), 'use_relativistic')
            use_relativistic = varargin{it+1};
            it = it + 1;
        elseif strcmp(varargin(it), 'check_90eg')
            check_90eg = varargin{it+1};
            it = it + 1;
        end
        it = it + 1;
    end


    % constants
    B0 = 0.312e-4; % Tesla
    q = -1.6e-19; % C !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    Re = 6.387e6; % meters
    mc2 = 0.511;  % MeV
    c_si = 299792458;   % m/sec

    % X, Y coordinates inside the minb surface
    X.arr = zeros(size(P.arr(:,:,1,1)));
    Y.arr = zeros(size(P.arr(:,:,1,1)));
    [X.arr, Y.arr] = pol2cart(P.arr(:, :, 1, 1), R.arr(:, :, 1, 1));
    
    %% V_GC drifts, energy and pitch-angle dependent 

    V_GC.r = zeros(size(P.arr));
    V_GC.p = zeros(size(P.arr));
    V_GC.z = zeros(size(P.arr));
    
    pc.arr = pfunc(en.arr);
    
    % We can use different, much more accurate, equation for 90deg
    % for equatorial particles we can use a simpler formula
    % Roeder, 1.27, not bounce-averaged

    if (check_90eg)
        if (max(K.arr(:)) > 1e-7)
            error('VERB error: K != 0')
        end
    end
    
    Grad_Bf.r = zeros(size(Bf.z));
    Grad_Bf.p = zeros(size(Bf.z));
    Grad_Bf.z = zeros(size(Bf.z));

    % central differences
    % Remember that Bf is in Tesla
    Grad_Bf.p(2:end-1, :) = (Bf.z(3:end, :) - Bf.z(1:end-2, :)) ./ (P.arr(3:end, :, 1, 1) - P.arr(1:end-2, :, 1, 1)) ./ R.arr(3:end, :, 1, 1) ./ Re; % !!!;
    Grad_Bf.p(1, :) = (Bf.z(2, :) - Bf.z(end-2, :)) ./ (P.arr(2, :, 1, 1) - P.arr(end-2, :, 1, 1)) ./ R.arr(2, :, 1, 1) ./ Re; % !!!; % cyclic boundary condition
    Grad_Bf.p(end, :) = (Bf.z(3, :) - Bf.z(end-1, :)) ./ (P.arr(3, :, 1, 1) - P.arr(end-1, :, 1, 1)) ./ R.arr(3, :, 1, 1) ./ Re; % !!!;

    Grad_Bf.r(:, 2:end-1) = (Bf.z(:, 3:end) - Bf.z(:, 1:end-2)) ./ (R.arr(:, 3:end, 1, 1) - R.arr(:, 1:end-2, 1, 1)) / Re;
    Grad_Bf.r(:, 1)       = (Bf.z(:, 2)     - Bf.z(:, 1))       ./ (R.arr(:, 2, 1, 1)     - R.arr(:, 1, 1, 1))       / Re;
    Grad_Bf.r(:, end)     = (Bf.z(:, end)   - Bf.z(:, end-1))   ./ (R.arr(:, end, 1, 1)   - R.arr(:, end-1, 1, 1))   / Re;

    for iP = 1:P.size
        for iR = 1:R.size
            for iV = 1:size(en.arr, 3)
                for iK = 1:K.size

                    if (use_relativistic)
                        %mv2        = mc2 .* (en.arr(iP, iR, iV, iK) .* (en.arr(iP, iR, iV, iK) + 2*mc2)) ./ (en.arr(iP, iR, iV, iK) + mc2)^2; % relativistic?
                        %error('!!!!relativistic eq for 90 deg is wrong!!!!')

                        % g m v = E/c^2 ??
                        mv2 = (pc.arr(iP, iR, iV, iK)).^2 ./ (mc2 + en.arr(iP, iR, iV, iK));

                    else
                        mv2        = pc.arr(iP, iR, iV, iK).^2 ./ mc2; %% NON-RELATIVISTIC
                    end

                    % TODO: Check, if it;s relativistic
                    MeV2J = 1.60217646e-13;
                    V_tmp = mv2*MeV2J ./ 2 ./ q ./ Bf.z(iP, iR).^3 .* (1 + cos(al.arr(iP, iR, iV, iK)).^2) .* cross([Bf.r(iP, iR), Bf.p(iP, iR), Bf.z(iP, iR)], [Grad_Bf.r(iP, iR), Grad_Bf.p(iP, iR), Grad_Bf.z(iP, iR)]);

                    V_tmp = V_tmp ./ Re; % Re/s

                    V_GC.r(iP, iR, iV, 1) = V_tmp(1);
                    V_GC.p(iP, iR, iV, 1) = V_tmp(2);
                    V_GC.z(iP, iR, iV, 1) = V_tmp(3);                    
                end
            end
        end
    end

    % Convert into Re/days and 1/days
    V_GC.p = V_GC.p ./ R.arr * (60*60*24); % 1/day       
    V_GC.r = V_GC.r * (60*60*24); % Re/day
    
    %% Test-output
    MLT_test = 12;
    R_test = 4.5;
    en_test = 1;
    [tmp, P_idx] = min(abs(mod(P.arr(:, 1, 1, 1)*12/pi-12,24) - MLT_test));
    [tmp, L_rot] = min(abs(R.arr(P_idx, :, 1, 1) - R_test));
    [tmp, En1] = min(abs(en.arr(P_idx, L_rot, :, 1) - en_test));
    %drift_time = (2.*pi.*R.arr(1, R_5, En1, end)) ./ V_GC.p(1, R_5, En1, end);
    drift_time = (2.*pi) ./ V_GC.p(P_idx, L_rot, En1, end) * 24 * 60;
    fprintf(1, 'Rotation time (sec) at L=%f of %f MeV electron: %f min\n', R.arr(1,L_rot,En1,1), en.arr(P_idx, L_rot, En1, end), drift_time)
    fprintf(1, 'B x grad(B) drift velocity at L=%f of %f MeV electron: %e rad/sec\n', R.arr(P_idx,L_rot,En1,1), en.arr(P_idx, L_rot, En1, end), V_GC.p(P_idx, L_rot, En1, end) / 24 / 60 / 60)


end
