function [new_arr] = fast_interpolate(L_arr, epc_arr, alpha_arr, f_arr, new_L, new_epc, new_alpha, varargin)
    %%
    % FAST_INTERPOLATE fast interpolation for PSD on L, energy, and pitch-angle grid
    %
    % new_arr = fast_interpolate(L_arr, epc_arr, alpha_arr, f_arr, new_L, new_epc, new_alpha, varargin)
    %
    % Input:
    %   L_arr - Lstar
    %   epc_arr - energy array
    %   alpha_arr - pitch-angle array, should be independent of energy, e.g. alpha_arr(1,:) == alpha_arr(2,:) == alpha_arr(*,:)
    %   f_arr - our function, 2D array I think
    %   new_L - new L, SHOULD BE A SUBSET OF L_arr!
    %   new_epc - new energy
    %   new_alpha - new pitch-angle, I think it also should be indeendent of energy
    %
    % Optional arguments, varargin:
    %   'log' - interpolate as log10(f_arr)
    %  
    % Output:
    %   new_arr - well, the interpolated function

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    loginterp = false;    
    if (size(varargin) > 0) 
        if (strcmp(varargin(1), 'log'))
            loginterp = true;
%            for it = 1:size(f_arr, 1)
%                f_arr(it, :, :, :) = log10(f_arr(it, :, :, :));
%            end
        end
    end
    
    size1 = size(L_arr, 1);
    size2 = size(L_arr, 2);
    size3 = size(L_arr, 3);

    new_arr = zeros(size(f_arr, 1), size(f_arr, 2));
    
    for it = 1:size(f_arr, 1)
        %['interpolation step = ', num2str(it)]

        if (loginterp) 
           new_arr(it, 1) = -22; %Trick - PSD=0 for L=1
        else
           new_arr(it, 1) = 1e-22; %Trick - PSD=0 for L=1
        end
        for L_ind = 2:size(f_arr, 2)

            if (loginterp) 
                f_arr_current = reshape(squeeze(log10(f_arr(it, L_ind, :, :))), [size2 size3]);
            else
                f_arr_current = reshape(squeeze(f_arr(it, L_ind, :, :)), [size2 size3]);;
            end
            
            %try 
                
            % using the fact, that alpha is undependent of epc!
            alpha_l_ind = 1;
            while (alpha_l_ind < size3 && alpha_arr(L_ind,1,alpha_l_ind+1)<new_alpha(L_ind))
                alpha_l_ind = alpha_l_ind + 1;
            end   
            alpha_r_ind = size3;
            while (alpha_r_ind > 1 && alpha_arr(L_ind,1,alpha_r_ind-1)>new_alpha(L_ind))
                alpha_r_ind = alpha_r_ind - 1;
            end   

            %alpha_l_ind = max(find(alpha_arr(L_ind,1,:)<new_alpha(L_ind)));
            %alpha_r_ind = min(find(alpha_arr(L_ind,1,:)>new_alpha(L_ind)));
            
            % finding bottom and top indexes of epc for each alpha
            epc_lb_ind = 1;
            while (epc_lb_ind < size2 && epc_arr(L_ind,epc_lb_ind+1,alpha_l_ind)<new_epc(L_ind))
                epc_lb_ind = epc_lb_ind + 1;
            end   
            epc_rb_ind = 1;
            while (epc_rb_ind < size2 && epc_arr(L_ind,epc_rb_ind+1,alpha_r_ind)<new_epc(L_ind))
                epc_rb_ind = epc_rb_ind + 1;
            end   
            epc_lt_ind = size2;
            while (epc_lt_ind > 1 && epc_arr(L_ind,epc_lt_ind-1,alpha_l_ind)>new_epc(L_ind))
                epc_lt_ind = epc_lt_ind - 1;
            end   
            epc_rt_ind = size2;
            while (epc_rt_ind > 1 && epc_arr(L_ind,epc_rt_ind-1,alpha_r_ind)>new_epc(L_ind))
                epc_rt_ind = epc_rt_ind - 1;
            end               

            %epc_lb_ind = max(find(epc_arr(L_ind,:,alpha_l_ind)<new_epc(L_ind)));
            %epc_lt_ind = min(find(epc_arr(L_ind,:,alpha_l_ind)>new_epc(L_ind)));
            %epc_rb_ind = max(find(epc_arr(L_ind,:,alpha_r_ind)<new_epc(L_ind)));
            %epc_rt_ind = min(find(epc_arr(L_ind,:,alpha_r_ind)>new_epc(L_ind)));
            
            %['L=', num2str(L_arr(L_ind,1,1)), '; al=', num2str(alpha_l_ind), '; ar=', num2str(alpha_r_ind),'; epc_lb=', num2str(epc_lb_ind), '; epc_lt=', num2str(epc_lt_ind), '; epc_rb=', num2str(epc_rb_ind), '; epc_rt=', num2str(epc_rt_ind)]
            
            % interpolating
%				double a = (x - m_x[i - 1]) / (m_x[i] - m_x[i - 1]);
%				return m_y[i - 1] + a * (m_y[i] - m_y[i - 1]);
            % left side
            if (epc_lb_ind == epc_lt_ind) 
                new_arr_l = f_arr_current(epc_lb_ind, alpha_l_ind);
            else
                fb = f_arr_current(epc_lb_ind, alpha_l_ind);
                ft = f_arr_current(epc_lt_ind, alpha_l_ind);
                epcb = epc_arr(L_ind, epc_lb_ind, alpha_l_ind);
                epct = epc_arr(L_ind, epc_lt_ind, alpha_l_ind);

                a = ( new_epc(L_ind) - epcb ) / ( epct - epcb );
                new_arr_l = fb + a * ( ft - fb );
            end
            
            % right side
            if (epc_rb_ind == epc_rt_ind) 
                new_arr_r = f_arr_current(epc_rb_ind, alpha_r_ind);
            else
                fb = f_arr_current(epc_rb_ind, alpha_r_ind);
                ft = f_arr_current(epc_rt_ind, alpha_r_ind);
                epcb = epc_arr(L_ind, epc_rb_ind, alpha_r_ind);
                epct = epc_arr(L_ind, epc_rt_ind, alpha_r_ind);
            
                a = ( new_epc(L_ind) - epcb ) / ( epct - epcb );
                new_arr_r = fb + a * ( ft - fb );
            end
            
            % between sides
            if (alpha_l_ind == alpha_r_ind)
                new_arr(it, L_ind) = new_arr_l;
            else
                alphab = alpha_arr(L_ind, epc_lb_ind, alpha_l_ind);
                alphat = alpha_arr(L_ind, epc_rb_ind, alpha_r_ind);

                a = ( new_alpha(L_ind) - alphab ) / ( alphat - alphab );
                new_arr(it, L_ind) = new_arr_l + a * ( new_arr_r - new_arr_l );
            end
            
            % temp!!!
            %new_arr(it, L_ind) = f_arr_current(epc_rt_ind, alpha_r_ind);
            %catch
            %    err = lasterror;
            %    [err.message, '; L_ind=', num2str(L_ind)]
            %    if (loginterp) 
            %       new_arr(it, L_ind) = -22; %Trick - PSD=0 for L=1
            %    else
            %       new_arr(it, L_ind) = 1e-22; %Trick - PSD=0 for L=1
            %    end
            %    error(err)
            %end
        end            
        
    end

    if (loginterp) 
        new_arr = 10.^new_arr;
    end
    
end