function result_value = fast_interpolate_2D2(Q1_arr, Q2_arr, f_arr, new_Q1_value, new_Q2_value)
    %%
    % FAST_INTERPOLATE_2D2 fast interpolation - assumes that Q2 coordinate is independent of Q1 coordinate (Q1 can still be dependent of Q2)
    %
    % result_value = fast_interpolate_2D2(Q1_arr, Q2_arr, f_arr, new_Q1_value, new_Q2_value)
    %
    % Input:
    %   Q1_arr - Q1 array, 2D I think
    %   Q2_arr - Q2 array, 2D I think, should be independent of Q1, e.g. Q2_arr(1,:) == Q2_arr(2,:) == Q2_arr(*,:)
    %   f_arr - our function, 2D array I think
    %   new_Q1_value - new Q1 array
    %   new_Q2_value - new Q2 array, I think it also should be indeendent of Q2
    %  
    % Output:
    %   result_value - well, the interpolated function

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits    
    
    %save debug_input Q1_arr Q2_arr f_arr new_Q1_value new_Q2_value
    %load debug_input

    Q1_size = size(f_arr, 1);
    Q2_size = size(f_arr, 2);

    if (Q1_size == 1 && Q2_size == 1)
        %'Warning: size 1x1, no interpolation'
        result_value = f_arr;
        return;
    elseif (   ((new_Q1_value < min(Q1_arr(:)) || new_Q1_value > max(Q1_arr(:))) && Q1_size > 1) ... 
        || ((new_Q2_value < min(Q2_arr(:)) || new_Q2_value > max(Q2_arr(:))) && Q2_size > 1))
        % no extrapolation (it's for good)
        result_value = NaN;
        return;
    end
    
    % we search for 4 points around the (new_Q1_value, new_Q2_value) point
    % let's name these points as _1l_2l, _1l_2r, _1r_2l, _1r_2r
    % alpha_1l_2l = alpha_1r_2l, and alpha_1l_2r = alpha_1r_2r by grid
    % constructon, so let's use alpha_2l and alpha_2r

    Q2_ind_2l = 1;
    Q1_ind_1l_2l = 1;
    
    % start from previous indexes and ajust them left or right to catch
    % (new_Q1_value, new_Q2_value) inside (*_ind_1l_2l, *_ind_1r_2l, *_ind_1l_2r, *_ind_1r_2r)
    if (Q2_arr(Q1_ind_1l_2l, 1) < Q2_arr(Q1_ind_1l_2l, end)) 
        while (Q2_ind_2l < Q2_size && Q2_arr(Q1_ind_1l_2l, Q2_ind_2l + 1) < new_Q2_value)
            Q2_ind_2l = Q2_ind_2l + 1;
        end
        while (Q2_ind_2l > 2 && Q2_arr(Q1_ind_1l_2l, Q2_ind_2l - 1) > new_Q2_value)
            Q2_ind_2l = Q2_ind_2l - 1;
        end
        Q2_ind_2r = min(Q2_size, Q2_ind_2l + 1);
    else
        %error('Grid should be increasing')
        % in case grid is decreasing, and not increasing
        while (Q2_ind_2l < Q2_size && Q2_arr(Q1_ind_1l_2l, Q2_ind_2l + 1) > new_Q2_value)
            Q2_ind_2l = Q2_ind_2l + 1;
        end
        while (Q2_ind_2l > 2 && Q2_arr(Q1_ind_1l_2l, Q2_ind_2l - 1) < new_Q2_value)
            Q2_ind_2l = Q2_ind_2l - 1;
        end
        Q2_ind_2r = min(Q2_size, Q2_ind_2l + 1);
    end
    
    while (Q1_ind_1l_2l < Q1_size && Q1_arr(Q1_ind_1l_2l + 1, Q2_ind_2l) < new_Q1_value)
        Q1_ind_1l_2l = Q1_ind_1l_2l + 1;
    end   
    Q1_ind_1r_2l = min(Q1_size, Q1_ind_1l_2l + 1);
    
    Q1_ind_1l_2r = Q1_ind_1l_2l; % start from Q1_ind_1l_2l and ajust it left or right if is not near the new_Q1_value
    while (Q1_ind_1l_2r < Q1_size && Q1_arr(Q1_ind_1l_2r + 1, Q2_ind_2r) < new_Q1_value)
        Q1_ind_1l_2r = Q1_ind_1l_2r + 1;
    end
    while (Q1_ind_1l_2r > 2 && Q1_arr(Q1_ind_1l_2r - 1, Q2_ind_2r) > new_Q1_value)
        Q1_ind_1l_2r = Q1_ind_1l_2r - 1;
    end
    Q1_ind_1r_2r = min(Q1_size, Q1_ind_1l_2r + 1);
    
    % interpolating

    % alpha left side
    if (Q1_ind_1r_2l == Q1_ind_1l_2l) % if there is only one point in that direction - plot that point
        result_value_2l = f_arr(Q1_ind_1l_2l, Q2_ind_2l);
    else
        f_1l_2l = f_arr(Q1_ind_1l_2l, Q2_ind_2l);
        f_1r_2l = f_arr(Q1_ind_1r_2l, Q2_ind_2l);
        Q1_1l_2l = Q1_arr(Q1_ind_1l_2l, Q2_ind_2l);
        Q1_1r_2l = Q1_arr(Q1_ind_1r_2l, Q2_ind_2l);
        
        a = ( new_Q1_value - Q1_1l_2l ) / ( Q1_1r_2l - Q1_1l_2l );
        result_value_2l = f_1l_2l + a * ( f_1r_2l - f_1l_2l );
    end

    % alpha right side
    if (Q1_ind_1r_2r == Q1_ind_1l_2r) 
        result_value_2r = f_arr(Q1_ind_1l_2r, Q2_ind_2r);
    else
        f_1l_2r = f_arr(Q1_ind_1l_2r, Q2_ind_2r);
        f_1r_2r = f_arr(Q1_ind_1r_2r, Q2_ind_2r);
        Q1_1l_2r = Q1_arr(Q1_ind_1l_2r, Q2_ind_2r);
        Q1_1r_2r = Q1_arr(Q1_ind_1r_2r, Q2_ind_2r);

        a = ( new_Q1_value - Q1_1l_2r ) / ( Q1_1r_2r - Q1_1l_2r );
        result_value_2r = f_1l_2r + a * ( f_1r_2r - f_1l_2r );
    end

    % between alpha sides
    if (Q2_ind_2l == Q2_ind_2r)
        result_value = result_value_2l;
    else
        Q2_2l = Q2_arr(Q1_ind_1l_2l, Q2_ind_2l);
        Q2_2r = Q2_arr(Q1_ind_1l_2r, Q2_ind_2r);

        a = ( new_Q2_value - Q2_2l ) / ( Q2_2r - Q2_2l );
        result_value = result_value_2l + a * ( result_value_2r - result_value_2l );
    end
    
end
