function PSD0_arr = initial_PSD_3D(L_arr, pc_arr, al_arr, Kp0, tau)
    %%
    % INITIAL_PSD_3D initial PSD calciltion as a steady-state slution of radial diffusion equation
    %
    % PSD0_arr = initial_PSD_3D(L_arr, pc_arr, al_arr, Kp0, tau)
    %
    % Input:
    %   Arrays are 3D : [number of points in L] x [number of points in energy] x [number of points in pitch-angle]
    %
    %   L_arr - L* array
    %   pc_arr - pc array
    %   al_arr - pitch-angle array
    %   Kp0 - Kp index
    %   tau - electron lifetime
    %
    % Output:
    %   PSD0 in 1/3e7*#/(cm MeV/c)^3 units!!

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits

    import VERB4D_scripts.steady_state
    import sat_data.J_L7_corrected
    import plasma.Dip.B_Dip
    import plasma.Kfunc
    import plasma.Dip.alc_Dip    
    
    L_size = size(L_arr, 1);
    pc_size = size(L_arr, 2);
    al_size = size(L_arr, 3);
    PSD0_arr = zeros(L_size, pc_size, al_size);

    % Initial condition, PSD0. 
    %Radial diffusion steady state
    L1d = squeeze(L_arr(:,1,1));
    
    first_idx = 1;
    last_idx = L_size;
    
    % Add L=7 point if it's not there
    if min(L1d) > 1 L1d = [1, L1d']'; 
        first_idx = first_idx + 1;
        last_idx = last_idx + 1;
    end;
    if max(L1d) < 7 L1d = [L1d', 7]'; 
    end;
    
    %tau = 1e99;
    %tau = 4./Kp0;
    %tau = 3/2/Kp0;
    D1d = pow(10, 0.506.*Kp0 - 9.325) .* pow(L1d, 10);

    PSD_SS = steady_state(L1d, D1d, tau);
    
    % PSD should be 1 at L=7. This is important if max(L) > 7
    [tmp, L7_idx] = min(abs(L1d - 7));
    PSD_SS = PSD_SS / PSD_SS(L7_idx);
        
    if (true)
        % PSD at L=7
        pc_7 = pc_arr(end, :, :) .* sqrt(B_Dip(7) ./ B_Dip(L_arr(end, :, :)));
        en_7 = Kfunc(pc_7);
        PSD_7 = J_L7_corrected(en_7) ./ pc_7.^2;

        % PSD at L=L-max
        PSD_L_last = PSD_7 ./ PSD_SS(L7_idx) .* PSD_SS(last_idx) .* sin(al_arr(L_size, :, :)); % the PA distribution depends on where the maximum L* is

        for ie = 1:pc_size
            for ia = 1:al_size
                PSD0_arr(:, ie, ia) = PSD_L_last(1, ie, ia) .* PSD_SS(first_idx:last_idx);% ./ PSD_SS(last_idx);
            end
        end
    end

    % commented
    if (false)    
        for iL = 1:Q1_size
            for ie = 1:Q2_size
                for ia = 1:Q3_size

                    % energy at L=7, by conservation of mu
                    % This is not perfectly correct, we need to recalculate pc_7 for each L
                    % But I think this is how we did it in VERB 2.0
                    % The difference is barely noticable anyway
                    pc_7 = pc_Q.arr(end, ie, ia) .* sqrt(B0(7) ./ B0(L_Q.arr(end, ie, ia)));

                    %this is more correct
                    %pc_7 = pc_Q.arr(iL, ie, ia) .* sqrt(B0(7) ./ B0(L_Q.arr(iL, ie, ia)));
                    en_7 = Kfunc(pc_7);

                    % PSD at L=7
                    PSD_7 = J_L7_corrected(en_7) ./ pc_7.^2 .* sin(al_Q.arr(iL, ie, ia));

                    % at all L according to steady state solution PSD1d, and multiplied by sin(alpha)
                    PSD0_arr(iL, ie, ia) = PSD_7 .* PSD_SS(iL);%PSD_SS(1:Q1_size);
                end
            end
        end 
    end

    % loss cone
    lc_idx = al_arr(:) < alc_Dip(L_arr(:));
    PSD0_arr(lc_idx) = PSD0_arr(lc_idx) .* (sinh(al_arr(lc_idx) ./ alc_Dip(L_arr(lc_idx)))).^2 / sinh(1).^2;

    % Number of order of magnitude of PSD to model
    %PSD_ORDERS = 30;
    %psd_min = 10^(log10(max(PSD0_arr(:))) - PSD_ORDERS);
    %psd_min = 1e-15;
    %PSD0_arr(PSD0_arr<psd_min) = psd_min;
    
return
