function PSD0_arr = initial_PSD_two_belts_3D(L_arr, pc_arr, al_arr, Kp0, Dfe_scale, L_chorus_limit)
    %%
    % INITIAL_PSD_TWO_BELTS_3D initial PSD with two belts
    %
    % PSD0_arr = initial_PSD_two_belts_3D(L_arr, pc_arr, al_arr, Kp0)
    %
    % Initial PSD as a steady-state slution of radial diffusion 
    % with parameterized chorus and hiss losses and other stuff
    % required for two-belts structure
    %
    % Input:
    %   Arrays are 3D : [number of points in L] x [number of points in energy] x [number of points in pitch-angle]
    %
    %   L_arr - L* array
    %   pc_arr - pc array
    %   al_arr - pitch-angle array
    %   Kp0 - Kp index
    %
    % Output:
    %   PSD0 in 1/3e7*#/(cm MeV/c)^3 units!!

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plasma.Kfunc
    import plasma.pc2mu
    import plasma.Dip.Lalpha2K
    import plasma.Dip.LK2alpha
    import plasma.mu2pc
    
    import plasma.calc_Coulomb
    import plasma.calc_Precipitation
    import plasma.Dip.alc_Dip
    
    import diff_coeff.calc_DLL
    import diff_coeff.calc_Dfe

    import VERB4D_scripts.steady_state

    import sat_data.J_L7_corrected
    
    try
        Dfe_scale;
    catch
        Dfe_scale = 0.1;
    end

    try
        L_chorus_limit;
    catch
        L_chorus_limit = [2, 3.5];
    end
    
    L_size = size(L_arr, 1);
    pc_size = size(L_arr, 2);
    al_size = size(L_arr, 3);
    PSD0_arr = zeros(L_size, pc_size, al_size);
    
    en_arr = Kfunc(pc_arr);
    
    % Initial condition, PSD0. 
    %Radial diffusion steady state
        
    warning('off', 'VERB4D:DIPOLE')
    if(true) 
        
        
        for iE = 1:size(en_arr, 2)
            for ia = 1:size(al_arr, 3)

                % PSD_SS from L_max to L_min, should include L=7 and L=1, and PSD_SS(L=7) = 1;
                L1d = L_arr(:, iE, ia);
                pc1d = pc_arr(:, iE, ia);
                al1d = al_arr(:, iE, ia);
                if (min(L1d) > 1)
                    L1d = [1, L1d']';
                    mu_val = pc2mu(L_arr(1, iE, ia), pc_arr(1, iE, ia), al_arr(1, iE, ia));
                    K_val = Lalpha2K(L_arr(1, iE, ia), al_arr(1, iE, ia));                    
                    al1d = [LK2alpha(L1d(1), K_val), al1d']';
                    pc1d = [mu2pc(L1d(1), mu_val, al1d(1)), pc1d']';
                end
                if (max(L1d) < 7)
                    L1d = [L1d', 7]';
                    mu_val = pc2mu(L_arr(end, iE, ia), pc_arr(end, iE, ia), al_arr(end, iE, ia));
                    K_val = Lalpha2K(L_arr(end, iE, ia), al_arr(end, iE, ia));                    
                    al1d = [al1d', LK2alpha(L1d(end), K_val)]';
                    pc1d = [pc1d', mu2pc(L1d(end), mu_val, al1d(end))]';
                end
                en1d = Kfunc(pc1d); % it's energy at each L1d

                [tmp, first_idx] = min(abs(L1d - L_arr(1, iE, ia)));
                [tmp, last_idx] = min(abs(L1d - L_arr(end, iE, ia)));
                
                tau_c = 1e99; tau_p = 1e99;
                tau_c = calc_Coulomb(L1d, en1d); % doesn't really do anything :(
                
                % calculate tau_p and limit it to the slot region
                tau_p = calc_Precipitation(Kp0, L1d); % precipitation in slot region
                tau_p(L1d<L_chorus_limit(1) | L1d>L_chorus_limit(2)) = 1e99;  % limit to the slot region from 2 to 3.5 L*
                
                tau_chorus = 1e99 * ones(size(L1d));%40 * ones(size(L1d));
                tau_chorus(L1d<=3.1) = 1e99;  % limit to the slot region from 2 to 3.5 L*
                
                %tau = tau_p .* tau_c ./ (tau_p + tau_c); %1.0/(1.0/tau_p + 1.0/tau_c);
                tau = 1 ./ (1 ./ tau_p + 1 ./ tau_c + 1 ./ tau_chorus);

                DLLB = calc_DLL(Kp0, L1d);
                DLLE = Dfe_scale*calc_Dfe(al1d, en1d, L1d, Kp0);
                D1d = DLLB + DLLE;

                PSD_SS = steady_state(L1d, D1d, tau);

                
                
                
                
                % PSD should be 1 at L=7. This is important if max(L) > 7
                [tmp, L7_idx] = min(abs(L1d - 7));
                PSD_SS = PSD_SS / PSD_SS(L7_idx);
                                
                % PSD(L=7, en, al) from Fluxes at L=7 and the assumption sin(alpha)
                pc_L7 = pc1d(L7_idx);
                en_L7 = Kfunc(pc_L7);
                al_L7 = al1d(L7_idx);
                PSD_L7 = J_L7_corrected(en_L7) ./ pc_L7.^2 * sin(al_L7);

                % PSD = PSD_SS * PSD(L=7)
                PSD0_arr(:, iE, ia) = PSD_L7 * PSD_SS([first_idx:last_idx]);                
                
            end
        end
        
    else 
        %tau = 1e99;
        %tau = 4./Kp0;
        %tau = 3/2/Kp0;
        %tau = 1e99;

        first_idx = 1;
        last_idx = L_size;
        
        L1d = squeeze(L_arr(:,1,1));

        % Add L=7 point if it's not there
        if min(L1d) > 1 
            L1d = [1, L1d']'; 
            first_idx = first_idx + 1;
            last_idx = last_idx + 1;
        end;
        if max(L1d) < 7 
            L1d = [L1d', 7]'; 
        end;
        
        %% Create sready-state
        PSD_SS = zeros(length(L1d), size(en_arr,2));
        for iE = 1:size(en_arr, 2)
            en1d = squeeze(en_arr(:, iE, end));
            al1d = squeeze(al_arr(:, iE, end));

            K_val = Lalpha2K(L_arr(end, iE, end), al_arr(1,iE,end));
            mu_val = pc2mu(L1d(1), pc_arr(1, iE, end), al_arr(1,iE,end));
            en1d = Kfunc(mu2pc(L1d, mu_val, al_arr(1,iE,end)));
            al1d = LK2alpha(L1d, K_val);

            tau_c = 1e99; tau_p = 1e99;
            tau_c = calculate_Coulomb(L1d, en1d);
            tau_p = calculate_Precipitation(Kp0, L1d);
            tau = tau_p .* tau_c ./ (tau_p + tau_c); %1.0/(1.0/tau_p + 1.0/tau_c);

            DLLB = calculate_DLL(Kp0, L1d);
            DLLE = calculate_Dfe(al1d, en1d, L1d, Kp0);
            D1d = DLLB + DLLE;
            %D1d = pow(10, 0.506.*Kp0 - 9.325) .* pow(L1d, 10);
            %D1d(:) = 1;

            %tau = 4./Kp0;
            %D1d = pow(10, 0.506.*Kp0 - 9.325) .* pow(L1d, 10);

            PSD_SS(:, iE) = steady_state(L1d, D1d, tau);

            PSD_SS(PSD_SS<1e-9) = 1e-9;

            % PSD should be 1 at L=7. This is important if max(L) > 7
            [tmp, L7_idx] = min(abs(L1d - 7));
            PSD_SS(:, iE) = PSD_SS(:, iE) / PSD_SS(L7_idx, iE);
        end

        %figure
        %plot(L1d, tau, '*-');


        PSD_L_last = zeros(1, size(en_arr,2), size(al_arr,3));
        if (true)
            % PSD at L=7
            pc_7 = pc_arr(end, :, :) .* sqrt(B_dip(7) ./ B_dip(L_arr(end, :, :)));
            en_7 = Kfunc(pc_7);
            PSD_7 = J_L7_corrected(en_7) ./ pc_7.^2;

            % PSD at L=L-max
            for iE = 1:size(en_arr, 2)
                PSD_L_last(1, iE, :) = PSD_7(1, iE, :) ./ PSD_SS(L7_idx, iE) .* PSD_SS(last_idx, iE) .* sin(al_arr(L_size, iE, :)); % the PA distribution depends on where the maximum L* is
            end

            for iE = 1:pc_size
                for iA = 1:al_size
                    PSD0_arr(:, iE, iA) = PSD_L_last(1, iE, iA) .* PSD_SS(first_idx:last_idx, iE);% ./ PSD_SS(last_idx);
                end
            end
        end
    
    end
    
    % commented
    if (false)    
        for iL = 1:Q1_size
            for ie = 1:Q2_size
                for ia = 1:Q3_size

                    % energy at L=7, by conservation of mu
                    % This is not perfectly correct, we need to recalculate pc_7 for each L
                    % But I think this is how we did it in VERB 2.0
                    % The difference is barely noticable anyway
                    pc_7 = pc_Q.arr(end, ie, ia) .* sqrt(B0(7) ./ B0(L_Q.arr(end, ie, ia)));

                    %this is more correct
                    %pc_7 = pc_Q.arr(iL, ie, ia) .* sqrt(B0(7) ./ B0(L_Q.arr(iL, ie, ia)));
                    en_7 = Kfunc(pc_7);

                    % PSD at L=7
                    PSD_7 = J_L7_corrected(en_7) ./ pc_7.^2 .* sin(al_Q.arr(iL, ie, ia));

                    % at all L according to steady state solution PSD1d, and multiplied by sin(alpha)
                    PSD0_arr(iL, ie, ia) = PSD_7 .* PSD_SS(iL);%PSD_SS(1:Q1_size);
                end
            end
        end 
    end
    
    % loss cone
    lc_idx = al_arr(:) < alc_Dip(L_arr(:));
    PSD0_arr(lc_idx) = PSD0_arr(lc_idx) .* (sinh(al_arr(lc_idx) ./ alc_Dip(L_arr(lc_idx)))).^2 / sinh(1).^2;

    % Number of order of magnitude of PSD to model
    PSD_ORDERS = 20;
    psd_min = 10^(log10(max(PSD0_arr(:))) - PSD_ORDERS);
    %psd_min = 1e-15;
    PSD0_arr(PSD0_arr<psd_min) = psd_min;

    warning('on', 'VERB4D:DIPOLE')
    
end
