function [Dpp_Q_arr, Dpa_Q_arr, Daa_Q_arr] = interpolate_extrapolate_3D(L_arr, pc_arr, al_arr, Dpp_arr, Dpa_arr, Daa_arr, L_Q_arr, pc_Q_arr, al_Q_arr, L_tolerance)
    %%
    % INTERPOLATE_EXTRAPOLATE_3D interpolate Diffusion Coefficient to a new gird
    % Extrapolates at the edges if the new grid is larger than the old grid
    %
    % [Dpp_Q_arr, Dpa_Q_arr, Daa_Q_arr] = interpolate_extrapolate_3D(L_arr, pc_arr, al_arr, Dpp_arr, Dpa_arr, Daa_arr, L_Q_arr, pc_Q_arr, al_Q_arr, L_tolerance)
    %
    % Input:
    %   Arrays are 3D : [number of points in L] x [number of points in energy] x [number of points in pitch-angle]
    %
    %   L_arr - L* array where the diffusion coefficients are computed
    %   pc_arr - pc array where the diffusion coefficients are computed
    %   al_arr - pitch-angle array where the diffusion coefficients are computed
    %
    %   Dpp_arr, Dpa_arr, Daa_arr - diffusion coefficients arrays
    %
    %   L_arr - L* array, new grid
    %   pc_arr - pc array, new grid
    %   al_arr - pitch-angle array, new grid
    %
    %   L_tolerance - (optional) we don't interpolate over L*, so we search for the same L*. This is a 'tolerance' parameter for that search
    %   
    % Output:
    %  Dxx on the new grid

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    try
        L_tolerance;
    catch
        L_tolerance = 0.001;
    end


    L_size = size(L_arr, 1);
    pc_size = size(L_arr, 2);
    al_size = size(L_arr, 3);

    L_Q_size = size(L_Q_arr, 1);
    pc_Q_size = size(L_Q_arr, 2);
    al_Q_size = size(L_Q_arr, 3);
    
    Dpp_Q_arr = zeros(L_Q_size, pc_Q_size, al_Q_size);
    Dpa_Q_arr = zeros(L_Q_size, pc_Q_size, al_Q_size);
    Daa_Q_arr = zeros(L_Q_size, pc_Q_size, al_Q_size);

%    try
%        Dpp_Q.name = Dpp.name;
%        Dpa_Q.name = Dpa.name;
%        Daa_Q.name = Daa.name;
%    catch
%        Dpp_Q.name = 'Dpp ';
%        Dpa_Q.name = 'Dpa ';
%        Daa_Q.name = 'Daa ';
%    end
    
    max_L = max(L_arr(:));
    min_L = min(L_arr(:));

    % adding grid points at alpha=0, alpha=90, and energy=0.01MeV
    if (true)
        % adding 0 and pi/2
        alpha_size = size(al_arr,3);
        al_arr(:,:,[2:alpha_size+1]) = al_arr;
        al_arr(:,:,alpha_size+2) = pi/2;
        al_arr(:,:,1) = 0;
        % grid is uniform
        pc_arr(:,:,[3:alpha_size+2])    = pc_arr;
        Daa_arr(:,:,[2:alpha_size+1])   = Daa_arr;
        Daa_arr(:,:,alpha_size+2)       = Daa_arr(:,:,alpha_size);
        Dpa_arr(:,:,[2:alpha_size+1])   = Dpa_arr;
        Dpa_arr(:,:,alpha_size+2)       = Dpa_arr(:,:,alpha_size);
        Dpp_arr(:,:,[2:alpha_size+1])   = Dpp_arr;
        Dpp_arr(:,:,alpha_size+2)       = Dpp_arr(:,:,alpha_size);

        % adding min energy
        pc_size = size(pc_arr,2);
        pc_arr(:,[2:pc_size+1],:) = pc_arr;
        pc_arr(:,1,:) = 0.001;
        % grid is uniform
        al_arr(:,[2:pc_size+1],:) = al_arr;
        Daa_arr(:,[2:pc_size+1],:) = Daa_arr;
        Dpa_arr(:,[2:pc_size+1],:) = Dpa_arr;
        Dpp_arr(:,[2:pc_size+1],:) = Dpp_arr;
    end
    
    
    %fprintf(1, 'Interpolating %s: L=    ', Dpp.name(4:end));
    for iL_Q = 1:L_Q_size
        % go to 2d
        %iL_Q

        if (((L_Q_arr(iL_Q, 1, 1) > max_L) || (L_Q_arr(iL_Q, 1, 1) < min_L)) && (max_L > min_L))
            % if we're outside of Dxx range
            Dpp_Q_2d = zeros(pc_Q_size, al_Q_size);
            Dpa_Q_2d = zeros(pc_Q_size, al_Q_size);
            Daa_Q_2d = zeros(pc_Q_size, al_Q_size);
        else        
            % Find the same L* in diffusion coefficient file (we don't interpolate between L*)
            [tmp, L_slice] = min(abs(squeeze(L_arr(:, 1, 1)) - L_Q_arr(iL_Q, 1, 1)));
            if (tmp > L_tolerance)
                error(['L-grid is different in the diffusion coefficient files: ', num2str(L_arr(L_slice, 1, 1)), ' vs ', num2str(L_Q_arr(iL_Q, 1, 1)), '. ', 'We do not interpolate between L'])
            else
                fprintf(1, '\b\b\b\b%4.1f', L_arr(L_slice,1,1))
            end
            L_val = L_arr(L_slice, 1, 1);

            al_2d = squeeze(al_arr(L_slice,:,:));
            pc_2d = squeeze(pc_arr(L_slice,:,:));

            Dpp_2d = squeeze(Dpp_arr(L_slice,:,:));
            Dpa_2d = squeeze(Dpa_arr(L_slice,:,:));
            Daa_2d = squeeze(Daa_arr(L_slice,:,:));

            pc_Q_2d = squeeze(pc_Q_arr(iL_Q, :, :));
            al_Q_2d = squeeze(al_Q_arr(iL_Q, :, :));
            
            % normalize coordinates - othervise MATLAB gets really confused
            % with small numbers. Really confused. And it's impossible to
            % find the reason, very annoying.
            a_norm = max(al_2d(:));
            pc_norm = max(pc_2d(:));
            
            % Interpolating Dxx
            Dpp_Q_2d = griddata(pc_2d./pc_norm, al_2d./a_norm, Dpp_2d, pc_Q_2d./pc_norm, al_Q_2d./a_norm, 'linear'); % nearest is used to 'extrapolate', to not have zeros at low-E boundary
            Daa_Q_2d = griddata(pc_2d./pc_norm, al_2d./a_norm, Daa_2d, pc_Q_2d./pc_norm, al_Q_2d./a_norm, 'linear');
            Dpa_Q_2d = griddata(pc_2d./pc_norm, al_2d./a_norm, Dpa_2d, pc_Q_2d./pc_norm, al_Q_2d./a_norm, 'linear');
            %Dpp_Q_2d = griddata(pc_2d, al_2d, Dpp_2d, pc_Q_2d, al_Q_2d, 'nearest', {'QJ'}); % nearest is used to 'extrapolate', to not have zeros at low-E boundary
            %Dpa_Q_2d = griddata(pc_2d, al_2d, Dpa_2d, pc_Q_2d, al_Q_2d, 'nearest', {'QJ'});
            %Daa_Q_2d = griddata(pc_2d, al_2d, Daa_2d, pc_Q_2d, al_Q_2d, 'nearest', {'QJ'});

                    
            %% Fix zeros at the edges after extrapolation
            Dpp_Q_2d(isnan(Dpp_Q_2d)) = 1e-21;
            Daa_Q_2d(isnan(Daa_Q_2d)) = 1e-21;
            Dpp_Q_2d(Dpp_Q_2d<1e-21) = 1e-21;
            Daa_Q_2d(Daa_Q_2d<1e-21) = 1e-21;
            Dpa_Q_2d(isnan(Dpa_Q_2d)) = 0;
        end
        
        % Save back to 3D
        Dpp_Q_arr(iL_Q,:,:) = Dpp_Q_2d;
        Dpa_Q_arr(iL_Q,:,:) = Dpa_Q_2d;
        Daa_Q_arr(iL_Q,:,:) = Daa_Q_2d;
        
    end

    fprintf(1, '\n');
    
end
