function PSD_interp = load_and_interpolate_PSD_mat(folder, Q1, Q2, target_Q1, target_Q2, varargin)
    %%
    % LOAD_AND_INTERPOLATE_PSD loads and interpolate PSD from VERB 4D format
    %
    % PSD_interp = load_and_interpolate_PSD(folder, Q1, Q2, target_Q1, target_Q2, varargin)
    %
    % Input:
    %   Q1 - either mu or energy grid array
    %   Q2 - either K or alpha grid array (!!! Q2 must be independent of Q1 !!!)
    %   target_Q1 - target value of Q1 for interpolation
    %   target_Q2 - target value of Q2 for interpolation
    %   varargin - optional argument pairs as "argument, value":
    %     'min' or 'first', N - skip first N files
    %     'max' or 'last', N - read N files total
    %     'step', N - skip N files before reading the next one
    %     'reload' - force reload PSD from original files (otherwise will try to load from temporary matlab files first)
    %     'method',  'interp'|'nearest' - interpolation method
    %     'tmp_file_name' - filename for temporary file
    %
    % Output:
    %   PSD_interp - loaded PSD as 5D array [ time x MLT x Radial distance x energy x pitch-angle ]
    %

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits

    import read_write.load_PSD;
    import read_write.load_PSD_mat;
    import VERB4D_scripts.fast_interpolate_2D2;

    reload = false;
    method = 'interp';
    files_last = 1e99;
    files_first = 0;
    files_step = 1;

    try
        Q1_name = strrep(Q1.name,'\','');
    catch
        Q1_name = 'Q1';
    end
    try
        Q2_name = strrep(Q2.name,'\','');
    catch
        Q2_name = 'Q2';
    end

    tmp_file_name = -1;
    optional_arguments = 5;
    if (nargin > optional_arguments)
        for it = 1:1:nargin-optional_arguments
            if strcmp(varargin(it), 'reload')
                reload = true;
            elseif strcmp(varargin(it), 'method')
                method = varargin{it+1};
                it = it + 1;
            elseif strcmp(varargin(it), 'tmp_file_name')
                tmp_file_name = varargin{it+1};
                it = it + 1;
            elseif (strcmp(varargin(it), 'max') || strcmp(varargin(it), 'last'))
                files_last = varargin{it+1};;
                it = it + 1;
            elseif (strcmp(varargin(it), 'min') || strcmp(varargin(it), 'first'))
                files_first = varargin{it+1};;
                it = it + 1;
            elseif strcmp(varargin(it), 'step')
                files_step = varargin{it+1};
                it = it + 1;
            end
        end
    end

    if (tmp_file_name == -1)
        tmp_file_name = ['plot_PSD_', Q1_name, '=', num2str(target_Q1), '_', Q2_name, '=', num2str(target_Q2), '_', method, '.mat'];
    %    tmp_file_name = ['PSD_', Q1_name, '=', num2str(target_Q1), '_', Q2_name, '=', num2str(target_Q2), '.mat'];
    end

    try
        if reload
            error('1')
        end
        
        load([folder, tmp_file_name]);
        ['Loaded from temporary file ', tmp_file_name]

    catch

        %% Load PSD    
        PSD = load_PSD_mat(folder, 'max', files_last, 'step', files_step, 'first', files_first);

        %%
        % Interpolate
        
        %PSD.arr(find(PSD.arr<0)) = 1e-31;
        %PSD.arr(isnan(PSD.arr)) = 1e-31;

        PSD_interp.arr = zeros(size(PSD.arr, 1), size(PSD.arr, 2), size(PSD.arr, 3));

        if (strcmp(method,'interp'))

            PSD.arr(PSD.arr <= 0) = NaN;
            PSD.arr(PSD.arr<=1e-21) = 1e-21;
            
            log_PSD.arr = log10(PSD.arr);
            for it = 1:size(PSD.arr, 1)
                for iP = 1:size(PSD.arr, 2)
                    for iR = 1:size(PSD.arr, 3)
                        %PSD_interp.arr(it, iP, iR) = 10.^fast_interpolate_2D2(squeeze(mu.arr(iP, iR, :, :)), squeeze(K.arr(iP, iR, :, :)), squeeze(log_PSD.arr(it, iP, iR, :, :)), target_mu, target_K);
                        %PSD_interp.arr(it, iP, iR) = fast_interpolate_2D2(squeeze(en.arr(iP, iR, :, :)), squeeze(al.arr(iP, iR, :, :)), squeeze(log_PSD.arr(it, iP, iR, :, :)), target_en, target_al);
                        %PSD_interp.arr(it, iP, iR) = fast_interpolate_2D2(squeeze(en.arr(iP, iR, :, :)), squeeze(al.arr(iP, iR, :, :)), squeeze(PSD.arr(it, iP, iR, :, :)), target_en, target_al);
                        %PSD_interp.arr(it, iP, iR) = fast_interpolate_2D2(squeeze(Q1.arr(iP, iR, :, :)), squeeze(Q2.arr(iP, iR, :, :)), squeeze(PSD.arr(it, iP, iR, :, :)), target_Q1, target_Q2);
                        PSD_interp.arr(it, iP, iR) = 10.^fast_interpolate_2D2(squeeze(Q1.arr(iP, iR, :, :)), squeeze(Q2.arr(iP, iR, :, :)), log10(squeeze(PSD.arr(it, iP, iR, :, :))), target_Q1, target_Q2);
                    end
                end
            end

            PSD_interp.Q1 = target_Q1;
            PSD_interp.Q2 = target_Q2;
                
        elseif (strcmp(method,'nearest'))

            % Nearest
            [tmp, Q2_idx] = min(abs(Q2.arr(1,1,1,:) - target_Q2)) % K is one of the coordinates and should be orthogonal to other coordinates
            [tmp, Q1_idx] = min(abs(Q1.arr(1,1,:,Q2_idx) - target_Q1))
            %K_idx = 1;
            %V_idx = 45;
            PSD_interp.arr = (PSD.arr(:, :, :, Q1_idx, Q2_idx));
            %log_PSD_interp = log10(PSD_interp);    
            
            PSD_interp.Q1 = unique(Q1.arr(1,1,Q1_idx, Q2_idx));
            PSD_interp.Q2 = unique(Q2.arr(1,1,Q1_idx, Q2_idx));
            
        else 
            error(['Unknown method ', method])
        end    

        try
            PSD_interp.Q1_text = [Q1.name, ' = ', num2str(PSD_interp.Q1), Q1.units];
            PSD_interp.Q2_text = [Q2.name, ' = ', num2str(PSD_interp.Q2), Q2.units];
        catch end
        
        try
            PSD_interp.units = PSD.units;
        catch end
        
        PSD_interp.time = PSD.time;
        
        
        %save([folder, 'PSD_mu=', num2str(target_mu), '_K=', num2str(target_K), '.mat'], 'PSD_interp', 'new_L', 'PSD_time')
        [folder, tmp_file_name]
        save([folder, tmp_file_name], 'PSD_interp')

    end


end
