function  Field_line_arr = trace_field_line(External_field, Internal_field, sysaxes, matlabd, maginput, Bmin_xGEO_arr, Bmin_arr, varargin)
    %%
    % TRACE_FIELD_LINE traces magnetic field line in Tsyganenko field
    %
    % Field_line_arr = trace_field_line(External_field, Internal_field, sysaxes, matlabd, maginput, Bmin_xGEO_arr, Bmin_arr, varargin)
    %
    % Input:
    %   External_field - Onera external field parameter, '0' or '4' are acceptable
    %   Internal_field - Onera internal field parameter, '0', '1', and '5' are acceptable
    %   sysaxes - Onera sysaxis parameter, use '1' to avoid problems
    %   matlabd - Onera matlabd parameter, just the date you wanna use in matlab format, e.g. datenum('1990-01-01')
    %   maginput - Onera maginput parameter, just read Onera's description
    %   Bmin_xGEO - coordinates os magnetic equator in sysaxis coordinates
    %%   Bmin_arr - magnetic field in Tesla
    %   Optional arguments (varargin), []-indicates default value:
    %         'do_interpolate', true|[false] - if interpolate to a denser latitudinal grid
    %         'ever_add_Bmin_points', true|[true] - if ever add equatorial point from Onera library to the field line tracing
    %         'plot_wrong_Bmin', true|[false] - if plot wrong equatorial points
    %         'add_points_inside_Earth', [true]|false - if add dipolar points inside the Earth to the field line 
    %
    % Output:
    %   Field_line.R - radial distance of the field line points (array)
    %   Field_line.lat - latitude of the field line points (array)
    %   Field_line.phi - MLT angle of the field line points (array), in rad probably, also not sure where phi=0 is
    %   Field_line.B - magnetic field value along the field line

    % Author: Ksenia Orlova
    % Email: korlova@atmos.ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plasma.Dip.B_Dip

    do_interpolate = false;
    ever_add_Bmin_points = true;
    add_points_inside_Earth = true;
    plot_wrong_Bmin = false;
    
    % Optional arguments
    it = 1;
    while it <= length(varargin)
        if strcmp(varargin(it), 'do_interpolate')
            do_interpolate = varargin{it+1};
            it = it + 1;
        elseif strcmp(varargin(it), 'ever_add_Bmin_points')
            ever_add_Bmin_points = varargin{it+1};
            it = it + 1;
        elseif strcmp(varargin(it), 'add_points_inside_Earth')
            add_points_inside_Earth = varargin{it+1};
            it = it + 1;
        elseif strcmp(varargin(it), 'plot_wrong_Bmin')
            plot_wrong_Bmin = varargin{it+1};
            it = it + 1;
        else

            error(['Unknown optional argument: ', num2str(varargin{it})])
        end
        it = it + 1;
    end
    
    % Check that B is in Tesla :(
    if (max(Bmin_arr(:)) > 1e-4)
        max_Bmin = max(Bmin_arr(:));
        idx = find(Bmin_arr == max_Bmin);
        error(['B should be in Tesla! (Bmax(', num2str(Bmin_xGEO_arr(idx, 1)), ',', num2str(Bmin_xGEO_arr(idx, 2)), ',', num2str(Bmin_xGEO_arr(idx, 3)), ')=', num2str(Bmin_arr(idx)), ' ?? ', num2str(idx), ')']);
    else
        ['Bmax=', num2str(max(Bmin_arr(:)))]
    end
    
    if (sysaxes ~= 1)
        error('trace_field_line currently works only for sysaxis=1, I think')
        % That's probably only because of onera_desp_lib_rotate(Bmin_xGEO,'car2sph');
        % that we use to find latitude
    end

    Field_line_arr = struct('R',cell(size(Bmin_arr)),'lat',cell(size(Bmin_arr)),'phi',cell(size(Bmin_arr)),'B',cell(size(Bmin_arr)));

    for x_ind = 1:length(Bmin_arr)
        if (isnan(Bmin_arr(x_ind)))
            continue;
        end

        Bmin = Bmin_arr(x_ind) * 1e9; % Tesla to NanoTesla
        Bmin_xGEO = Bmin_xGEO_arr(x_ind, :);

        Field_line.R = -1;
        Field_line.lat = -1;
        Field_line.phi = -1;
        Field_line.B = -1;

        R0=1.0156; % distance of 100 km from the Earth's surface (where electrons are lost)
        %R0 = 1;
        %R0=0.01;
        %R0=0.2;

        options = [0, 0, 0, 0, Internal_field];

        xSPH = onera_desp_lib_rotate(Bmin_xGEO,'car2sph');
        %xSPH(1,4) = Bmin*1e-9; % from nT to T
        eq_R = xSPH(:,1);
        eq_lat = xSPH(:,2);
        eq_phi = xSPH(:,3);
        eq_B = Bmin*1e-9; % from nT to T

        if (eq_R < R0)
            Lm = NaN;
            Blocal = Bmin;
            Bmin = Bmin;
            J = NaN;
            POSIT = Bmin_xGEO;
        else
            % TODO: Move tracing to a separate function
            %['Tracing: ', num2str(Bmin_xGEO)]
            [Lm,Blocal,Bmin2,J,POSIT] = onera_desp_lib_trace_field_line(External_field,options,sysaxes,matlabd,Bmin_xGEO(1),Bmin_xGEO(2),Bmin_xGEO(3),maginput,R0);
            %if (Bmin2 ~= Bmin)
            %   error(['Bmin2 ~= Bmin: ', num2str(Bmin2), ', ', num2str(Bmin)])
            %end
        end

        if (length(POSIT) == 0)
            ['Tracing failed: ', num2str(Bmin_xGEO)]
            return
        end

        POSIT_sph = onera_desp_lib_rotate(POSIT,'car2sph');
        Field_line.R = POSIT_sph(:,1)';
        Field_line.lat = POSIT_sph(:,2)';
        Field_line.phi = POSIT_sph(:,3)';
        Field_line.B = Blocal'*1e-9; % nT to T

        %lambda_equator_deg = Field_line.equator(2); % latitude of magnetic equator (=minimum of magnetic field)
        if (ever_add_Bmin_points)
            add_eq_point = false;
            if (min(Field_line.B) > eq_B)
                try
                    north_idx = find(Field_line.lat > eq_lat);
                    south_idx = find(Field_line.lat < eq_lat);

                    if (Field_line.B(north_idx(end)) < min(Field_line.B(north_idx(1:end-1))) && ...
                            Field_line.B(south_idx(1)) < min(Field_line.B(south_idx(2:end))))

                        %if (Field_line.B(max(north_idx)) <= min(Field_line.B) && ...
                        %        Field_line.B(min(south_idx)) < min(Field_line.B))

                        add_eq_point = true;

                    end
                end
            end

            if (add_eq_point)
                Field_line.R = [Field_line.R(north_idx), eq_R, Field_line.R(south_idx)];
                Field_line.lat = [Field_line.lat(north_idx), eq_lat, Field_line.lat(south_idx)];
                Field_line.phi = [Field_line.phi(north_idx), eq_phi, Field_line.phi(south_idx)];
                Field_line.B = [Field_line.B(north_idx), eq_B, Field_line.B(south_idx)];
            else
                if (plot_wrong_Bmin)
                    %%
                    figure
                    hold on
                    plot(Field_line.lat, Field_line.B, '.-');
                    plot(eq_lat, eq_B, '*r');
                    set(gca, 'yscale', 'log');
                    text(max(xlim), max(ylim), {num2str(Bmin_xGEO)}, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'Color', [0, 0, 0]);
                end
            end
        end


        % Fix Phi_field
        for i = 2:length(Field_line.phi)
            if (Field_line.phi(i) > Field_line.phi(i-1) + 360/2)
                Field_line.phi(i) = Field_line.phi(i) - 360;
            elseif (Field_line.phi(i) < Field_line.phi(i-1) - 360/2)
                Field_line.phi(i) = Field_line.phi(i) + 360;
            end
        end

        % interpolate
        if (do_interpolate & length(Field_line.lat) > 1)
            new_lat = [min(Field_line.lat(:)):(max(Field_line.lat(:)) - min(Field_line.lat(:)))/1000:max(Field_line.lat(:))];
            Field_line.R = interp1(Field_line.lat, Field_line.R, new_lat, 'PCHIP');
            Field_line.phi = interp1(Field_line.lat, Field_line.phi, new_lat, 'PCHIP');
            Field_line.B = interp1(Field_line.lat, Field_line.B, new_lat, 'PCHIP');
            Field_line.lat = new_lat;
        end


        %% add points inside the Earth
        if (add_points_inside_Earth)
            %            if (max(Field_line.B) < B_m) Bmin_xGEO_arr

            % not enough traced points
            % points inside the earth
            %R_add = [0.1:0.1:Field_line.R(1)-0.1+2];
            %lat_add = acos(sqrt(cos(Field_line.lat(1)*pi/180).^2 .* R_add ./ Field_line.R(1)));

            lat0_south = min(Field_line.lat);
            lat0_north = max(Field_line.lat);
            lat_add_south = [lat0_south-1:-1:-89]*pi/180;
            lat_add_north = [89:-1:lat0_north+1]*pi/180;
            %R0 = sqrt(sum(Bmin_xGEO_arr(1,:).^2));
            R0_south = Field_line.R(1)/cos(lat0_south*pi/180).^2;
            R0_north = Field_line.R(end)/cos(lat0_north*pi/180).^2;
            R_add_south = Field_line.R(1)/cos(lat0_south*pi/180).^2 .* cos(lat_add_south).^2;
            R_add_north = Field_line.R(end)/cos(lat0_north*pi/180).^2 .* cos(lat_add_north).^2;
            B_add_south = B_Dip(R0_south, lat_add_south, 0.31) * 1e-4; % In Tesla
            B_add_north = B_Dip(R0_north, lat_add_north, 0.31) * 1e-4; % In Tesla

            if (false)
                %%
                figure
                hold on
                plot(lat_add_south*180/pi, B_add_south*1e4, 'r-*');
                plot(lat_add_north*180/pi, B_add_north*1e4, 'g-*');
                plot(Field_line.lat, Field_line.B*1e4, '-*');
                %xlim([-90 90])
                %ylim([1e-6 1e-2]);
                set(gca, 'yscale', 'log')
                xlabel('Latitude')
                ylabel('B')
                '1'
            end


            %if (max(Field_line.B) > min(B_add))
            %    error('Adding B error')
            %end

            Field_line.lat = [lat_add_north*180/pi, Field_line.lat, lat_add_south*180/pi];
            Field_line.R   = [R_add_north, Field_line.R, R_add_south];
            Field_line.phi = [repmat(Field_line.phi(1), [length(lat_add_north) 1])', Field_line.phi, repmat(Field_line.phi(end), [length(lat_add_south) 1])'];
            Field_line.B   = [B_add_north, Field_line.B, B_add_south]; % 1e9 - some big number
            %            end
        end

        Field_line_arr(x_ind) = Field_line;
    end

end
