classdef Module < handle
    %MODULE general class for all modules

    properties
        Log = VERB_Classes.Logger('mlog.txt');
        
        config = {}; % configuration for the plugins that may be loaded from the config.ini file
    end
    
    properties(Access=private)
         executed = false; % flag,  has the module been run
       
    end
    
    properties(Access=public)
          manager; % manager object
    end
    
    methods
        function obj = Module(manager)
            obj.executed = false; 
            obj.manager = manager;
        end
        
        function execute(obj,varargin)
            
            %varargin - any objects that must be loaded into the module            
            obj.Log.log(['[Execute module] ' class(obj)]);
            
            obj.filter('pre_load',varargin{:});
            obj.load(varargin{:});
            obj.filter('post_load',varargin{:});
            
            %check the dependence                      
            M = properties(obj); % Get list of the properties for the module that is inherited from the class Module
            for iM = 1:numel(M)
                if(isa(obj.(M{iM}),'VERB_Modules.Module')) % Check that the object is a Module (inherited)                   
                   if(~obj.(M{iM}).executed) % execute module if it was not
                       obj.Log.log(['Dependence ' class(obj.(M{iM})) ' needs to be executed for ' class(obj)]);
                       obj.(M{iM}).execute(varargin{:});
                   end
                end
            end             
            
            % config may be asked in the run plugins... 
            % (maybe it needs to be changed)...
            obj.config = [obj.config, varargin];            
            
            obj.filter('pre_run');
            obj.run();
            obj.filter('post_run');
            
            obj.filter('pre_save');
            obj.save();            
            obj.filter('post_save');
            
            obj.executed = true; % run complete
            
            % curret module executed. Reset all modules that have the current
            % module as a property (as the depencende). 
            M = properties(obj.manager); % Get list of the all modules from the manager
            for iM = 1:numel(M)
                if(isa(obj.manager.(M{iM}),'VERB_Modules.Module')) % Check that the object is a Module (inherited)   
                   P = properties(obj.manager.(M{iM})); % Get list of all modules (dependences)
                   for iP = 1:numel(P)
                       % If the module from the manager has the dependence 
                       % of the current object then
                       % reset the module from the manager
                       if(isa(obj.manager.(M{iM}).(P{iP}),class(obj))) 
                           obj.manager.(M{iM}).reset;
                       end                                              
                   end
                end
            end 
            
            obj.Log.log(['[Executed module] ' class(obj)]);
        end        
        
        function reset(obj)
            % forget that module was executed
            obj.Log.log(['[Reset module] ' class(obj)]);
            obj.clear();
            obj.executed = false;            
        end
        
        function clear(obj)
            % function design to clear the module content
        end
        
        function b = is_executed(obj)
            b = obj.executed;
        end
        
        function plugins(obj)
            %show available plugins
            module_name   = strrep(class(obj),'VERB_Modules.','');
            %function_name = sprintf('*_%s', module_name);
            %plugin_name   = sprintf('%s.*',class(obj),function_name);
            
            %local function
            lDir = dir(['*_' module_name '.m']);
            fprintf('Local plugins and funcitons: \n');
            for id = 1:numel(lDir)
                fprintf('%s\n',lDir(id).name);
            end
            fprintf('\nModule plugins: \n');
            s = what(['VERB_Modules\' module_name]);
            if(~isempty(s))
             if(~isempty(s.m))
                for id=1:numel(s.m)
                    fprintf('%s\n',s.m{id});
                end
             end
            end
        end
    end
    
    methods(Abstract)        
        load(obj,varargin);
        run(obj);
        save(obj);        
    end
    
    methods(Access=protected)
        function filter(obj,filterName,varargin)
            % find and call the function (filter or plugin)
            % obj.filter(stage,parameters)
            % filterName - name of filter or plugin. filterName_moduleName            
            % parameters - varargin
            
            module_name   = strrep(class(obj),'VERB_Modules.','');
            function_name = sprintf('%s_%s',filterName, module_name);
            plugin_name   = sprintf('%s.%s',class(obj),function_name);
            
            obj.Log.debug(3,sprintf('Search the function %s', function_name))
            
            if(~strcmp(which(function_name),''))
                obj.Log.log(['Found function: ', which(function_name)]); % Show the path to the function
                
                if(nargin > 2)
                    eval([ function_name '(obj,varargin{:});' ])
                else
                    eval([ function_name '(obj);' ])
                end
                
            elseif(~strcmp(which(plugin_name),'')) % looking for a package
                obj.Log.log(['Found plugin: ', which(plugin_name)]); % Show the path to the function
                
                if(nargin > 2)
                    eval([ plugin_name '(obj,varargin{:});' ])
                else
                    eval([ plugin_name '(obj);' ])
                end
                                
            end
                
        end        
        
        function run_plugin(obj, pluginVariable, pluginName)
            % function search and run the plugin. Otherwise it will
            % through an error
            
            plugin = eval(['obj.' pluginVariable]); % I have to go with eval function...
                        
            if( ~isempty(plugin) )
               obj.filter(['plugin_', pluginName, '_',plugin],obj.config{:});
            else
               obj.Log.warning([pluginName ' undefined for ' pluginVariable]);
            end
        end
    end
    
end

