function DLLe = calc_Dfe(alpha, Ke, L, Kp)
    %%
    % CALC_DFE calculates electrostatic radial diffusion coefficient [see Brautigam and Albert(2000), JGR]
    % DLLe = calc_Dfe(alpha, Ke, L, Kp) calculates the coefficient from
    % pitch-angle (alpha), energy (Ke), L, and Kp
    % This model seems to overestimate radial diffusion rate, so we recommend to scale it down
    % by adjusting a parameter "scale_factor" OUTSIDE OF THAT FUNCTION.
    %
    % Inputs: 
    %  alpha in radian, 
    %  Ke in MeV, 
    %  L and Kp in dimensionless unit
    % 
    % Outputs:
    %  DLLe in 1/day or Re^2/day
    
    % Author: Kung-Chan Kim
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    %double E_kp,DLLe;
    %double fac,dy,ty,factor,gamma,beta,wd;
    Re = 6.378*1e8;    % Earth radius(cm)
    mc2 = 0.511;

    % Sometime E-field effect is too strong, so we need to scale down
    % by adjusting this parameter until more accurate DLLe model is found
    %scale_factor = 0.5;
    %scale_factor = 0.65;
    scale_factor = 1;

    % exponenetial decay time in sec
    T0 = 2700.0;

    % convective electric field in mV/m, Kp=1 to 6;
    if(Kp > 6) 
        Kp=6;
    elseif (Kp < 1)
        Kp = 1;
    end
    E_kp = 0.26 * (Kp-1) + 0.1;

    % convert energy(MeV) to mu(MeV/G)
    %pc = Ke*(Ke+2.0*VC::mc2);    % pc in MeV
    %mu = (pc*sin(alpha)*sin(alpha)) / (VF::B(L)*VC::mc2);

    % angular drift velocity wd (rad/s) averaged over a bounce in a dipole
    % [see Schulz and Lanzerotti, 1974 and Schulz, 1991]

    fac = (sin(alpha) .^ (3./4));
    dy = (5.520692-2.357194*sin(alpha)+1.279385*fac) ./ 12.0;
    ty = 1.380173-(0.639693*fac);
    factor = dy./ty;              % pitch angle dependency

    gamma = (Ke + mc2) ./ mc2;
    beta = sqrt(1.0 - 1 ./ ((gamma .^ 2.0)));
    %gamma = sqrt(1.0+2.0*VF::B(L)*mu/VC::mc2);
    %wd = (1.475*1e-3*mu) / gamma / pow(L,2.0) * factor;  % wd(rad/s),mu(MeV/G)
    wd = 1.216 .* 1e-3 .* gamma .* (beta .^ 2.0) .* L .* factor;

    %wd = (1.475*1e-3*Ke) / VF::B(L) / pow(L,2.0) * factor; % wd(rad/s),Ke(MeV),B(G)

    % electrostatic radial diffusion coefficient in cm^2/sec
    B_earth = 0.312;
    %DLLe = (T0 .* (E_kp .^ 2.0)*1e6) ./ (4.0 * (B_dip(1) .^ 2)) ./ (1.0 + ((wd*T0/2.0) .^ 2.0)) .* (L .^ 6); % E_kp(mV/m),wd(1/s),T0(s),B_0(G)
    DLLe = (T0 .* (E_kp .^ 2.0)*1e6) ./ (4.0 * (B_earth .^ 2)) ./ (1.0 + ((wd*T0/2.0) .^ 2.0)) .* (L .^ 6); % E_kp(mV/m),wd(1/s),T0(s),B_0(G)

    DLLe = scale_factor .* DLLe .* (60.0*60.0*24.0) ./ (Re .^ 2);  % DLLe in Re^2/day or 1/day

end
