function [dV_dpc, dV_da, dK_dpc, dK_da] = calc_derivatives_LVK_LPA_3D(L, pc, al, g_shift)    
    %%
    % CALC_DERIVATIVES_LVK_LPA_3D calculates derivatives analitically
    % [dV_dpc, dV_da, dK_dpc, dK_da] = calc_derivatives_LVK_LPA_3D(L, pc,
    % al, g_shift) analitically calculates derivatives dV/d(pc),
    % dV/da, dK/d(pc), dK/da in a DIPOLE field, where 
    % V = mu / (K + g_shift)^2 - is a new adiabatic invariant 
    % K - is the second adiabatic invariant
    % pc = p*c, momentum times speed on light, in MeV
    % al - is equatorial pitch-angle
    % L - is inversially proportional to the third adiabatic invariant
    % g_shift usually equals 0.5
    % (see [Subbotin and Shprits, 2009])
    % 
    % Input: 
    %   L - is the third adiabatic invariant
    %   pc = p*c, momentum times speed on light, in MeV
    %   al - pitch-angle in radians
    %   g_shift = 0.5, a constant
    %   
    % Output:
    %   dV_dp, dV_da, dK_dp, dK_da - derivatives

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plasma.Dip.B_Dip
    import plasma.Dip.Y

    mc2 = 0.511;
    
    try 
        g_shift;
    catch
        g_shift = 0.5;
    end
    
    % I forgot what this is :(
    %Van_arr = pc_Q_arr.^2 .* sin(al_Q_arr).^2 .* L_Q_arr.^3 ./ (2 .* mc2 .* B0(1)) .* (Y(al_Q_arr) ./ sin(al_Q_arr) .* sqrt(B0(1) ./ L_Q_arr) + g_shift).^2;
    %Kan_arr = Y(al_Q_arr) ./ sin(al_Q_arr) .* sqrt(B0(1)./L_Q_arr);

    y = sin(al);
    T_alpha = (1.38 + 0.055 * pow(y, 1.0/3) - 0.32 * pow(y, 1.0/2) - 0.037 * pow(y, 2.0/3) - 0.394 * y + 0.056 * pow(y, 4.0/3));

    dV_dpc = pc .* (sin(al).^2) .* L.^3 ./ (mc2 .* B_Dip(1)) .* ...
        (Y(al)./sin(al) .* sqrt(B_Dip(1)./L) + g_shift).^2;
    %dV_da_arr = - 4 .* cos(al_Q_arr).^2 ./ sin(al_Q_arr) .* pc_Q_arr.^2 ./ mc2 .* L_Q_arr.^3./B0(1) .* sqrt(B0(1)./L_Q_arr) .* T_alpha .*  ...
    %    (Y(al_Q_arr)./sin(al_Q_arr) .* sqrt(B0(1)./L_Q_arr) + g_shift);

    dV_da = pc.^2 .* sin(al) .* cos(al) .* L.^3 ./ mc2 ./ B_Dip(1) .* (Y(al) ./ sin(al) .* sqrt(B_Dip(1)./L) + g_shift) .* ...
        ((Y(al) ./ sin(al) .* sqrt(B_Dip(1)./L) + g_shift) - sin(al) .* sqrt(B_Dip(1)./L) .* 2 .* T_alpha ./ sin(al).^2);

    dK_dpc = zeros(size(L));
    dK_da = - 2 .* cos(al) ./ sin(al).^2 .* sqrt(B_Dip(1)./L) .* T_alpha;
    
end
