function [dJ2_dQ2_arr, dJ2_dQ3_arr, dJ3_dQ2_arr, dJ3_dQ3_arr, G_JQ_arr, dQ2_dJ2_arr, dQ2_dJ3_arr, dQ3_dJ2_arr, dQ3_dJ3_arr, G_QJ_arr] = calc_derivatives_ort_3D(J1_arr, J2_arr, J3_arr, Q1_arr, Q2_arr, Q3_arr)
    %%
    % CALC_DERIVATIVES_ORT_3D calculates transformation from ORTHOGONAL coordinates J to arbitrary coordinates Q in 3D space (i, J1, J2) -> (i, Q1, Q2)
    % [dJ2_dQ2_arr, dJ2_dQ3_arr, dJ3_dQ2_arr, dJ3_dQ3_arr, G_JQ_arr, dQ2_dJ2_arr, dQ2_dJ3_arr, dQ3_dJ2_arr, dQ3_dJ3_arr, G_QJ_arr] = calc_derivatives_ort_3D(J1_arr, J2_arr, J3_arr, Q1_arr, Q2_arr, Q3_arr)
    % calls 2D function coord_transform_ort(J, Q) for each i to calculate derivatives dJ/dQ and dQ/dJ, Jacobian and invert Jacobian that can be used later for coordinates transformation
    %
    % Assume:
    % J - old coordinates
    % Q - new coordinates
    % i - grid indexes
    %
    % Then:
    % G(J->Q) = d(J)/d(Q)
    % G(J->Q) = G(J->ij->Q) = G(J->i) * G(i->Q) = G(J->i) / G(Q->i) = (dJ/di) / (dQ/di)
    %
    % See also: coord_transform_ort

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import diff_coeff.coord_transform_ort
    
    G_JQ_arr = zeros(size(J1_arr));
    dQ2_dJ2_arr = zeros(size(J1_arr));
    dQ2_dJ3_arr = zeros(size(J1_arr));
    dQ3_dJ2_arr = zeros(size(J1_arr));
    dQ3_dJ3_arr = zeros(size(J1_arr));

    G_QJ_arr = zeros(size(J1_arr));
    dJ2_dQ2_arr = zeros(size(J1_arr));
    dJ2_dQ3_arr = zeros(size(J1_arr));
    dJ3_dQ2_arr = zeros(size(J1_arr));
    dJ3_dQ3_arr = zeros(size(J1_arr));
    
    % Calculating derivatives 
    for iQ1 = 1:size(Q1_arr, 1)

        % Check if J1, J2 are orthogonal
        if (length(unique(J2_arr(iQ1,:,:))) ~= size(J2_arr, 2) || length(unique(J3_arr(iQ1,:,:))) ~= size(J3_arr, 3))
            error('J2, J3 are not orthogonal!')
        end
        
        Q2_2d = squeeze(Q2_arr(iQ1,:,:));
        Q3_2d = squeeze(Q3_arr(iQ1,:,:));

        J2_2d = squeeze(J2_arr(iQ1, :, :));
        J3_2d = squeeze(J3_arr(iQ1, :, :));

        % 2D transformation from !! ORTHOGONAL !! coordinates J to arbitrary coordinates Q
        [dJ2_dQ2_2d, dJ2_dQ3_2d, dJ3_dQ2_2d, dJ3_dQ3_2d, G_JQ_2d, dQ2_dJ2_2d, dQ2_dJ3_2d, dQ3_dJ2_2d, dQ3_dJ3_2d, G_QJ_2d] = coord_transform_ort(J2_2d, J3_2d, Q2_2d, Q3_2d);

        G_JQ_arr(iQ1,:,:) = G_JQ_2d;
        dQ2_dJ2_arr(iQ1, :, :) = dQ2_dJ2_2d;
        dQ2_dJ3_arr(iQ1, :, :) = dQ2_dJ3_2d;
        dQ3_dJ2_arr(iQ1, :, :) = dQ3_dJ2_2d;
        dQ3_dJ3_arr(iQ1, :, :) = dQ3_dJ3_2d;

        G_QJ_arr(iQ1,:,:) = G_QJ_2d;
        dJ2_dQ2_arr(iQ1, :, :) = dJ2_dQ2_2d;
        dJ2_dQ3_arr(iQ1, :, :) = dJ2_dQ3_2d;
        dJ3_dQ2_arr(iQ1, :, :) = dJ3_dQ2_2d;
        dJ3_dQ3_arr(iQ1, :, :) = dJ3_dQ3_2d;
        
    end

end
