function [DQ1Q1, DQ1Q2, DQ2Q2] = convert_diff_coeff(DJ1J1, DJ1J2, DJ2J2, dQ1_dJ1, dQ1_dJ2, dQ2_dJ1, dQ2_dJ2)
    %%
    % CONVERT_DIFF_COEFF recalculates diffusion coefficients into a 
    % diffirent coordinate system
    %
    % [DQ1Q1, DQ1Q2, DQ2Q2] = 
    % convert_diff_coeff(DJ1J1, DJ1J2, DJ2J2, dQ1_dJ1, dQ1_dJ2, dQ2_dJ1, dQ2_dJ2)
    % where (J1, J2) and (Q1, Q2) are arbitrary coordinate systems
    % converts DJ1J1, DJ1J2, and DJ2J2 into DQ1Q1, DQ1Q2, and DQ2Q2
    % using given devivatives dQ1/dJ1, dQ1/dJ2, dQ2/dJ1, dQ2/dJ2
    %
    %        D_Q1Q1 = (dQ1/dJ1 .* DJ1J1 .* dQ1/dJ1) + (dQ1/dJ1 .* DJ1J2 .* dQ1/dJ2) + (dQ1/dJ2 .* DJ2J1 .* dQ1/dJ1) + (dQ1/dJ2 .* DJ2J2 .* dQ1/dJ2);
    %        D_Q1Q2 = (dQ1/dJ1 .* DJ1J1 .* dQ2/dJ1) + (dQ1/dJ1 .* DJ1J2 .* dQ2/dJ2) + (dQ1/dJ2 .* DJ2J1 .* dQ2/dJ1) + (dQ1/dJ2 .* DJ2J2 .* dQ2/dJ2);
    %        D_Q2Q2 = (dQ2/dJ1 .* DJ1J1 .* dQ2/dJ1) + (dQ2/dJ1 .* DJ1J2 .* dQ2/dJ2) + (dQ2/dJ2 .* DJ2J1 .* dQ2/dJ1) + (dQ2/dJ2 .* DJ2J2 .* dQ2/dJ2);
    %
    % IF DJ1J1 HAS MORE THAN 2 DIMENTIONS, THE CONVERSION WILL BE PERFORMED 
    % FOR THE LAST TWO DIMENTIONS
    %
    % Example:
    %  [DQ1Q1, DQ1Q2, DQ2Q2] = 
    % convert_diff_coeff(DJ1J1, DJ1J2, DJ2J2, dQ1_dJ1, dQ1_dJ2, dQ2_dJ1, dQ2_dJ2)
    %  where DJ1J1 is a 4D array will convert 
    % (i, j, Q1, Q2) -> (i, j, Q2, Q3) for each i and j
    
    % Author: Dmitriy Subbotin
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    % Shift dimentions, movind the last two dimentions to me first
    nd = length(size(DJ1J1));
    if (nd > 2)
        DJ1J1 = shiftdim(DJ1J1, nd-2);
        DJ1J2 = shiftdim(DJ1J2, nd-2);
        DJ2J2 = shiftdim(DJ2J2, nd-2);
        dQ1_dJ1 = shiftdim(dQ1_dJ1, nd-2);
        dQ1_dJ2 = shiftdim(dQ1_dJ2, nd-2);
        dQ2_dJ1 = shiftdim(dQ2_dJ1, nd-2);
        dQ2_dJ2 = shiftdim(dQ2_dJ2, nd-2);
    end

    DQ1Q1 = zeros(size(DJ1J1));
    DQ1Q2 = zeros(size(DJ1J2));
    DQ2Q2 = zeros(size(DJ2J2));
    
    for ind = 1:size(DJ1J1(1,1,:), 3) % index through all dimentions after first two
    
        % go to 2D
        Dpp_Q_2d = squeeze(DJ1J1(:, :, ind));
        Dpa_Q_2d = squeeze(DJ1J2(:, :, ind));
        Daa_Q_2d = squeeze(DJ2J2(:, :, ind));
        
        dQ2_dJ2_2d = squeeze(dQ1_dJ1(:, :, ind));
        dQ2_dJ3_2d = squeeze(dQ1_dJ2(:, :, ind));
        dQ3_dJ2_2d = squeeze(dQ2_dJ1(:, :, ind));
        dQ3_dJ3_2d = squeeze(dQ2_dJ2(:, :, ind));

        % Converting diffusion coefficients to the new grid
        DQ2Q2_2d = (dQ2_dJ2_2d .* Dpp_Q_2d .* dQ2_dJ2_2d) + (dQ2_dJ2_2d .* Dpa_Q_2d .* dQ2_dJ3_2d) + (dQ2_dJ3_2d .* Dpa_Q_2d .* dQ2_dJ2_2d) + (dQ2_dJ3_2d .* Daa_Q_2d .* dQ2_dJ3_2d);
        DQ2Q3_2d = (dQ2_dJ2_2d .* Dpp_Q_2d .* dQ3_dJ2_2d) + (dQ2_dJ2_2d .* Dpa_Q_2d .* dQ3_dJ3_2d) + (dQ2_dJ3_2d .* Dpa_Q_2d .* dQ3_dJ2_2d) + (dQ2_dJ3_2d .* Daa_Q_2d .* dQ3_dJ3_2d);
        DQ3Q3_2d = (dQ3_dJ2_2d .* Dpp_Q_2d .* dQ3_dJ2_2d) + (dQ3_dJ2_2d .* Dpa_Q_2d .* dQ3_dJ3_2d) + (dQ3_dJ3_2d .* Dpa_Q_2d .* dQ3_dJ2_2d) + (dQ3_dJ3_2d .* Daa_Q_2d .* dQ3_dJ3_2d);

        % Save back to 3D
        DQ1Q1(:,:, ind) = DQ2Q2_2d;
        DQ1Q2(:,:, ind) = DQ2Q3_2d;
        DQ2Q2(:,:, ind) = DQ3Q3_2d;
    
    end
    
    % shift dimentions of the result back
    if (nd > 2)
        DQ1Q1 = shiftdim(DQ1Q1, 2);
        DQ1Q2 = shiftdim(DQ1Q2, 2);
        DQ2Q2 = shiftdim(DQ2Q2, 2);
    end


end
