function [dJ1_dQ1, dJ1_dQ2, dJ2_dQ1, dJ2_dQ2, G_JQ, dQ1_dJ1, dQ1_dJ2, dQ2_dJ1, dQ2_dJ2, G_QJ] = coord_transform_ort(J1, J2, Q1, Q2)
    %%
    % COORD_TRANSFORM_ORT transformation from ORTHOGONAL coordinates J to arbitrary coordinates Q
    %
    % [dJ1_dQ1, dJ1_dQ2, dJ2_dQ1, dJ2_dQ2, G_JQ, dQ1_dJ1, dQ1_dJ2, dQ2_dJ1, dQ2_dJ2, G_QJ] = 
    % coord_transform_ort(J1, J2, Q1, Q2) calculates derivatives dJ/dQ and 
    % dQ/dJ, Jacobian and inert Jacobian that can be used later for 
    % coordinates transformation
    %
    % If:
    % J - old coordinates
    % Q - new coordinates
    % i - grid indexes
    % Then:
    % G(J->Q) = d(J)/d(Q)
    % G(J->Q) = G(J->ij->Q) = G(J->i) * G(i->Q) = G(J->i) / G(Q->i) = (dJ/di) / (dQ/di)

    % Author: Dmitriy Subbotin
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits

    % Check if J1, J2 are orthogonal
    if (length(unique(J1)) ~= size(J1, 1) || length(unique(J2)) ~= size(J2, 2))
        error('J2, J3 are not orthogonal!')
    end

    n1 = size(J1, 1);
    n2 = size(J1, 2);

    % Right and left differences, for EACH coordinate in EACH direction. Half of them will be zeros IF coordinates lie along the grid lines.
    dJ1_1_R = zeros(n1, n2);
    dJ1_1_L = zeros(n1, n2);
    dJ2_2_R = zeros(n1, n2);
    dJ2_2_L = zeros(n1, n2);
    
    dQ1_1_R = zeros(n1, n2);
    dQ1_1_L = zeros(n1, n2);
    dQ2_1_R = zeros(n1, n2);
    dQ2_1_L = zeros(n1, n2);

    dQ1_2_R = zeros(n1, n2);
    dQ1_2_L = zeros(n1, n2);
    dQ2_2_R = zeros(n1, n2);
    dQ2_2_L = zeros(n1, n2);


    % Right and left differences. Some are undefined at corresponding left and right boundary grid points.
    dJ1_1_R(1:end-1, :) = J1(2:end, :) - J1(1:end-1, :);
    dJ2_2_R(:, 1:end-1) = J2(:, 2:end) - J2(:, 1:end-1);
    
    dJ1_1_L(2:end, :) = J1(2:end, :) - J1(1:end-1, :);
    dJ2_2_L(:, 2:end) = J2(:, 2:end) - J2(:, 1:end-1);
    
    dQ1_1_R(1:end-1, :) = Q1(2:end, :) - Q1(1:end-1, :);
    dQ2_1_R(1:end-1, :) = Q2(2:end, :) - Q2(1:end-1, :);
    dQ1_2_R(:, 1:end-1) = Q1(:, 2:end) - Q1(:, 1:end-1);
    dQ2_2_R(:, 1:end-1) = Q2(:, 2:end) - Q2(:, 1:end-1);

    dQ1_1_L(2:end, :) = Q1(2:end, :) - Q1(1:end-1, :);
    dQ2_1_L(2:end, :) = Q2(2:end, :) - Q2(1:end-1, :);
    dQ1_2_L(:, 2:end) = Q1(:, 2:end) - Q1(:, 1:end-1);
    dQ2_2_L(:, 2:end) = Q2(:, 2:end) - Q2(:, 1:end-1);

    % Approximate derivatives at boundaries
    if (false)
        % using 3 points
        dJ1_1_R(end, :) = 2*dJ1_1_R(end-1, :) - dJ1_1_R(end-2, :); %dJ1_1_R(end-1, :);
        dJ2_2_R(:, end) = 2*dJ2_2_R(:, end-1) - dJ2_2_R(:, end-2);%dJ2_2_R(:, end-1);

        dQ1_1_R(end, :) = 2*dQ1_1_R(end-1, :) - dQ1_1_R(end-2, :);%dQ1_1_R(end-1, :);
        dQ2_1_R(end, :) = 2*dQ2_1_R(end-1, :) - dQ2_1_R(end-2, :);%dQ2_1_R(end-1, :);
        dQ1_2_R(:, end) = 2*dQ1_2_R(:, end-1) - dQ1_2_R(:, end-2);%dQ1_2_R(:, end-1);
        dQ2_2_R(:, end) = 2*dQ2_2_R(:, end-1) - dQ2_2_R(:, end-2);%dQ2_2_R(:, end-1);

        dJ1_1_L(1, :) = 2*dJ1_1_L(2, :) - dJ1_1_L(3, :);%dJ1_1_L(2, :);
        dJ2_2_L(:, 1) = 2*dJ2_2_L(:, 2) - dJ2_2_L(:, 3);%dJ2_2_L(:, 2);

        dQ1_1_L(1, :) = 2*dQ1_1_L(2, :) - dQ1_1_L(3, :);%dQ1_1_L(2, :);
        dQ2_1_L(1, :) = 2*dQ2_1_L(2, :) - dQ2_1_L(3, :);%dQ2_1_L(2, :);
        dQ1_2_L(:, 1) = 2*dQ1_2_L(:, 2) - dQ1_2_L(:, 3);%dQ1_2_L(:, 2);
        dQ2_2_L(:, 1) = 2*dQ2_2_L(:, 2) - dQ2_2_L(:, 3);%dQ2_2_L(:, 2);
    else
        % using 2 points
        dJ1_1_R(end, :) = dJ1_1_R(end-1, :);
        dJ2_2_R(:, end) = dJ2_2_R(:, end-1);
        dJ1_1_L(1, :) = dJ1_1_L(2, :);
        dJ2_2_L(:, 1) = dJ2_2_L(:, 2);

        dQ1_1_R(end, :) = dQ1_1_R(end-1, :);
        dQ2_1_R(end, :) = dQ2_1_R(end-1, :);
        dQ1_2_R(:, end) = dQ1_2_R(:, end-1);
        dQ2_2_R(:, end) = dQ2_2_R(:, end-1);

        dQ1_1_L(1, :) = dQ1_1_L(2, :);
        dQ2_1_L(1, :) = dQ2_1_L(2, :);
        dQ1_2_L(:, 1) = dQ1_2_L(:, 2);
        dQ2_2_L(:, 1) = dQ2_2_L(:, 2);
    end

    % DERIVATIVES

    dQ1_dJ1_R = dQ1_1_R./dJ1_1_R;
    dQ2_dJ1_R = dQ2_1_R./dJ1_1_R;
    dQ1_dJ2_R = dQ1_2_R./dJ2_2_R;
    dQ2_dJ2_R = dQ2_2_R./dJ2_2_R;

    dQ1_dJ1_L = dQ1_1_L./dJ1_1_L;
    dQ2_dJ1_L = dQ2_1_L./dJ1_1_L;
    dQ1_dJ2_L = dQ1_2_L./dJ2_2_L;
    dQ2_dJ2_L = dQ2_2_L./dJ2_2_L;

    % "central" derivatives
    dQ1_dJ1 = (dQ1_dJ1_R + dQ1_dJ1_L)/2;
    dQ1_dJ2 = (dQ1_dJ2_R + dQ1_dJ2_L)/2;
    dQ2_dJ1 = (dQ2_dJ1_R + dQ2_dJ1_L)/2;
    dQ2_dJ2 = (dQ2_dJ2_R + dQ2_dJ2_L)/2;
    
    G_QJ = dQ1_dJ1 .* dQ2_dJ2 - dQ1_dJ2 .* dQ2_dJ1;
    
    %INVERSE DERIVATIVES
    
    % jacobian (at each point!), and left and right dJ/dQ derivatives
    dJ1_dQ1_L = zeros(n1, n2);
    dJ1_dQ2_L = zeros(n1, n2);
    dJ2_dQ1_L = zeros(n1, n2);
    dJ2_dQ2_L = zeros(n1, n2);
    dJ1_dQ1_R = zeros(n1, n2);
    dJ1_dQ2_R = zeros(n1, n2);
    dJ2_dQ1_R = zeros(n1, n2);
    dJ2_dQ2_R = zeros(n1, n2);
    for i=1:n1
        for j=1:n2
            % left
            %G_QJ_L = [[dQ1_dJ1_L(i, j), dQ1_dJ2_L(i, j)]', [dQ2_dJ1_L(i, j), dQ2_dJ2_L(i, j)]'];
            G_QJ_L = [[dQ1_dJ1_L(i, j), dQ2_dJ1_L(i, j)]', [dQ1_dJ2_L(i, j), dQ2_dJ2_L(i, j)]'];

            G_JQ_L = inv(G_QJ_L); % matrix division
            dJ1_dQ1_L(i, j) = G_JQ_L(1,1);
            dJ1_dQ2_L(i, j) = G_JQ_L(1,2);
            dJ2_dQ1_L(i, j) = G_JQ_L(2,1);
            dJ2_dQ2_L(i, j) = G_JQ_L(2,2);

            % right
            %G_QJ_R = [[dQ1_dJ1_R(i, j), dQ1_dJ2_R(i, j)]', [dQ2_dJ1_R(i, j), dQ2_dJ2_R(i, j)]'];
            G_QJ_R = [[dQ1_dJ1_R(i, j), dQ2_dJ1_R(i, j)]', [dQ1_dJ2_R(i, j), dQ2_dJ2_R(i, j)]'];

            G_JQ_R = inv(G_QJ_R); % matrix division
            dJ1_dQ1_R(i, j) = G_JQ_R(1,1);
            dJ1_dQ2_R(i, j) = G_JQ_R(1,2);
            dJ2_dQ1_R(i, j) = G_JQ_R(2,1);
            dJ2_dQ2_R(i, j) = G_JQ_R(2,2);        
        end
    end

    % "central" derivatives
    dJ1_dQ1 = (dJ1_dQ1_R + dJ1_dQ1_L)/2;
    dJ1_dQ2 = (dJ1_dQ2_R + dJ1_dQ2_L)/2;
    dJ2_dQ1 = (dJ2_dQ1_R + dJ2_dQ1_L)/2;
    dJ2_dQ2 = (dJ2_dQ2_R + dJ2_dQ2_L)/2;

    % Determinant of the Jacobians
    G_JQ = dJ1_dQ1 .* dJ2_dQ2 - dJ1_dQ2 .* dJ2_dQ1;
    
end
