function [Bf, Bmin_surface, maginput] = calc_B_field_Tsy(P_VERB, R_VERB, matlabd, Kp, kext, internal_field, SW_data, varargin)
    %%
    % CALC_B_FIELD_TSY calculates magnetic field using onera library
    %
    % [Bf, Bmin_surface, maginput] = calc_B_field_Tsy(P_VERB, R_VERB, matlabd, Kp, kext, internal_field, SW_data, varargin)
    %
    % Input:
    %   P_VERB - MLT in rad starting from zero at noon
    %   R_VERB - Radial distance in Re
    %   matlabd - date in days
    %   Kp - kp index
    %   kext - external field, possible values:
    %     0  = no external field
    %     1  = Mead & Fairfield [1975] (uses 0≤Kp≤9 - Valid for rGEO≤17. Re)
    %     2  = Tsyganenko short [1987] (uses 0≤Kp≤9 - Valid for rGEO≤30. Re)
    %     3  = Tsyganenko long [1987] (uses 0≤Kp≤9 - Valid for rGEO≤70. Re)
    %     4  = Tsyganenko [1989c] (uses 0≤Kp≤9 - Valid for rGEO≤70. Re)
    %     5  = Olson & Pfitzer quiet [1977] (default - Valid for rGEO≤15. Re)
    %     6  = Olson & Pfitzer dynamic [1988] (uses 5.≤dens≤50., 300.≤velo≤500., -100.≤Dst≤20. - Valid for rGEO≤60. Re)
    %     7  = Tsyganenko [1996] (uses -100.≤Dst (nT)≤20., 0.5≤Pdyn (nPa)≤10., |ByIMF| (nT)≤10., |BzIMF| (nT)≤10. - Valid for rGEO≤40. Re)
    %     8  = Ostapenko & Maltsev [1997] (uses dst,Pdyn,BzIMF, Kp)
    %     9  = Tsyganenko [2001] (uses -50.≤Dst (nT)≤20., 0.5≤Pdyn (nPa)≤5., |ByIMF| (nT)≤5., |BzIMF| (nT)≤5., 0.≤G1≤10., 0.≤G2≤10. - Valid for xGSM≥-15. Re)
    %     10 = Tsyganenko [2001] storm  (uses Dst, Pdyn, ByIMF, BzIMF, G2, G3 - there is no upper or lower limit for those inputs - Valid for xGSM≥-15. Re)
    %     11 = Tsyganenko [2004] storm  (uses Dst, Pdyn, ByIMF, BzIMF, W1, W2, W3, W4, W5, W6 - there is no upper or lower limit for those inputs - Valid for xGSM≥-15. Re)
    %     12 = Alexeev [2000] - also known as Paraboloid model - Submitted to ISO  (uses Dens, velo, Dst, BzIMF, AL)
    %
    %   internal_field - internal magnetic field, possible vaues:
    %     0 = IGRF
    %     1 = Eccentric tilted dipole
    %     2 = Jensen&Cain 1960
    %     3 = GSFC 12/66 updated to 1970
    %     4 = User own magnetic field. The library then called a routine which has to be written by the user myOwnMagField(xGEO,Bxint) where inputs are xGEO a double array of 3 elements (x,y,z) containing geographic cartesian coordinates in Re and outputs are Bxint a double array of 3 elements (Bx,By,Bz) containing magnetic field components in geographic cartesian coordinates in nT
    %     5 = Centered dipole
    %
    %   SW_data - solar wind data for T96, a structure with following arrays:
    %     SW_data.SW_N - Solar Wind density (cm-3)
    %     SW_data.SW_V - Solar Wind velocity (km/s)
    %     SW_data.Dst - Dst index (nT)
    %     SW_data.IMF_By - GSM y component of IMF mag. field (nT)
    %     SW_data.IMF_Bz - GSM z component of IMF mag. field (nT)
    %     SW_data.IMF_Dp - Dynamic Pressure, nPa
    %
    %  Optional parameter pairs - vardata:
    %     'use_Bmin_surface',  [true]|false - use min(B) surface, instead of simple Z=0 surface
    %     'fix_magnetic_field_distance',  true|[false] - 'Fix' magnetic field - it shouldn't increase with distance
    %     'interpolate_in',  'X'|'P' - interpolate in X,Y coordinates or P,R coordinates
    
    % Output:
    %%  Bf.z - magnetic field values in Tesla at the magnetic equator
    %   Bmin_surface - magnetic equator, has both GSM and GEO coordinates
    %   maginput - some Tsyganenko field input values (actually Onera input values, check Onera help)
    %

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    use_Bmin_surface = true;
    fix_magnetic_field_distance = false;
    interpolate_in_X = true;    
    
    it = 1;
    while it <= length(varargin)
        if strcmp(varargin(it), 'use_Bmin_surface')
            use_Bmin_surface = varargin{it+1};
            it = it + 1;
        elseif strcmp(varargin(it), 'fix_magnetic_field_distance')
            fix_magnetic_field_distance = varargin{it+1};
            it = it + 1;
        elseif strcmp(varargin(it), 'interpolate_in')
            interpolate_in = varargin{it+1};
            it = it + 1;
            if (interpolate_in == 'X')
                interpolate_in_X = true;
            else 
                interpolate_in_X = false;
            end
        elseif strcmp(varargin(it), 'dont_fix_magnetic_field_distance')
            fix_magnetic_field_distance = varargin{it+1};
            it = it + 1;
        else
    	    error(['Unknown parameter: ', varargin(it)]);
        end
        it = it + 1;
    end

    if (true)
        Kp10 = Kp*10;

        options = [0,0,0,0,internal_field];

        % sysaxes - coordinate system for oneray
        %0: GDZ (alti, lati, East longi - km,deg.,deg)
        %1: GEO (cartesian) - Re
        %2: GSM (cartesian) - Re
        %3: GSE (cartesian) - Re
        %4: SM (cartesian) - Re
        %5: GEI (cartesian) - Re
        %6: MAG (cartesian) - Re
        %7: SPH (geo in spherical) - (radial distance, lati, East longi - Re, deg., deg.)
        %8: RLL  (radial distance, lati, East longi - Re, deg., deg. - prefered to 7)
        sysaxes = 1;

        if (sysaxes == 1)
            sysaxis_name = 'geo'; % Z=0 is similar to Centered Dipole eq-plane
        elseif (sysaxes == 2)
            sysaxis_name = 'gsm'; %
        elseif (sysaxes == 4)
            sysaxis_name = 'sm'; % Z=0 is similar to Tilted Dipole eq-plane
        else
            error('Unsupported sysaxes')
        end

        %% Create grid for Onera
        onera_R_size = 100;
        onera_P_size = 100;
        onera_R_max = max(R_VERB.arr(:)) + 5;
        P1d = [0: 2*pi/onera_P_size: 2*pi];
        R1d = [1:   20/onera_R_size:   onera_R_max];
        [R_Bf_sysaxis, P_Bf_sysaxis] = meshgrid(R1d, P1d);

        % convert to X,Y,Z
        [X_sysaxes, Y_sysaxes] = pol2cart(P_Bf_sysaxis, R_Bf_sysaxis);
        Z_sysaxes = -0.0*ones(size(X_sysaxes));

        if (kext == 7)
            SW_N = SW_data.SW_N; % : Solar Wind density (cm-3)
            SW_V = SW_data.SW_V; % : Solar Wind velocity (km/s)
            Dst = SW_data.Dst; % : Dst index (nT)
            IMF_By = SW_data.IMF_By; % : GSM y component of IMF mag. field (nT)
            IMF_Bz = SW_data.IMF_Bz; % : GSM z component of IMF mag. field (nT)

            try
                SW_P_dyn = SW_data.IMF_Dp;
            catch
                SW_P_dyn = 1.67e-6 .* SW_V .* SW_V .* SW_N; % : Solar Wind dynamic pressure (nPa)
            end
        else
            SW_N = NaN; % : Solar Wind density (cm-3)
            SW_V = NaN; % : Solar Wind velocity (km/s)
            Dst = NaN; % : Dst index (nT)
            IMF_By = NaN; % : GSM y component of IMF mag. field (nT)
            IMF_Bz = NaN; % : GSM z component of IMF mag. field (nT)
            SW_P_dyn = NaN;
        end


        %! 'fix' bad parameters
        %! Tsyganenko [1996] (uses -100.≤Dst (nT)≤20., 0.5≤Pdyn (nPa)≤10., |ByIMF| (nT)≤10., |BzIMF| (nT)≤10. - Valid for rGEO≤40. Re)
        if (Dst > 20)       Dst = 20;       end;
        if (Dst < -100)     Dst = -100;     end;
        if (SW_P_dyn > 10)  SW_P_dyn = 10;  end;
        if (SW_P_dyn < 0.5) SW_P_dyn = 0.5; end;
        if (IMF_By > 10)    IMF_By = 10;    end;
        if (IMF_By < -10)   IMF_By = -10;   end;
        if (IMF_Bz > 10)    IMF_Bz = 10;    end;
        if (IMF_Bz < -10)   IMF_Bz = -10;   end;

        maginput = [Kp10, Dst, SW_N, SW_V, SW_P_dyn, IMF_Bz, IMF_By,  0,0,0,  0,0,0,0,0,0,0,  0,0,0,0,0,0,0,0];
        %maginput = [Kp10, Dst, dens, velo, Pdyn, ByIMF, BzIMF, G1, G2, G3, W1, W2, W3, W4, W5, W6, AL, fugure_use];

        %% Calculate B at min-B or at Z=0 surfaces
        if (use_Bmin_surface)
            % Use B-minimum for computations
            % Bf_bmin_onera in nT
            [Bf_bmin_onera.z, vec_bmin_geo] = onera_desp_lib_find_magequator(kext, options, sysaxes, matlabd, X_sysaxes(:), Y_sysaxes(:), Z_sysaxes(:), maginput);
            Bf_bmin_onera.units = 'nT';
        else
            % Use Z=0 for computation
            % Bf_bmin_onera in nT
            [B_geo, B_total] = onera_desp_lib_get_field(kext, options, sysaxes, matlabd, X_sysaxes(:), Y_sysaxes(:), Z_sysaxes(:), maginput);
            Bf_bmin_onera.z = B_total;%B_geo(:,3);
            Bf_bmin_onera.units = 'nT';
            vec_bmin_sysaxis = [X_sysaxes(:), Y_sysaxes(:), Z_sysaxes(:)];
            vec_bmin_geo = onera_desp_lib_rotate(vec_bmin_sysaxis, [sysaxis_name, '2geo'], matlabd);
        end
        Bf_bmin_onera.z = reshape(Bf_bmin_onera.z, size(Z_sysaxes)) * 1e-9; % to Tesla
        Bf_bmin_onera.units = 'Tesla';

        % calculate R and MLT for B-min surface
        R_bmin_local = sqrt(sum(vec_bmin_geo(:,:).^2, 2));
        R_bmin_local = reshape(R_bmin_local, size(Z_sysaxes));

        MLT = onera_desp_lib_get_mlt(matlabd, vec_bmin_geo);
        MLT = reshape(MLT, size(Z_sysaxes));

        % Calculate P and R from MLT and R_bmin
        % we need monotonic azimuthal angle
        P_bmin_local = mod(MLT-12, 24)/12*pi;

        % X and Y for interpolation
        % these are local coordinates on the Bmin-surface
        [X_bmin_local, Y_bmin_local] = pol2cart(P_bmin_local, R_bmin_local);
        
        X_bmin_geo = reshape(vec_bmin_geo(:,1), size(Z_sysaxes));
        Y_bmin_geo = reshape(vec_bmin_geo(:,2), size(Z_sysaxes));
        Z_bmin_geo = reshape(vec_bmin_geo(:,3), size(Z_sysaxes));

        vec_bmin_gsm = onera_desp_lib_rotate(vec_bmin_geo, 'geo2gsm', matlabd);
        X_bmin_gsm = reshape(vec_bmin_gsm(:,1), size(Z_sysaxes));
        Y_bmin_gsm = reshape(vec_bmin_gsm(:,2), size(Z_sysaxes));
        Z_bmin_gsm = reshape(vec_bmin_gsm(:,3), size(Z_sysaxes));

        if (~interpolate_in_X)
            %% Rotate the surface, so P(1) = 0, e.g. so P starts from noon
            % TODO: modify for 4D
            [tmp, ind0] = min(P_bmin_local(:,1)); % find the next index after P=0
            %index_shift = [ind0-1,ind0:length(P1d),2:ind0-1]; % P(ind) == P(1), need to skip one of them
            index_shift = [ind0:length(P1d),2:ind0];
            P_bmin_local = P_bmin_local(index_shift, :); % loop it, and add last point to the begining as well
            %P_bmin(1, :) = P_bmin(1, :)-2*pi; % substract 2pi from the first point
            P_bmin_local(end, :) = P_bmin_local(end, :)+2*pi; % substract 2pi from the first point

            MLT = MLT(index_shift, :);
            R_bmin_local = R_bmin_local(index_shift, :);
            Bf_bmin_onera.z = Bf_bmin_onera.z(index_shift, :);

            X_bmin_geo = X_bmin_geo(index_shift, :);
            Y_bmin_geo = Y_bmin_geo(index_shift, :);
            Z_bmin_geo = Z_bmin_geo(index_shift, :);

            X_bmin_gsm = X_bmin_gsm(index_shift, :);
            Y_bmin_gsm = Y_bmin_gsm(index_shift, :);
            Z_bmin_gsm = Z_bmin_gsm(index_shift, :);
        end

        %% Smooth B-field - optional
        if (false)
            Bf_bmin_onera.z = smooth2(Bf_bmin_onera.z, 1, 20);
        end

        %% Interpolate Bf_bmin_onera on VERB grid

        % There are two options - to interpolate on (P,R) or on (X,Y)
        interpolatin_method = 'linear'; % 'cubic' is betterm but doesn't work on RBM16 for some reason :(
        if (interpolate_in_X)
            [X_VERB.arr, Y_VERB.arr] = pol2cart(P_VERB.arr(:,:,1,1), R_VERB.arr(:,:,1,1));

            % Far in the tail X_bmin_local, Y_bmin_local are undefined (B-min was not found)
            X_bmin_local(isnan(X_bmin_local)) = 0;
            Y_bmin_local(isnan(Y_bmin_local)) = 0;
            
            warning('off', 'MATLAB:griddata:DuplicateDataPoints');
            Bf_bmin_onera_interp.z = griddata(X_bmin_local, Y_bmin_local, Bf_bmin_onera.z, X_VERB.arr, Y_VERB.arr, interpolatin_method);
            Bf_bmin_onera_interp.units = Bf_bmin_onera.units;

            %X_VERB_gsm.arr = griddata(X_bmin_local, Y_bmin_local, X_bmin_gsm, X_VERB.arr, Y_VERB.arr, interpolatin_method);
            %Y_VERB_gsm.arr = griddata(X_bmin_local, Y_bmin_local, Y_bmin_gsm, X_VERB.arr, Y_VERB.arr, interpolatin_method);
            %Z_VERB_gsm.arr = griddata(X_bmin_local, Y_bmin_local, Z_bmin_gsm, X_VERB.arr, Y_VERB.arr, interpolatin_method);

            X_VERB_geo.arr = griddata(X_bmin_local, Y_bmin_local, X_bmin_geo, X_VERB.arr, Y_VERB.arr, interpolatin_method);
            Y_VERB_geo.arr = griddata(X_bmin_local, Y_bmin_local, Y_bmin_geo, X_VERB.arr, Y_VERB.arr, interpolatin_method);
            Z_VERB_geo.arr = griddata(X_bmin_local, Y_bmin_local, Z_bmin_geo, X_VERB.arr, Y_VERB.arr, interpolatin_method);
            warning('on', 'MATLAB:griddata:DuplicateDataPoints');

        else
            % Far in the tail P_bmin_local, R_bmin_local are undefined (B-min was not found)
            P_bmin_local(isnan(P_bmin_local)) = 0;
            R_bmin_local(isnan(R_bmin_local)) = 0;

            if (min(P_bmin_local(:,1)) ~= P_bmin_local(1,1))
                error('P_bmin grid should start from minimum P')
            end
            % Sometimes P_bmin(1) > P_VERB(2) :(
            Bf_bmin_onera_interp.z = griddata(P_bmin_local, R_bmin_local, Bf_bmin_onera.z, squeeze(P_VERB.arr(:,:,1,1)), squeeze(R_VERB.arr(:,:,1,1)), interpolatin_method);
            Bf_bmin_onera_interp.z(1, :) = Bf_bmin_onera_interp.z(end, :);
            Bf_bmin_onera_interp.units = Bf_bmin_onera.units;

            % coordinates, interpolating into VERB code grid
            %X_VERB_gsm.arr = griddata(P_bmin_local, R_bmin_local, X_bmin_gsm, squeeze(P_VERB.arr(:,:,1,1)), squeeze(R_VERB.arr(:,:,1,1)), interpolatin_method);
            %X_VERB_gsm.arr(1, :) = X_VERB_gsm.arr(end, :);
            %Y_VERB_gsm.arr = griddata(P_bmin_local, R_bmin_local, Y_bmin_gsm, squeeze(P_VERB.arr(:,:,1,1)), squeeze(R_VERB.arr(:,:,1,1)), interpolatin_method);
            %Y_VERB_gsm.arr(1, :) = Y_VERB_gsm.arr(end, :);
            %Z_VERB_gsm.arr = griddata(P_bmin_local, R_bmin_local, Z_bmin_gsm, squeeze(P_VERB.arr(:,:,1,1)), squeeze(R_VERB.arr(:,:,1,1)), interpolatin_method);
            %Z_VERB_gsm.arr(1, :) = Z_VERB_gsm.arr(end, :);

            X_VERB_geo.arr = griddata(P_bmin_local, R_bmin_local, X_bmin_geo, squeeze(P_VERB.arr(:,:,1,1)), squeeze(R_VERB.arr(:,:,1,1)), interpolatin_method);
            X_VERB_geo.arr(1, :) = X_VERB_geo.arr(end, :);
            Y_VERB_geo.arr = griddata(P_bmin_local, R_bmin_local, Y_bmin_geo, squeeze(P_VERB.arr(:,:,1,1)), squeeze(R_VERB.arr(:,:,1,1)), interpolatin_method);
            Y_VERB_geo.arr(1, :) = Y_VERB_geo.arr(end, :);
            Z_VERB_geo.arr = griddata(P_bmin_local, R_bmin_local, Z_bmin_geo, squeeze(P_VERB.arr(:,:,1,1)), squeeze(R_VERB.arr(:,:,1,1)), interpolatin_method);
            Z_VERB_geo.arr(1, :) = Z_VERB_geo.arr(end, :);

        end

        Bf_bmin_onera_interp.z(Bf_bmin_onera_interp.z<=0) = NaN;
        %Bf_bmin_onera_interp.z(isnan(Bf_bmin_onera_interp.z)) = min(Bf_bmin_onera_interp.z(:));

        %Bmin_surface.X_VERB_gsm = X_VERB_gsm;
        %Bmin_surface.Y_VERB_gsm = Y_VERB_gsm;
        %Bmin_surface.Z_VERB_gsm = Z_VERB_gsm;
        Bmin_surface.X_VERB_geo = X_VERB_geo;
        Bmin_surface.Y_VERB_geo = Y_VERB_geo;
        Bmin_surface.Z_VERB_geo = Z_VERB_geo;

        % fill nans
        %for iP = 1:size(P.arr,1) % cause P(1) == P(end)
        %    notnan = ~isnan(Bf_bmin_onera_interp.z(iP, :));
        %    Bf_bmin_onera_interp.z(iP, :) = interp1(squeeze(R.arr(iP, notnan)), squeeze(Bf_bmin_onera_interp.z(iP, notnan)), squeeze(R.arr(iP, :)));
        %end

        % 'Fix' magnetic field - it shouldn't increase with distance
        Bf_bmin_onera_interp_fixed.z = Bf_bmin_onera_interp.z;
        if (fix_magnetic_field_distance)
            for iP = 1:length(P_VERB.arr(:,1,1,1))
                for iR = 2:length(R_VERB.arr(1,:,1,1))
                    if (Bf_bmin_onera_interp_fixed.z(iP,iR) > Bf_bmin_onera_interp_fixed.z(iP,iR-1))
                        Bf_bmin_onera_interp_fixed.z(iP,iR) = Bf_bmin_onera_interp_fixed.z(iP,iR-1)*0.999;
                    end
                end
            end
        end

        %% Smooth B-field - optional
        %Bf_bmin_onera_interp_fixed.z = smooth2(Bf_bmin_onera_interp_fixed.z, 1, 20);

        Bf = Bf_bmin_onera_interp_fixed.z;

    end
end
