function [Bounce_period_arr, K_invariant_CGS_arr] = calc_K_Onera_Tsy(Field_line_arr, kext, internal_field, sysaxes, Bmin_arr, Bmin_xGEO_arr, alpha_eq)
    %%
    % CALC_K_ONERA_TSY calculates bounce period and K invariant 
    % in a given magnetic field, specified as traced field lines array
    %
    % Input:
    %   Field_line_arr - array of traced field lines that contains:
    %            Field_line.R - radial distance in Re
    %            Field_line.lat - latitude in rad
    %            Field_line.phi - MLT in rad starting from zero at noon
    %            Field_line.B - magnetic field in Tesla
    %
    %  kext - external magnetic field (check Onera help)
    %  internal_field - internal field (check Onera help)
    %  sysaxes - coordinate system  (check Onera help)
    %  matlabd - date in days (check Onera help)
    %%  Bmin_arr - equatorial magnetic field in Tesla!!!!
    %  Bmin_xGEO_arr - Equatorial surface in GEO coordinates
    %  alpha_eq - equatorial pitch-angle in rad
    %
    % Output
    %   Bounce_period_arr - dimentionless bounce time
    %   K_invariant_CGS_arr - invariant K in G^0.5 Re

    % Author: Ksenia Orlova
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
        
    % Check that B is in Tesla :(
    if (max(Bmin_arr(:)) > 1e-4)
       error('B should be in Tesla!'); 
    end
    
    show_warnings = false;
    interpolate_over_NaN = false;

    if (kext ~= 4 && kext ~= 0 && kext ~= 7)
       error(['Unimlemented external field: ', num2str(kext)]); 
    end
    
    options = [0, 0, 0, 0, internal_field];
    if (options(5) ~= 5)
       error(['You need to modify adding points inside the earth to use internal magnetic field other than central dipolar.']); 
       % Specifically, when you add points inside the Earth you assume that Z=0 is magnetic equator
    end
    
    if (sysaxes ~= 1)
       error(['Only GEO sysaxis waere tested, however it should be easy to modify for other syaxis.']); 
    end
    
    Re=6378*1e3;% Earth radius, meters

    Bounce_period_arr = NaN*zeros(length(alpha_eq), length(Bmin_arr)); 
    K_invariant_CGS_arr = NaN*zeros(length(alpha_eq), length(Bmin_arr));

    length_Bmin_arr = length(Bmin_arr);
    for x_ind = 1:length_Bmin_arr
        %fprintf(1, '\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b%5d / %5d: (%2.1f, %2.1f, %2.1f)', x_ind, length_Bmin_arr, Bmin_xGEO_arr(x_ind,1), Bmin_xGEO_arr(x_ind,2), Bmin_xGEO_arr(x_ind,3))
        
        Bmin = Bmin_arr(x_ind) * 1e9; % Tesla to NanoTesla
        Bmin_xGEO = Bmin_xGEO_arr(x_ind,:);

        if (max(isnan(Bmin_xGEO)))
            %error(['NaN coordinates ', num2str(x_ind)]);
            if (show_warnings) warning(['NaN coordinates ', num2str(x_ind)]); end;
            continue;
        end                
        
        % Load Field_line here
        Field_line = Field_line_arr(x_ind);
        try
            if (isnan(Field_line))
                if (show_warnings) warning(['NaN field line: ', num2str(Bmin_xGEO)]); end;
                continue
            end
        end
        if (length(Field_line.R) < 2)
            if (show_warnings) warning(['Tracing failed: ', num2str(Bmin_xGEO)]); end;
            continue
        end

        % fix phi so it doesn't jump
        for i = 2:length(Field_line.phi)
            if (Field_line.phi(i) > Field_line.phi(i-1) + 360/2)
                Field_line.phi(i) = Field_line.phi(i) - 360;
            elseif (Field_line.phi(i) < Field_line.phi(i-1) - 360/2)
                Field_line.phi(i) = Field_line.phi(i) + 360;
            end
        end
        
        %% Calculade s and ds
        if (true)    
            [eq_B, eq_idx] = min(abs(Field_line.B));
            Field_line.s = zeros(size(Field_line.R));

            dlat = zeros(size(Field_line.R));
            dr   = zeros(size(Field_line.R));
            dphi = zeros(size(Field_line.R));
            ds   = zeros(size(Field_line.R));
            
            dlat(2:end-1)    = (Field_line.lat(3:end) - Field_line.lat(1:end-2))/2;
            dr(2:end-1)      = (Field_line.R(3:end)   - Field_line.R(1:end-2))/2;
            dphi(2:end-1)    = (Field_line.phi(3:end) - Field_line.phi(1:end-2))/2;                            
            
            ds(2:end-1)      = sqrt(dr(2:end-1).^2 + Field_line.R(2:end-1).^2 .* (dlat(2:end-1) .* pi./180).^2 + ...
                (Field_line.R(2:end-1) .* cos(Field_line.lat(2:end-1).*pi./180) .* dphi(2:end-1) .* pi./180).^2);

            for i = 2:length(Field_line.s)-1
                Field_line.s(i) = Field_line.s(i-1) + ds(i);
            end
            
            Field_line.s = Field_line.s - Field_line.s(eq_idx);

            Field_line.s(1) = interp1(Field_line.lat(2:end-1), Field_line.s(2:end-1), Field_line.lat(1), 'spline', 'extrap');
            Field_line.s(end) = interp1(Field_line.lat(2:end-1), Field_line.s(2:end-1), Field_line.lat(end), 'spline', 'extrap');
        else
            [eq_B, eq_idx] = min(abs(Field_line.B));
            Field_line.s = zeros(size(Field_line.R));
            
            for i = 2:length(Field_line.s)-1
                dlat    = (Field_line.lat(i+1) - Field_line.lat(i-1))/2;
                dr      = (Field_line.R(i+1)   - Field_line.R(i-1))/2;
                dphi    = (Field_line.phi(i+1) - Field_line.phi(i-1))/2;                            

                ds      = sqrt(dr.^2 + Field_line.R(i).^2 * (dlat * pi/180)^2 + (Field_line.R(i) * cos(Field_line.lat(i)*pi/180) * dphi * pi/180)^2);

                Field_line.s(i) = Field_line.s(i-1) + ds;
            end
            
            Field_line.s = Field_line.s - Field_line.s(eq_idx);

            Field_line.s(1) = interp1(Field_line.lat(2:end-1), Field_line.s(2:end-1), Field_line.lat(1), 'spline', 'extrap');
            Field_line.s(end) = interp1(Field_line.lat(2:end-1), Field_line.s(2:end-1), Field_line.lat(end), 'spline', 'extrap');

            if (false)
                %%
                figure
                plot(Field_line.lat, Field_line.s);
            end
            
        end        
        
        Bounce_period = zeros(size(alpha_eq));
        K_invariant_CGS = zeros(size(alpha_eq));
        
        B_m = eq_B./sin(alpha_eq).^2; % Magnetic field at mirror point in T
        interp_method = 'pchip';
        s_m_north_arr = interp1(Field_line.B(1:eq_idx), Field_line.s(1:eq_idx), B_m, interp_method);
        s_m_south_arr = interp1(Field_line.B(eq_idx:end), Field_line.s(eq_idx:end), B_m, interp_method);
        
        for ia = 1:length(alpha_eq)

            [eq_B, eq_idx] = min(abs(Field_line.B));
            B_m = eq_B./sin(alpha_eq(ia)).^2; % Magnetic field at mirror point in T
                        
            % eq_idx is new
            [eq_B, eq_idx] = min(abs(Field_line.B));
            
            if (alpha_eq(ia) > pi/2 + 1e-4)
                error(['Pitch-angle > 90 deg: alpha = ', num2str(alpha_eq(ia)*180/pi)]);
            end

            if (isnan(alpha_eq(ia)))
                %['WARING: Pitch-angle(', num2str(ia), ') is NaN; coords: ', num2str(Bmin_xGEO_arr)]
                if (show_warnings) warning(['Pitch-angle(', num2str(ia), ') is NaN; coords: ', num2str(Bmin_xGEO_arr)]); end;
                Bounce_period(ia) = NaN;
                K_invariant_CGS(ia) = NaN;
                continue
            end

            %if (abs(alpha_eq(ia) - pi/2) < 1e-4)
            %if (alpha_eq(ia) >= 1.5597)
            %alpha_tmp_max = 1.5517;
            alpha_tmp_max = 87.0 * pi/180;
            if (alpha_eq(ia) > alpha_tmp_max)
                % put NaN here, cause calculation is incorrect anyway
                % we'll interpolato to these values later
                Bounce_period(ia) = NaN;
                K_invariant_CGS(ia) = NaN;
                continue                
            end            
            
            
            % B increases realy fast toward the earth,
            % Let's include only points we need to the interpolation
            idx = find(Field_line.B < B_m);
            if (length(idx) == 0)
                idx = eq_idx;
            end
            limit_north_idx = max(min(idx)-2, 1);
            limit_south_idx = min(max(idx)+2,length(Field_line.B));

            %limit_north_idx = 1;
            %limit_south_idx = length(Field_line.B);
            
            %lat_m_north = fzero(@(x)func_tmp(x, alpha_eq(ia), Field_line), [Field_line.lat(eq_idx) Field_line.lat(1)]); % Mirror point 2, degree 
            %lat_m_south = fzero(@(x)func_tmp(x, alpha_eq(ia), Field_line), [Field_line.lat(end) Field_line.lat(eq_idx)]); % Mirror point 1, degree 
            
            %interp_method = 'cubic'; % Cubic seems to be woring better here and more often
            interp_method = 'pchip'; % Matlab said to change cubic to pchip :-/
            
            %lat_m_north = interp1(Field_line.B(limit_north_idx:eq_idx), Field_line.lat(limit_north_idx:eq_idx), B_m, interp_method);
            %lat_m_south = interp1(Field_line.B(eq_idx:limit_south_idx), Field_line.lat(eq_idx:limit_south_idx), B_m, interp_method);

            %if (lat_m_north<=lat_m_south)
            %    error(['Mirror latitude error. Alpha: ', num2str(alpha_eq(ia)*180/pi), ', GEO: ', num2str(Bmin_xGEO), ', lat: ', num2str(lat_m_north), ', ', num2str(lat_m_south)])
            %end
            
            %% !!!
            %s_m_north = interp1(Field_line.B(limit_north_idx:eq_idx), Field_line.s(limit_north_idx:eq_idx), B_m, interp_method);
            %s_m_south = interp1(Field_line.B(eq_idx:limit_south_idx), Field_line.s(eq_idx:limit_south_idx), B_m, interp_method);
            s_m_north = s_m_north_arr(ia);
            s_m_south = s_m_south_arr(ia);
            
            %idx = find(Field_line.lat <= lat_m_north & Field_line.lat >= lat_m_south);
            idx = find(Field_line.B < B_m); % we add B_m later
            north_idx = idx(idx <= eq_idx);
            south_idx = idx(idx >= eq_idx);

            north_idx = north_idx(end:-1:1); % reverse order, so equator is the first point        

            north_add = min(north_idx)-1;
            south_add = max(south_idx)+1;
            
            Bounce_period2 = quad_ba_peiod4( ...
                [Field_line.s([eq_idx+1, north_idx]),   s_m_north,      Field_line.s(north_add)], ...
                [Field_line.B([eq_idx+1, north_idx]),   B_m,            Field_line.B(north_add)], alpha_eq(ia)); % Nothern hemisphere 
            Bounce_period1 = quad_ba_peiod4( ...
                [Field_line.s([eq_idx-1, south_idx]),   s_m_south,      Field_line.s(south_add)], ...
                [Field_line.B([eq_idx-1, south_idx]),   B_m,            Field_line.B(south_add)], alpha_eq(ia)); % Nothern hemisphere         
            Bounce_period(ia)=(Bounce_period1+Bounce_period2)*4; % It is dimensionless bounce period. To get dimension you should multiply it by Re and divide by velocity, = 4 L T(y) (eq 11a, Orlova 2011)

            % Calculation of the K invariant 
            K_invariant2 = quad_K4( ...
                [Field_line.s([eq_idx+1, north_idx]),   s_m_north], ...
                [Field_line.B([eq_idx+1, north_idx]),   B_m]); % Nothern hemisphere 
            K_invariant1 = quad_K4( ... 
                [Field_line.s([eq_idx-1, south_idx]), s_m_south], ...
                [Field_line.B([eq_idx-1, south_idx]),   B_m]); % Nothern hemisphere         
            K_invariant = (K_invariant2 + K_invariant1)*Re; % In SI units (B is taken in T, Re in meters)
            K_invariant_CGS(ia) = (K_invariant2 + K_invariant1)*100; % Units G^0.5*Re

        end
        
        %interpolate over NaNs
        if (interpolate_over_NaN)
            
                % equatorial pitch-angle
                d2B_ds2 = (Field_line.B(eq_idx+1) - 2*Field_line.B(eq_idx) + Field_line.B(eq_idx-1))/((Field_line.s(eq_idx+1) - Field_line.s(eq_idx-1))/2)^2;
                %d2B_ds2 = 0.5*1/(Field_line.s(eq_idx+1) - Field_line.s(eq_idx)) * ( (Field_line.B(eq_idx+1) - Field_line.B(eq_idx))/(Field_line.s(eq_idx+1) - Field_line.s(eq_idx)) - (Field_line.B(eq_idx) - Field_line.B(eq_idx-1))/(Field_line.s(eq_idx) - Field_line.s(eq_idx-1)) ) + ...
                %          0.5*1/(Field_line.s(eq_idx) - Field_line.s(eq_idx-1)) * ( (Field_line.B(eq_idx+1) - Field_line.B(eq_idx))/(Field_line.s(eq_idx+1) - Field_line.s(eq_idx)) - (Field_line.B(eq_idx) - Field_line.B(eq_idx-1))/(Field_line.s(eq_idx) - Field_line.s(eq_idx-1)) );
                %Bt_eq = 2*pi / Re * sqrt(Field_line.B(eq_idx)/(0.5 * d2B_ds2));
                Bt_eq = 2*pi * sqrt(Field_line.B(eq_idx)/(0.5 * d2B_ds2));

                K_eq_CGS = 0;                
                
                % interpolate...
                
                nan_idx = isnan(Bounce_period);
                not_nan = ~nan_idx;
                if (length(find(not_nan)) > 1)
                    Bounce_period(nan_idx) = interp1([alpha_eq(not_nan), pi/2], [Bounce_period(not_nan), Bt_eq], alpha_eq(nan_idx), 'PCHIP', 'extrap');
                    %Bounce_period(nan_idx) = interp1([alpha_eq(not_nan), pi/2], [Bounce_period(not_nan), Bt_eq], alpha_eq(nan_idx), 'linear', 'extrap');
                else
                    Bounce_period(nan_idx) = Bt_eq;
                end

                nan_idx = isnan(K_invariant_CGS);
                not_nan = ~nan_idx;
                if (length(find(not_nan)) > 1)
                    K_invariant_CGS(nan_idx) = interp1([alpha_eq(not_nan), pi/2], [K_invariant_CGS(not_nan), 0], alpha_eq(nan_idx), 'PCHIP', 'extrap');
                    %K_invariant_CGS(nan_idx) = interp1([alpha_eq(not_nan), pi/2], [K_invariant_CGS(not_nan), 0], alpha_eq(nan_idx), 'linear', 'extrap');
                else
                    K_invariant_CGS(nan_idx) = K_eq_CGS;
                end
                
        end

        Bounce_period_arr(:, x_ind) = Bounce_period; 
        K_invariant_CGS_arr(:, x_ind) = K_invariant_CGS;
    end
    

    
end



% Eq idx should be == 2
% Bm idx should be == end-1
function S = quad_ba_peiod4(s, B_field, alpha_eq)

    %[B_m, idx_Bm] = max(B_field);
    %[eq_B, eq_idx] = min(B_field);
    %% !!!!!!!!!!!!!!!
    idx_Bm = length(B_field)-1;
    B_m = B_field(idx_Bm);

    %eq_idx = 2;
    %eq_B = B_field(eq_idx);
    [eq_B, eq_idx] = min(B_field);
    
    if (eq_idx ~= 2)
        figure
        plot(lat, B_field, '-*');
        
        error(['Something is wrong: ', num2str(eq_idx)])
    end

    if (abs(eq_B/(sin(alpha_eq)^2) - B_m) > 1e-9)
        error('Something is wrong with B-mitrror'); 
    end

    % coordinate change
    %psi = zeros(size(B_field));
    psi = asin(sqrt(1-B_field./B_m)/cos(alpha_eq));
    psi(eq_idx)=pi/2;
    psi(idx_Bm)=0;
    
    %F = zeros(1, idx_Bm);
    %F1 = NaN*zeros(1, idx_Bm);
    %ds = NaN*zeros(1, idx_Bm);
    %dB = NaN*zeros(1, idx_Bm);
    
    % for i between eq_idx and idx_Bm
    ds(eq_idx:idx_Bm) = abs(s(eq_idx+1:idx_Bm+1) - s(eq_idx-1:idx_Bm-1));
    dB(eq_idx:idx_Bm) = B_field(eq_idx+1:idx_Bm+1) - B_field(eq_idx-1:idx_Bm-1);
    F1(eq_idx:idx_Bm) = ds(eq_idx:idx_Bm)./dB(eq_idx:idx_Bm).*cos(psi(eq_idx:idx_Bm));
    F(eq_idx:idx_Bm) = -F1(eq_idx:idx_Bm).*(psi(eq_idx+1:idx_Bm+1) - psi(eq_idx-1:idx_Bm-1));

    % special point i=eq_idx
    ds(eq_idx) = abs(s(eq_idx+1) - s(eq_idx));
    dpsi    = psi(eq_idx+1) - psi(eq_idx);
    F1(eq_idx) = ds(eq_idx)./dpsi * tan(alpha_eq)^2/2/eq_B; % analytic integration at equator
    F(eq_idx) = -F1(eq_idx)*(psi(eq_idx+1) - psi(eq_idx));

    % special point i=idx_Bm
    F(idx_Bm) = -F1(idx_Bm)*(psi(idx_Bm) - psi(idx_Bm-1));
    
    S = sum(F(eq_idx:idx_Bm))/2 * cos(alpha_eq) * B_m;
end    

function S = quad_K4(s, B_field)

    N_int = length(B_field);
    B_m = max(B_field);
    [eq_B, eq_idx] = min(B_field);
    
    if (eq_idx ~= 2)
        error('Something is wrong')
    end
    
    %%
    dBm = sqrt(B_m - B_field);
    
    F = zeros(1, N_int);
    %F(eq_idx+1:N_int-1) = dBm(eq_idx+1:N_int-1).*(abs(s(eq_idx+2:N_int) - s(eq_idx+1:N_int-1)) + abs(s(eq_idx+1:N_int-1) - s(eq_idx:N_int-2)))./2;
    F(eq_idx+1:N_int-1) = dBm(eq_idx+1:N_int-1).*(abs(s(eq_idx+2:N_int) - s(eq_idx:N_int-2)))./2;
    F(eq_idx) = dBm(eq_idx).*(abs(s(eq_idx+1) - s(eq_idx)))./2;
    F(N_int) = dBm(N_int).*(abs(s(N_int) - s(N_int-1)))./2;

    S = sum(F);

end    


% Eq idx should be == 2
% Bm idx should be == end-1
function S = quad_ba_peiod3(s, B_field, alpha_eq)

    %[B_m, idx_Bm] = max(B_field);
    %[eq_B, eq_idx] = min(B_field);
    %% !!!!!!!!!!!!!!!
    idx_Bm = length(B_field)-1;
    B_m = B_field(idx_Bm);

    %eq_idx = 2;
    %eq_B = B_field(eq_idx);
    [eq_B, eq_idx] = min(B_field);
    
    if (eq_idx ~= 2)
        figure
        plot(lat, B_field, '-*');
        
        error(['Something is wrong: ', num2str(eq_idx)])
    end

    if (abs(eq_B/(sin(alpha_eq)^2) - B_m) > 1e-9)
        error('Something is wrong with B-mitrror'); 
    end

    % coordinate change
    psi = zeros(size(B_field));
    psi = asin(sqrt(1-B_field./B_m)/cos(alpha_eq));
    psi(eq_idx)=pi/2;
    psi(idx_Bm)=0;
    
    S = 0;
    
    for i = eq_idx:idx_Bm

        if (i == eq_idx)
            % uncertainty 0/0 at lambda=0 (psi=pi/2) solved analytically            
            im = i; %i-1
            ip = i+1;

            ds      = abs(s(ip) - s(im)); %% abs is temorary for compatibility!!!!
            dpsi    = psi(ip) - psi(im); %% !!!!!!!! It's actually zero with central differences

            F1 = ds/dpsi * tan(alpha_eq)^2/2/eq_B; % analytic integration

            F = -F1*(psi(i+1) - psi(i));
            
        elseif (i == idx_Bm)

            im = i-1;
            ip = i+1;

            ds      = abs(s(im) - s(ip));
            dB      = B_field(ip) - B_field(im);

            F1 = ds/dB*cos(psi(i));

            F = -F1*(psi(i) - psi(i-1));
            
        else

            im = i-1;
            ip = i+1;

            ds      = abs(s(im) - s(ip));
            dB      = B_field(ip) - B_field(im);

            F1 = ds/dB*cos(psi(i));

            F = -F1*(psi(i+1) - psi(i-1));
        
        end

        S = S + F;

    end

    S = S/2 * cos(alpha_eq) * B_m;
end    

function S = quad_K3(s, B_field)

    N_int = length(B_field);
    B_m = max(B_field);
    [eq_B, eq_idx] = min(B_field);
    
    if (eq_idx ~= 2)
        error('Something is wrong')
    end

    S = 0;
    ds_prev = 0;
    % Trapezoidal rule of integration    
    for i = eq_idx:N_int
                
        if (i == N_int)
            ds = NaN;
        else
            ds = abs(s(i+1) - s(i));
        end
        
        dBm = sqrt(B_m - B_field(i));

        if (i == eq_idx)
            F = dBm*ds/2;
        elseif (i == N_int)
            F = dBm*ds_prev/2;
        else
            F = dBm*(ds + ds_prev)/2;                      
        end

        S = S + F;
        
        ds_prev = ds;
    end

end    

 
