function [al, Bounce_time_dimentionless] = calc_al_Tsy(K, kext, internal_field, matlabd, maginput, Bf, Bmin_surface, Field_line_arr)
    %%
    % CALC_AL_TSY calculates pitch-angle and dimentionless bounce time
    % bounce_time == Re/v * bounce_time_dimentionless 
    %
    % [al, Bounce_time_dimentionless] = calc_al_Tsy(K, kext, internal_field, matlabd, maginput, Bf, Bmin_surface, Field_line_arr)
    %
    % Input:
    %   V, K - invariants
    %   kext - external field for Tsyganenko model
    %   internal_field - internal field for Tsyganenko model
    %   matlabd - matlab day
    %   maginput - parameters for Tsyganenko model
    %%  Bf_bmin_onera_interp - Magnetic field, in Tesla
    %   Bmin_surface - magnetic equator
    %   Field_line_arr - traced magnetic field
    %
    % Output
    %   al - pitch-angle in rad
    %   Bounce_time_dimentionless - dimentionless bounce time

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plasma.Tsy.calc_K_Onera_Tsy

    % Check that B is in Tesla :(
    if (max(Bf.z(:)) > 1e-4)
       error('B should be in Tesla!'); 
    end
    
    options = [0, 0, 0, 0, internal_field];
    
    % if all pith-angles are 90deg
    if (unique(K.arr) == 0)

        % if all pith-angles are 90deg
        al.arr = pi/2 * ones(size(K.arr));
        
        % Calculating bounce time
        sysaxis_geo = 1;
        
        %Bmin should be in NanoTesla!!!
        %[Bounce_time_dimentionless_tmp, ~]  = calc_K_Onera_Tsy(Field_line_arr, kext, internal_field, sysaxis_geo, matlabd, maginput, Bf.z(:), [Bmin_surface.X_VERB_geo.arr(:), Bmin_surface.Y_VERB_geo.arr(:), Bmin_surface.Z_VERB_geo.arr(:)], 87*pi/180); 
        [Bounce_time_dimentionless_tmp, ~]  = calc_K_Onera_Tsy(Field_line_arr, kext, internal_field, sysaxis_geo, Bf.z(:), [Bmin_surface.X_VERB_geo.arr(:), Bmin_surface.Y_VERB_geo.arr(:), Bmin_surface.Z_VERB_geo.arr(:)], 87*pi/180); 

        Bounce_time_dimentionless_tmp = reshape(Bounce_time_dimentionless_tmp, size(Bf.z));

        % dementionless bounce time is energy independent, need to copy the same values into all energies
        for iV = 1:size(K.arr, 3)
            Bounce_time_dimentionless.arr(:, :, iV, 1) = Bounce_time_dimentionless_tmp;
        end

    else
        %% Calculate K for several pitch-angles, then interpolate into our K grid to find our pitch-angles    
        % Create new temporary alpha array to calculate new temporary K, interpolate to our K and find our alpaha
        al_min = 0.01;
        al_max = 89.9;
        al_size = 91; % ok, we really need more points here to make it accurate
        al_tmp = [al_min:(al_max-al_min)/al_size:al_max] * pi/180;
        sysaxis_geo = 1;
        
        %Bmin should be in NanoTesla!!!
        %[Bounce_time_dimentionless_tmp, K_tmp] = calc_K_Onera_Tsy(Field_line_arr, kext, options, sysaxis_geo, matlabd, maginput, Bf.z(:), [Bmin_surface.X_VERB_geo.arr(:), Bmin_surface.Y_VERB_geo.arr(:), Bmin_surface.Z_VERB_geo.arr(:)], al_tmp); 
        [Bounce_time_dimentionless_tmp, K_tmp]  = calc_K_Onera_Tsy(Field_line_arr, kext, internal_field, sysaxis_geo, Bf.z(:), [Bmin_surface.X_VERB_geo.arr(:), Bmin_surface.Y_VERB_geo.arr(:), Bmin_surface.Z_VERB_geo.arr(:)], al_tmp);
        
        Bounce_time_dimentionless_tmp = reshape(Bounce_time_dimentionless_tmp, [length(al_tmp) size(Bf.z)]);
        K_tmp = reshape(K_tmp, [length(al_tmp) size(Bf.z)]);

        if (false)
            %% test-plot

            p_idx = 12;
            r_idx = 6;

            figure
            plot(al_tmp*180/pi, log10(squeeze(K_tmp(:, p_idx, r_idx))), '-*');

            figure
            plot(al_tmp*180/pi, log10(squeeze(Bounce_time_dimentionless_tmp(:, p_idx, r_idx))), '-*');
        end

        % Interpolating to our K to find out alpha
        al.arr = NaN*zeros(size(K.arr));
        target_K = squeeze(K.arr(1, 1, 1, :));
        for iP = 1:size(K.arr, 1)
            for iR = 1:size(K.arr, 2)
                K_tmp_1d = squeeze(K_tmp(:, iP, iR));
                not_nan = find(~isnan(K_tmp_1d));
                al_1d = NaN;
                if (length(not_nan)>1)
                    al_1d = interp1(K_tmp_1d(not_nan), al_tmp(not_nan), target_K, 'linear', 'extrap');
                    %al_1d = interp1(K_tmp_1d(not_nan), al_tmp(not_nan), target_K, 'spline', 'extrap');
                else
                    warning('al all NaN')
                end
%                if (any(isnan(al_1d)))
%                    warning('al has NaN')
%                end
                for iV = 1:size(K.arr, 3) %V.size
                    al.arr(iP, iR, iV, :) = al_1d;
                end
            end
        end

        % Fill NaNs with something
        if (false)
            R_1d = squeeze(R.arr(1, :, 1, 1));
            for iP = 1:size(K.arr, 1)
                for iK = 1:K.size
                    al_1d = squeeze(al.arr(iP, :, 1, iK));
                    is_nan = isnan(al_1d);
                    if (is_nan(end) == 1)
                        is_nan(end) = 0;
                        al_1d(end) = pi/2;
                    end
                    al_1d(is_nan) = interp1(R_1d(~is_nan), al_1d(~is_nan), R_1d(is_nan), 'linear', 'extrap');
                    for iV = 1:size(K.arr, 3)
                        al.arr(iP, :, iV, iK) = al_1d;
                    end

                end
            end
        end
        
        Bounce_time_dimentionless.arr = NaN*zeros(size(K.arr));
        for iP = 1:size(K.arr, 1)
            for iR = 1:size(K.arr, 2)
                K_tmp_1d = squeeze(K_tmp(:, iP, iR));
                bt_tmp_1d = squeeze(Bounce_time_dimentionless_tmp(:, iP, iR));
                not_nan = find(~isnan(K_tmp_1d));
                bt_1d = NaN;
                if (length(not_nan)>1)
                    bt_1d = interp1(K_tmp_1d(not_nan), bt_tmp_1d(not_nan), target_K, 'linear', 'extrap');
                else
                    %warning('bt all NaN')
                end
                for iV = 1:size(K.arr, 3)
                    Bounce_time_dimentionless.arr(iP, iR, iV, :) = bt_1d;
                end
            end
        end


        %if (any(isnan(al.arr(:))))
        %    error('NaN alpha')
        %end

        al.arr(al.arr<0) = 0;
        al.arr(al.arr>pi/2) = pi/2;

        Bounce_time_dimentionless.arr(Bounce_time_dimentionless.arr <=0 ) = 1e-1;

    end
        
end
