function [Ef, Pot] = calc_E_field_VS(P, R, Kp, return_type)
    %%
    % CALC_E_FIELD_VS calculates Volland-Stern electric field 
    % following William [2006] paper
    % with Maynard and Chen [1975] Kp-parameterization and parameters from 
    % GEM Near Earth Magnetospheric Chalange, Phase 0 (Ganushkina)
    %
    % [Ef, Pot] = calc_E_field_VS(P, R, Kp, return_type)
    %
    % Input:
    %   P - MLT in rad, starting from P=0 at noon
    %   R - radial distance in Re
    %   Kp - Kp index
    %   return_type:
    %     'conv' - return is convective electric field and potentials
    %     'corr' - return is corrotating electric field and potentials
    %     empty or absent - sum of 'conv'+'corr'
    %
    % Output:
    %   Ef.p, Ef.r, Ef.z - electric field in V/m
    %   Pot - electic potential
    
    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits

    % SI unit system
    
    try
        return_type;
    catch
        return_type = '';
    end

    % different set of parameters
    GEM_challange = true;
    gamma = 2; 
    
    % Solar wind pressure - for NOT gem challenge parameterization
    %P_sw_nPa = 5%2; % 

    %% Constants
    q = -1.6e-19; % C

    B0 = 0.312e-4; % Tesla
    Omega = 7.43e-5; % 1/s
    RE = 6.387e6; % meters

    % Volt, ~30 kv - quite time, ~90 kv - storm time
    %Pot_pc = 65e3;
    Pot_pc = 45e3;  % Volt

    %% Electric potential

    % Corotation
    Pot_c = -(Omega .* B0 .* RE.^2) .* 1./R.arr; % R is in RE, Pot_c is in V

    % Convection
    if (GEM_challange)
        % GEM challenge
        A = 0.045 / (1 - 0.159 * Kp + 0.0093 * Kp^2)^3 * 1e3; % V / Re^2 %% shoule be = 202 V/Re^2 ?????? 
        Pot_m = -A .* R.arr.^gamma .* sin(P.arr); % 
    else
        % William, 2006
        Ly = 14.4 ./ (P_sw_nPa).^(1/6); % magnetospheric  width
        %Ls = Ly .* (182 ./ (gamma .* Ly .* Pot_pc./1e3)).^(1./gamma + 1); % a point where E_conv = E_corr
        %C = -Omega .* B0 .* RE.^3 ./ (gamma .* (RE .* Ls)); % ???

        % William, 2006, eq 11
        Pot_m = -Pot_pc ./ 2 ./ (Ly.^gamma) .* R.^gamma .* sin(P); % R is in RE
    end

    % Convection + Corotation potential
    Pot = Pot_m + Pot_c;

    %% Electric fields (theoretically we can get it from the potential, but I just use the same formulas)
    % Corotation
    E_c.r = -Omega .* B0 .* RE ./ R.arr.^2; % R is in RE, E is in V/m
    %E_c.r = -Omega .* B0 .* RE.^2 ./ R.^2; % R is in RE, E is in V/Re

    % Convections
    if (GEM_challange)
        % GEM challenge
        E_m.p = +A .* (R.arr).^(gamma) ./ (R.arr .* RE) .* (cos(P.arr));  % R in RE, E is in V/m
        E_m.r = +A .* (R.arr).^(gamma-1) ./ RE .* (gamma .* sin(P.arr)); % R in RE, E is in V/m
        %E_m.p = A .* (R).^(gamma) ./ R .* (cos(P)); % R in RE, E in V/Re
        %E_m.r = A .* (R).^(gamma-1) .* gamma .* sin(P); % R in RE, E in V/Re
    else
        % William, 2006, eq 12
        E_m.p = +Pot_pc ./ (2 .* RE .* Ly) .* (R.arr ./ Ly) .^ (gamma-1) .* (cos(P.arr)); % R is in RE, V/m
        E_m.r = +Pot_pc ./ (2 .* RE .* Ly) .* (R.arr ./ Ly) .^ (gamma-1) .* (gamma .* sin(P.arr)); % R is in RE, V/m
    end

    % Convection + Corotation
    Ef.p = E_m.p;
    Ef.r = E_m.r + E_c.r;
    Ef.z = zeros(P.size, R.size);
    
    
    switch return_type
        case 'conv',
            Ef = E_m;
            Pot = Pot_m;
        case 'corr',
            Ef = E_c;
            Pot = Pot_c;
    end
    
end

