function [Lstar, Phi] = calc_Lstar(P, R, Bf, traj, MRe)
    %%
    % CALC_LSTAR calcuates L* 
    % using input information about trajectories TRAJ
    %
    % [Lstar, Phi] = calc_Lstar(P, R, Bf, traj, MRe)
    %
    % TRAJ (an array) is constant along particle's trajectoris,
    % e.g. for equatorially mirroring particles traj = mu*B
    % !! traj should monotonically decrease with distance from the Earth center !!
    %
    % Grid requirements:
    %   P should have 1 point overlap in MLT, i.e. P(1) = P(end)
    %   P and R should be orthogonal
    %
    % Input:
    %   P - MLT in rad, with P=0 at noon
    %   R - radial distance in Re
    %   Bf - magnetic field in GAUSS
    %   traj - anything that is constant along particle's trajectoris,
    %          e.g. for equatorially mirroring particles traj = mu*B
    %          !! traj should monotonically decrease with distance from !!
    %          !! the Earth                                             !!
    %   MRe - magnetic field at Earth's surface in Gauss, 0.31 by default
    %
    % Explanation:
    % We need to integrate magnetic field inside of each trajectory.
    % Array 'traj' indicate trajectories, e.g. each particle drift with constant
    % traj. Moreover, traj should be monotonic with distance from the Earth. It
    % is usually the case, because particles drift with constant B, and B
    % decreases away from the Earth. In fact, usually traj==B.
    % Let's say we're at pont 'i' with magnetic field B(i) and trajectory indicator traj(i)
    % We need to find integral(B < B(i)). We define integral of one grid point
    % as int_B(i) = B(i)*area(i), and find the total integral as a summation of
    % integrals at each grid point where B < B(i):
    %   int_B = sum(int_B(B < B(i))).
    % This is accurate when we have high grid resolution.
    % 
    % To increase accuracy, we can accound for partial int_B (particle
    % trajectory can cross some grid square, so we need to add only part of it
    % into the integral). So I add that 'correction' as partial integration,
    % proportional to the difference between magnetic field at current point,
    % and at next/previous radial point. It's hard to explain, but it works well 
    % and can be it can be switched on and off using partial_correction = true/false.
    %
    % Integral B = Integral(R<1) B + Integral (R>1 & R<R(i)) B

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plasma.Dip.B_Dip
    
    partial_correction = true;

    % Check that B is in Tesla :(
    if (max(Bf(:)) > 1e-4)
       error('B should be in Tesla!'); 
    end    
    Bf = Bf.*1e4; % convert to Gauss
    
    try
        MRe;
    catch
        MRe = 0.31e-4;
    end
    
    % Check that MRe is in Tesla :(
    if (MRe > 1e-4)
       error('MRe should be in Tesla!'); 
    end    
    MRe = MRe.*1e4;  % convert to Gauss
    
    % Checks
    %if (min(R(:) ~= 1))
    %    error('min(R) should be = 1')
    %end
%    if (P(1,end) - (P(end,end)-2*pi) > 1e-5)
%        warning('P(1) should be = P(end), Lstar = -1.')
%        Lstar = -1;
%        Phi = NaN;
%        return
%    end

    % use dipole field for R<R_min, because Tsyganenco fiels has some
    % gliches there
    R_min = 1.5;
    idx = find(R<=R_min);
    Bf(idx) = B_Dip(R(idx)); % GAUSS    

    dP = P(2,end) - P(1,end);    
    dR = R(1,2) - R(1,1);
    
    dBint=zeros(size(P));
    % dBint - integrated field at each grid point (grid square)
    % The overall accuracy depends on the accuracy of this
    if (false) % This calculates L-star in the down-left grid square, while the rest of the program expect it to be calculate in the center
        first_Ridx = 2;
        for iP = 2:size(P,1)
            for iR = 2:size(R,2)
                %Bint = Bint - Bf(iP, iR) .* dR .* dP .* R(iP, iR); % Gauss Re^2
                %Bint = Bint - (Bf(iP, iR) + Bf(iP-1, iR) + Bf(iP, iR-1) + Bf(iP-1, iR-1))/4 .* (pi*(R(iP, iR))^2*dP/(2*pi) - pi*(R(iP, iR-1))^2*dP/(2*pi));
                %Bint = Bint - Bf(iP, iR) .* (pi*(R(iP, iR)+dR/2)^2*dP/(2*pi) - pi*(R(iP, iR)-dR/2)^2*dP/(2*pi));

                % Sector area
                %dBint(iP, iR) = (Bf(iP, iR) + Bf(iP-1, iR) + Bf(iP, iR-1) + Bf(iP-1, iR-1))/4 .* (pi*(R(iP, iR))^2*dP/(2*pi) - pi*(R(iP, iR-1))^2*dP/(2*pi));
                dBint(iP, iR) = (Bf(iP, iR) + Bf(iP-1, iR) + Bf(iP, iR-1) + Bf(iP-1, iR-1))/4 .* (R(iP, iR)^2 - R(iP, iR-1)^2)*dP/2;
            end
        end
        %dBint(1, 2:end) = (Bf(1, 2:end) + Bf(2, 2:end) + Bf(1, 1:end-1) + Bf(2, 1:end-1))/4 .* (pi.*(R(1, 2:end)).^2.*dP./(2*pi) - pi.*(R(1, 1:end-1)).^2.*dP/(2.*pi));

        % Third invariant for R=R_min
        Phi0 = 2*pi*MRe/R(1,1); % Gauss Re^2
    
        % ! This calculates L-star in the down-left grid square, while the rest of the program expect it to be calculate in the center !
    else
        
        for iP = 2:size(P,1) % cause P(1) == P(end)
        %for iP = 1:size(P,1) % aparently it should be like this
            for iR = 2:size(R,2)
                % Sector area
                dBint(iP, iR) = Bf(iP, iR) .* (pi*(R(iP, iR)+dR/2)^2*dP/(2*pi) - pi*(R(iP, iR)-dR/2)^2*dP/(2*pi));
                
                %dBint(iP, iR) = Bf(iP, iR) .* (pi*(R(iP, iR))^2*dP/(2*pi) - pi*(R(iP, iR)-dR)^2*dP/(2*pi));
                %dBint(iP, iR) = Bf(iP, iR) .* (pi*(R(iP, iR)+dR)^2*dP/(2*pi) - pi*(R(iP, iR))^2*dP/(2*pi));%%
                %dBint(iP, iR) = Bf(iP, iR) .* (pi*(R(iP, iR) + 80*dR/100)^2*dP/(2*pi) - pi*(R(iP, iR) - 20*dR/100)^2*dP/(2*pi));
                
                %dBint(iP, iR) = (Bf(iP, iR-1)+Bf(iP, iR))/2 .* (pi*(R(iP, iR)+dR/2)^2*dP/(2*pi) - pi*(R(iP, iR)-dR/2)^2*dP/(2*pi));
                
                %dBint(iP, iR) = (Bf(iP, iR-1)*0.01 + 0.99*Bf(iP, iR)) .* (pi*(R(iP, iR)+dR/2)^2*dP/(2*pi) - pi*(R(iP, iR)-dR/2)^2*dP/(2*pi));
                
                %dBint(iP, iR) = Bf(iP, iR) .* (pi*(R(iP, iR))^2*dP/(2*pi) - pi*(R(iP, iR-1))^2*dP/(2*pi));
                
                %dBint(iP, iR) = Bf(iP, iR).*R(iP,iR)^3 / (2*dR) * ( - 1/R(iP,iR+1)^2/2 + 1/R(iP,iR-1)^2/2 ) ...
                %    .* (pi*(R(iP, iR)+dR/2)^2*dP/(2*pi) - pi*(R(iP, iR)-dR/2)^2*dP/(2*pi));
                %dBint(iP, iR) = 0.31 / (2*dR) * ( - 1/R(iP,iR+1)^2/2 + 1/R(iP,iR-1)^2/2 ) ...
                
                
                %% '!!!!!!!!!!'
            end
        end
        
        % Third invariant for R=R_min
        Phi0 = 2*pi*MRe/(R(1,2) - dR/2);% Gauss Re^2

        %Phi0 = 2*pi*MRe/(R(1,2) - 0.5*dR/2);% Gauss Re^2
        
    end
    
    nanidx = isnan(dBint);
    dBint(nanidx) = 0;

    % Sort grid points as trajectory. 
    % Integral of B-field for next point will include all the previous points
    [traj_sorted, dBidx] = sort(traj(:), 'descend');

    Phi = zeros(size(P));

    %% Experimental - attempt to account for cells, partially covered by drift path
    if (partial_correction)
        traj_next = zeros(size(traj));
        traj_prev = zeros(size(traj));
        traj_next(:,1:end-1) = traj(:,2:end);
        traj_prev(:,2:end) = traj(:,1:end-1);
    end
    
    % dBsum - integrated magnetic field
    dBsum = zeros(size(P));
    total = 0; 
    for ii = 1:length(Bf(:))
        %total = total + dBint(ii); % add integral over each grid square consequently
        total = total + dBint(dBidx(ii)); % add integral over each grid square consequently
        %dBsum(dBidx(ii)) = total; % save to the appropriate grid point
        
        delta = 0;
        %delta = 0.001*total .* R(dBidx(ii));
        %delta = 0.01*total;
        
        %% Experimental - attempt to account for cells, partially covered by drift path
        if (partial_correction)
            % So,
            next = traj_next(dBidx(ii));
            prev = traj_prev(dBidx(ii));

            % integrated in full instead of an integral of a prt of a cell
            overaccounted_idx = find((traj_sorted <= prev)' & [1:length(Bf(:))] <= ii);
            if (~isempty(overaccounted_idx))
                integral_part = 0.5*(traj_sorted(overaccounted_idx) - traj_sorted(overaccounted_idx(1))) / (traj_sorted(overaccounted_idx(end)) - traj_sorted(overaccounted_idx(1)));
                delta = delta - sum(integral_part .* dBint(dBidx(overaccounted_idx)));
            end

            % not integrated at all
            underaccounted_idx = find((traj_sorted >= next)' & [1:length(Bf(:))] > ii);
            if (~isempty(underaccounted_idx))
                integral_part = 0.5 - 0.5*(traj_sorted(underaccounted_idx) - traj_sorted(underaccounted_idx(1))) / (traj_sorted(underaccounted_idx(end)) - traj_sorted(underaccounted_idx(1)));
                integral_part(isnan(integral_part)) = 0;
                delta = delta + sum(integral_part .* dBint(dBidx(underaccounted_idx)));
            end
            %traj_sorted(overaccounted_idx)
            %traj_sorted(underaccounted_idx)
        end
                
        %%
        Phi(dBidx(ii)) = Phi0 - total - delta; % third invariant
    end
    
    Phi(1,:) = Phi(end,:);
    Phi(:,1) = 2*pi*MRe./(R(:,1));

    Phi(nanidx) = NaN;
    
    % Calculate third invariant as integral for R<1 + integral outside
%    Phi = zeros(size(P));
%    for iP = 2:size(P,1) % cause P(1) == P(end)
%        for iR = 2:size(R,2)
%            %Phi(iP, iR) = Phi0 - sum(dBint(traj>=traj(iP,iR)));
%            Phi(iP, iR) = Phi0 - dBsum(iP, iR);
%        end
%    end

    % Calculate Lstar
    Lstar = 2*pi*MRe./Phi;

end