function plot_drift_velocities(P, R, V, K, en, al, mu, Bf, Pot, vel, IMF_Bz, IMF_Dp, target_mu, target_K)
    %%
    % PLOT_DRIFT_VELOCITIES plots drift velocities
    %
    % plot_drift_velocities(P, R, V, K, en, al, mu, Bf, Pot, vel, IMF_Bz, IMF_Dp, target_mu, target_K)
    %
    % Input:
    %  P - MLT in rad, P=0 at noon
    %  R - radial distance in Re
    %  V=mu/(K+0.5)^2 - new invariant
    %  K - adiabatic invariant
    %  L - invariant
    %  en - energy
    %  al - pitch-angle
    %  mu - first invariant
    %  Bz - magnetic field in Tesla
    %  Pot - electric potential
    %  vel with
    %    vel.p - azimuthal drift velocity in 1/day
    %    vel.r - radial drift velocity in Re/day
    %  IMF_Bz - IMF Bz
    %  IMF_Dp - IMF dynamic pressure
    %  target_mu - at what mu to plot
    %  tarket_K - at what K to plot

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import read_write.load_4d
    import plasma.magnetopause_Shue
    import plotting.plot_earth

    [X.arr, Y.arr] = pol2cart(P.arr(:, :, 1, 1), R.arr(:, :, 1, 1));
    
    % constants
    B0 = 0.312e-4; % Tesla
    q = -1.6e-19; % C 
    Re = 6.387e6; % meters
    mc2 = 0.511;  % MeV
    c_si = 299792458;   % m/sec
    MeV2J = 1.60217646e-13;

    %EKp = AllKp_arr(1);
    %[P, R, V, K, Pot] = load_4d([VERB_input_folder, './E_field/R_pot_Kp', num2str(EKp), '.dat']);

    %[P, R, V, K, Bz] = load_4d([VERB_input_folder, 'B_field/B_field_Kp', num2str(EKp), '.dat']);
    %%Bf.z = squeeze(Bz.arr(:,:,1,1));
    %%Bf.r = zeros(size(Bz.arr));
    %%Bf.p = zeros(size(Bz.arr));
    %[P, R, V, K, L, en, al, mu] = load_4d([VERB_input_folder, 'B_field/Lstar_Kp', num2str(EKp), '.dat']);
    %pc.arr = pfunc(en.arr);

    %[P, R, V, K, VP] = load_4d([VERB_input_folder, 'Vx/VP_Kp', num2str(EKp), '.dat']);
    %[P, R, V, K, VR] = load_4d([VERB_input_folder, 'Vx/VR_Kp', num2str(EKp), '.dat']);
    V_tot.x = vel.r .* cos(P.arr) - vel.p .* sin(P.arr) .* R.arr; %!
    V_tot.y = vel.r .* sin(P.arr) + vel.p .* cos(P.arr) .* R.arr; %!

    P_idx = round(P.size/2);
    [tmp, R7_idx] = min(abs(squeeze(R.arr(P_idx,:,1,1) - 7)));
    [tmp, K_idx] = min(abs(squeeze(K.arr(P_idx,R7_idx,1,:) - target_K)));
    [tmp, V_idx] = min(abs(squeeze(mu.arr(P_idx,R7_idx,:,K_idx) - target_mu)));

    al_range = al.arr(:,:,:,K_idx)*180/pi;


    % V_GC.x(:,:, iV, 1)
    % V_GC.x(:,:, iV, 7)


    % mu * B + Phi * q
    % B should be in Gauss???
    traj = squeeze(mu.arr(:, :, V_idx, K_idx)) .* (Bf.z(:, :, 1, 1) / 1e-4) + Pot(:, :, V_idx, K_idx) .* q / MeV2J; % MeV
    %traj = Pot.arr(:, :, V_idx, K_idx) .* q / MeV2J; % MeV
    %traj = mu.arr(:, :, V_idx, K_idx) .* (Bf.z / 1e-4); % MeV
    traj = traj + 12e-2;

    %
    f1 = figure('Units', 'inches','Position', [0.0 0.0 10 10],... %1+2*Dxx_num], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, ...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%

    cp = colormap(jet(1));
    cp = [1 1 1];
    colormap(cp)

    hold on

    %
    Lmp.r = magnetopause_Shue(squeeze(P.arr(:, 1, 1, 1)), IMF_Bz, IMF_Dp);
    [Lmp.x, Lmp.y] = pol2cart(squeeze(P.arr(:,1,1,1)), Lmp.r);

    Lmp.x(abs(Lmp.x)>15) = NaN; 
    Lmp.y(abs(Lmp.y)>15) = NaN;

    outside.arr = zeros(P.size, R.size, V.size, K.size);
    for iP = 1:P.size
        for iR = 1:R.size
            if (R.arr(iP, iR, 1, 1) > Lmp.r(iP))
                outside.arr(iP, iR, :, :) = 1; %
            end
        end
    end
    %

    %traj(find(outside.arr)) = NaN;
    traj(squeeze(R.arr(:,:,1,1)) < 2) = NaN;

    x_start = [-6, -8, -10, -8, -8 ];
    y_start = [0, 0, 0, -7, 7];
    if (true)
        trace_idx = [1,2,3];
        %colors = {'blue', [0 0.4 0], 'magenta'};
        colors = {'blue', 'magenta', [0 0.4 0]};
    else
        trace_idx = [4,2,5];
        %colors = {'blue', [0 0.4 0], 'magenta'};
        colors = {'blue', 'magenta', [0 0.4 0]};
    end
    [X_unif, Y_unif] = meshgrid([-10:0.1:10], [-10:0.1:10]);
    V_x_int = griddata(X.arr, Y.arr, squeeze(V_tot.x(:, :, V_idx, K_idx)), X_unif, Y_unif, 'cubic');
    V_y_int = griddata(X.arr, Y.arr, squeeze(V_tot.y(:, :, V_idx, K_idx)), X_unif, Y_unif, 'cubic');
    %outside_int = griddata(X.arr, Y.arr, squeeze(outside.arr(:, :, V_idx, K_idx)), X_unif, Y_unif, 'cubic');
    %outside_int(isnan(outside_int)) = 1;
    %V_x_int(~(outside_int(:)==0)) = NaN;
    %V_y_int(~(outside_int(:)==0)) = NaN;

    contourf(X.arr, Y.arr, log10(traj), 50);
    min_r = 4;
    p_step = 4;
    r_step = 4;

    V_tot.x(find(outside.arr(:))) = NaN;
    V_tot.y(find(outside.arr(:))) = NaN;

    V_tot.x = V_tot.x./1e4;
    V_tot.y = V_tot.y./1e4;
    
    %V_tot.x = sign(V_tot.x) * log10(abs(V_tot.x));
    %V_tot.y = sign(V_tot.y) * log10(abs(V_tot.y));
    V_tot.x(V_tot.x>1e4)=1e4;V_tot.y(V_tot.y>1e4)=1e4;
    V_tot.x(V_tot.x<-1e4)=-1e4;V_tot.y(V_tot.y<-1e4)=-1e4;
    q1 = quiver(X.arr(1:p_step:end-1,min_r:r_step:end), Y.arr(1:p_step:end-1,min_r:r_step:end), V_tot.x(1:p_step:end-1,min_r:r_step:end, V_idx, K_idx), V_tot.y(1:p_step:end-1,min_r:r_step:end, V_idx, K_idx), 'Color', [0.1 0.1 0.1]);
    %q1 = quiver(X.arr, Y.arr, V_tot.x, V_tot.y, 'Color', [0.1 0.1 0.1]);

    %caxis([-2.5 -0.5])
    %colorbar
    xlabel('X, R_E');
    ylabel('Y, R_E');
    set(gca, 'xDir', 'reverse', 'yDir', 'reverse')

    plot(Lmp.x, Lmp.y, '--r', 'LineWidth', 4)


    hold on
    %

    h = streamline(X_unif, Y_unif, V_x_int, V_y_int, x_start(trace_idx), y_start(trace_idx));
    set(h, 'Color', colors{2})
    set(h(1), 'Color', colors{1})
    set(h(3), 'Color', colors{3})
    %
    plot(x_start(trace_idx), y_start(trace_idx), '.', 'MarkerSize', 40, 'Color', colors{2})
    plot(x_start(trace_idx(1)), y_start(trace_idx(1)), '.', 'MarkerSize', 40, 'Color', colors{1})
    plot(x_start(trace_idx(3)), y_start(trace_idx(3)), '.', 'MarkerSize', 40, 'Color', colors{3})

    pc1 = pcolor(X.arr, Y.arr, X.arr*NaN);
    set(pc1, 'LineWidth', 1, 'LineStyle', ':')


    plot_earth

    title({'Drift trajectories and velocity vectors', ...
        ['   ', '\mu = ', num2str(unique(mu.arr(:, :, V_idx, K_idx)), '%.0f'), ' MeV/G, ', ...
        '   ', 'K = ', num2str(unique(K.arr(:, :, V_idx, K_idx)), '%.3f'), ' G^{0.5} R_E, ', ...
        '   ', '\alpha ~= ', num2str(mean(al_range(~isnan(al_range))), '%.0f'), ' deg'] ...
    })
    %save_picture(gcf, ['Drift_M=', num2str(unique(mu.arr))], 'jpg', 'pdf', 'epsc');

    %xlim([-12 12]);
    %ylim([-12 12]);
    axis auto

    %title({'Drift contours and velocity vectors', ['E=', num2str(unique(en.arr(:, :, V_idx, K_idx))), ' MeV, ', 'Pitch-angle=', num2str(unique(al.arr(:, :, V_idx, K_idx)*180/pi)), ' deg']})
    %save_picture(gcf, ['Drift_E=', num2str(unique(en.arr))], 'jpg', 'pdf', 'epsc');
    %text(min(xlim), min(ylim), {...
    %    ['   ', '\mu = ', num2str(unique(mu.arr(:, :, V_idx, K_idx)), '%.0f'), ' MeV/G, '], [], ...
    %    ['   ', 'K = ', num2str(unique(K.arr(:, :, V_idx, K_idx)), '%.3f'), ' G^{0.5} R_E'], [], ...
    %    ['   ', '\alpha ~= ', num2str(mean(al_range(~isnan(al_range))), '%.0f'), ' deg'], ...
    %    ' '}, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'Color', [0, 0, 0]);



    %legend('Drift trajectories', 'Drift velocities', 'Magnetopause location', 'Drift trajectory', 'Location', 'NorthWest')
    %legend('Drift velocities', 'Magnetopause location', 'Drift trajectory', 'Location', 'NorthWest')

end
