function plot_f_L_time_4D(f, RL, varargin)  
    %%
    % PLOT_F_L_TIME_4D plots PSD or Flux as a function of L or R and time
    %
    % plot_f_L_time_4D(f, RL, vargin)
    %
    % Input:
    %  f - the function, should have f.arr and f.time
    %  RL - L or R, should have .arr
    %  optional arguments, vargin:
    %    figure_title - figure title
    %    plot_type - 'pcolor' or 'contourf'
    %    caxis = [c_min c_max] - caxis limitx
    %    xlim = [x_min x_max] - x-limits for time
    %    ylim = [y_min y_max] - y-limits
    %    relative_time - count time from zero and don't apply dateticks
    %    PanelX - lineplot panel to plot the next argument on, where X is panel number, e.g. Panel1, Panel2, Panel3, etc (numbered from bottom to top) with Panel1 being the main panel (with PSD or Fluxes)
    %      Lineplot arguments:
    %        A variable VAR with
    %          VAR.arr - array of values
    %          VAR.time - time
    %          VAR.name - (optional) name
    %      Example:
    %        'Panel2', Kp, ...
    %        where Kp has Kp.arr and Kp.time and Kp.name = 'Kp'
    %
    % Example:
    % plot_f_L_time_4D(PSD, L, 'Panel1', Lpp, 'Panel2', Kp, 'Panel3', Dst);
    %

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    figure_title = '';
    c_lim = [min(log10(f.arr(:))) max(log10(f.arr(:)))];
    x_lim = [f.time(1) f.time(end)];

    try
        f.time;
    catch
        f.arr = permute(f.arr, [3 1 2]);
        f.time = [0];
    end

    maxPanel = 1;
    Panel = {};
    plot_type = 'pcolor';
    time_shift = 0;
    plot_datetick = true;

    required_arguments = 3;
    if (nargin > required_arguments)
        %        for it = 1:1:nargin-required_arguments
        it = 1;
        while it <= nargin-required_arguments
            if strcmp(varargin(it), 'figure_title')
                figure_title = varargin{it+1};
                it = it + 1;
            elseif strcmp(varargin(it), 'caxis')
                c_lim = varargin{it+1};
                it = it + 1;
            elseif strcmp(varargin(it), 'xlim')
                x_lim = varargin{it+1};
                it = it + 1;
            elseif strcmp(varargin(it), 'ylim')
                y_lim = varargin{it+1};
                it = it + 1;
            elseif strcmp(varargin(it), 'relative_time')
                time_shift = varargin{it+1};
                plot_datetick = false;
                it = it + 1;
            elseif strcmp(varargin(it), 'plot_type')
                plot_type = varargin{it+1};
                it = it + 1;
            elseif strfind(varargin{it}, 'Panel')
                panelNumber = str2num(varargin{it}(6:end));
                if (maxPanel<panelNumber)
                    maxPanel = panelNumber;
                end;
                try
                    Panel{panelNumber}{end + 1} = varargin{it+1};
                catch
                    Panel{panelNumber}{1} = varargin{it+1};
                end
                it = it + 1;
            end
            it = it + 1;
        end
    end

    c_min = c_lim(1);
    c_max = c_lim(2);
    x_min = x_lim(1);
    x_max = x_lim(2);

    f.arr(find(f.arr<=0)) = NaN;

    log_abs_f = log10(abs(f.arr));
    log_abs_f(isinf(log_abs_f)) = 0;
    log_abs_f(find(log_abs_f>c_max)) = c_max;
    log_abs_f(find(log_abs_f<c_min)) = c_min;

    colors = [...
        [0 0 0]',...
        [1 0 0]',...
        [0 1 0]',...
        [0 0 1]',...
        [1 1 0]',...
        [1 0 1]',...
        [0 1 1]',...
        [1 1 1]']';
    
    ax_x_min = 0.1;
    ax_x_size = 0.8;
    
    
    ax_y_gap = 0.003;
    ax1_y_gap = 0.1;
    ax1_y_bottom = 0.1;
    ax1_y_top = 0.1;
    ax1_y_size = 0.4;
    if (maxPanel > 1)
        ax_y_size = (1 - ax1_y_bottom - ax1_y_top - ax1_y_size) ./ (maxPanel) - ax_y_gap;
    else
        ax_y_size = 0;
    end

    f1 = figure('Units', 'inches', 'Position', [1. 1. 8 10.5], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 10,...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%

    cmap = colormap;
    cmap(1,:) = [0 0 0];
    colormap(cmap);

    sp = subplot('Position', [ax_x_min, ax1_y_bottom, ax_x_size, ax1_y_size]);

    switch plot_type
        case 'pcolor'
            try
                pcolor(f.time - time_shift, RL.arr, log_abs_f);
            catch
                pcolor(f.time - time_shift, RL.arr, log_abs_f');
            end
        otherwise 
            try
                contourf(f.time - time_shift, RL.arr, log_abs_f, 100);
            catch
                contourf(f.time - time_shift, RL.arr, log_abs_f', 100);
            end
    end
    shading flat

    axis tight
    try ylim(ylim_param); end;
    if (plot_datetick)
        datetick('x', 'keeplimits')
    end

    try
        %text(max(xlim), (min(ylim) + max(ylim))/2, 'PSD (c/cm/MeV)^3', 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold', 'FontSize', 10);
        text(max(xlim), (min(ylim) + max(ylim))/2, ['log_{10} ', f.units], 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold', 'FontSize', 10);
    end

    xlabel('Time, days')
    try
        ylabel(RL.name);
    catch
        ylabel('R, Re');
    end

    try title(figure_title); end;

    hold on
    
    try
        iPanel = 1;
        names = {};
        for iPic = 1:length(Panel{iPanel})
            var = Panel{iPanel}{iPic};
            plot(var.time - time_shift, var.arr, '--w');
            try
                names = {names{:}, var.name};
            end
        end
    end
    %legend(names);

    xlim(x_lim - time_shift);
    try ylim(y_lim); end
    
    caxis([c_min c_max]);

    ax_pos = get(gca, 'Position');
    cl = colorbar('West');
    cl_pos = get(cl, 'Position');
    cl_x_pos = ax_pos(1) + ax_pos(3) + 0.025;%0.015; %0.925
    set(cl, 'Position', [cl_x_pos, cl_pos(2), cl_pos(3), cl_pos(4)]);
    
    for iPanel = 2:length(Panel)
        sp1 = subplot('Position', [ax_x_min, ax1_y_bottom+ax1_y_size+ax1_y_gap+(iPanel-2)*(ax_y_size+ax_y_gap), ax_x_size, ax_y_size]);
                
        names = {};
        hold on
        for iPic = 1:length(Panel{iPanel})
            var = Panel{iPanel}{iPic};
            plot(var.time - time_shift, var.arr, 'Color', colors(iPic,:));
            try
                names = {names{:}, var.name};
            end
        end
        xlim(x_lim - time_shift);

        if (length(names) == 1)
        	ylabel(names);
        else
            legend(names);
        end
        
        if (iPanel == 2)
            if (plot_datetick)
                datetick('x', 'keeplimits');
            end
            xlabel('Time');
        else
            set(sp1, 'xTick', []);
        end
        
    end

    tmp = pwd;
    tmp_idx = strfind(tmp,'\');
    tmp_idx = [tmp_idx, strfind(tmp,'/')];    
    set(gcf, 'name', tmp(end-tmp_idx(end):end));

    %save_picture(gcf, './Plots/plot_PSD_L_time', 'jpg', 'pdf', 'epsc');
    %save_picture(gcf, ['./Plots/', file_name], 'jpg', 'pdf', 'epsc');

end
