function video_f_mlt_4D(f, P, R, varargin)
    %%
    % VIDEO_F_MLT_4D plots PSD or Flux in X,Y coordinates (i.e. equatorial plane)
    % Will be a video if length(f.time) > 1
    %
    % video_f_mlt_4D(f, P, R, varargin) 
    %
    % Input:
    %  f - PSD or flux as _3D_ array: [time, MLT, radial distance]
    %  P - MLT in rad, with P=0 at noon
    %  R - radial distance in Re
    %  optional arguments, vargin:
    %    figure_title - figure title
    %    plot_type - 'pcolor' or 'contourf'
    %    caxis = [c_min c_max] - caxis limitx
    %    xlim = [x_min x_max] - x-limits for time
    %    PanelX - lineplot panel to plot the next argument on, where X is panel number, e.g. Panel1, Panel2, Panel3, etc (numbered from bottom to top) with Panel1 being the main panel (with PSD or Fluxes)
    %      Lineplot arguments:
    %        A variable VAR with
    %          VAR.arr - array of values
    %          VAR.time - time
    %          VAR.name - (optional) name
    %      Examle:
    %        'Panel2', Kp, ...
    %        where Kp has Kp.arr and Kp.time and Kp.name = 'Kp'
    %
    % Example:
    %  plot_f_L_time_4D(PSD, L, 'Panel1', Lpp, 'Panel2', Kp, 'Panel3', Dst);
    %

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    import plotting.subplot1
    import plotting.plot_earth

    import plasma.magnetopause_Shue

    figure_title = '';
    input_folder = './VERB_input/';
    c_lim = [min(log10(f.arr(:))) max(log10(f.arr(:)))];
    x_lim = [-max(R.arr(:)) max(R.arr(:))];

    try
        f.time;
    catch
        f.arr = permute(f.arr, [3 1 2]);
        f.time = [0];
    end

    maxPanel = 1;
    Panel{1} = {};
    plot_type = 'pcolor';

    required_arguments = 3;
    if (nargin > required_arguments)
    %    for it = 1:1:nargin-required_arguments
        it = 1;
        while it <= nargin-required_arguments
            if strcmp(varargin(it), 'figure_title')
                figure_title = varargin{it+1};
                it = it + 1;
            elseif strcmp(varargin(it), 'input_folder')
                input_folder = varargin{it+1};
                it = it + 1;
            elseif strcmp(varargin(it), 'caxis')
                c_lim = varargin{it+1};
                it = it + 1;
            elseif strcmp(varargin(it), 'xlim')
                x_lim = varargin{it+1};;
                it = it + 1;
            elseif strcmp(varargin(it), 'plot_type')
                plot_type = varargin{it+1};;
                it = it + 1;
            elseif strfind(varargin{it}, 'Panel')
                panelNumber = str2num(varargin{it}(6:end));
                if (maxPanel<panelNumber)
                    maxPanel = panelNumber;
                end;
                try
                    Panel{panelNumber}{end + 1} = varargin{it+1};
                catch
                    Panel{panelNumber}{1} = varargin{it+1};
                end
                it = it + 1;
            end
            it = it + 1;
        end
    end

    c_min = c_lim(1);
    c_max = c_lim(2);

    x_min = x_lim(1);
    x_max = x_lim(2);
    y_min = x_lim(1);
    y_max = x_lim(2);

    mkdir('./Video');
    %mov = avifile('./Video/Convection.avi', 'fps', 5, 'quality', 0, 'compression', 'none');
    mov = VideoWriter('./Video/Convection.avi');
    open(mov);

    %%
    f1 = figure('Units', 'inches', 'Position', [1. 1. 8 10.5], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 10,...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%

    tmp = pwd
    set(gcf, 'name', tmp(end-60:end));

    [X.arr, Y.arr] = pol2cart(P.arr(:,:,1,1), R.arr(:,:,1,1));
    
    colors = [...
        [0 0 0]',...
        [1 0 0]',...
        [0 1 0]',...
        [0 0 1]',...
        [1 1 0]',...
        [1 0 1]',...
        [0 1 1]',...
        [1 1 1]']';
    
    ax_x_min = 0.1;
    ax_x_size = 0.8;
    %ax_y_size = 0.147;

    ax_y_gap = 0.003;
    ax1_y_gap = 0.1;
    ax1_y_bottom = 0.1;
    ax1_y_top = 0.1;
    ax1_y_size = 0.4;
    if (maxPanel > 1)
        ax_y_size = (1 - ax1_y_bottom - ax1_y_top - ax1_y_size) ./ (maxPanel) - ax_y_gap;
    else
        ax_y_size = 0;
    end


    cl = 0;
    for it = 1:length(f.time)
        %it
        % magnetosphere
        %figure
        %[tmp, Lmp_it] = max(find(IMF_Bz(:, 1) <= f_interp.time(it)));
        
        try
            Lmp_it = max(find(IMF_Bz.time <= f.time(it)));
            Lmp.r = magnetopause_Shue(squeeze(P.arr(:, 1, 1, 1)), IMF_Bz.arr(Lmp_it), IMF_Dp.arr(Lmp_it));
            [Lmp.x, Lmp.y] = pol2cart(squeeze(P.arr(:,1,1,1)), Lmp.r);
            Lmp.x(abs(Lmp.x)>25) = NaN;
            Lmp.y(abs(Lmp.y)>25) = NaN;
        catch
            Lmp.x = NaN;
            Lmp.y = NaN;
            Lmp.p = NaN;
            Lmp.r(1:P.size) = 999;
        end
        
        outside.arr = zeros(P.size, R.size);
        for iP = 1:P.size
            outside.arr(iP, R.arr(iP, :, 1, 1) > Lmp.r(iP)) = 1;
        end
        
        
        sp3 = subplot('Position', [0.2, 0.1, 0.6, 0.4]);
        set(gca,'nextplot','replacechildren');
        cla(gca)
        hold on
        
        %f_plot = squeeze(log_f_interp(it, :, :));
        f_plot = squeeze(f.arr(it, :, :));
        f_plot(find(f_plot<=0)) = NaN;
        f_plot = log10(f_plot);
        f_plot(isinf(f_plot)) = NaN;
        f_plot(find(f_plot>c_max)) = c_max;
        f_plot(find(f_plot<c_min)) = c_min;
        
        switch plot_type
            case 'pcolor'
                pcolor(X.arr, Y.arr, f_plot);
            otherwise 
                contourf(X.arr, Y.arr, f_plot, 100);
        end
        shading flat
        
        % grid
        if (false)
            pc1 = pcolor(X.arr, Y.arr, f_plot*NaN);
            set(pc1, 'LineWidth', 1, 'LineStyle', ':')
        end
        xlim([x_min x_max]);
        ylim([y_min y_max]);
        
        set(gca, 'xDir', 'reverse', 'yDir', 'reverse')
        xlabel('X, R_E');
        ylabel('Y, R_E');
        
        caxis([c_min c_max]);
        if (it == 1)
            ax_pos = get(gca, 'Position');
            cl = colorbar('West');
            cl_pos = get(cl, 'Position');
            cl_x_pos = ax_pos(1) + ax_pos(3) + 0.035;%0.015; %0.925
            set(cl, 'Position', [cl_x_pos, cl_pos(2), cl_pos(3), cl_pos(4)]);

            title(figure_title);
        end
        try text(min(xlim), (min(ylim) + max(ylim))/2, ['log_{10} ', f.units], 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold', 'FontSize', 10); end;
        text(min(xlim), min(ylim), [datestr(f.time(it))], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'right', 'color', 'black')
        
        plot_earth

        hold on
        iPanel = 1;
        names = {};
        for iPic = 1:length(Panel{iPanel})
            var = Panel{iPanel}{iPic};
            var_it = max(find(var.time <= f.time(it)));
            %plot(var.arr(var_it)*exp(i*linspace(0,2*pi,1000))+[0,0]*[1 i].', ':w')
            %plot(var.arr(var_it, :) .* exp(i * P.arr(:,1,1,1)), ':w');
            
            [var.x, var.y] = pol2cart(squeeze(P.arr(:,1,1,1)), var.arr(var_it, :)');
            try
                var.linespecs;
            catch
                var.linespecs = ''':w''';
            end
            
            eval(['plot(var.x, var.y, ', var.linespecs, ')']);
            
    %        try
    %            names = {names{:}, var.name};
    %        end
        end

    %%    

        for iPanel = 2:length(Panel)
            sp1 = subplot('Position', [ax_x_min, ax1_y_bottom+ax1_y_size+ax1_y_gap+(iPanel-2)*(ax_y_size+ax_y_gap), ax_x_size, ax_y_size]);
            cla(gca)
            
            names = {};
            hold on
            for iPic = 1:length(Panel{iPanel})
                var = Panel{iPanel}{iPic};
                plot(var.time, var.arr, 'k', 'Color', colors(iPic,:));
                try
                    names = {names{:}, var.name};
                end
            end
            
            if (length(names) == 1)
                ylabel(names);
            else
                legend(names);
            end

            try xlim([f.time(1) f.time(end)]); end

            if (it == 1)
                if (iPanel == 2)
                    datetick('x', 'keeplimits');
                    xlabel('Time');
                else
                    set(sp1, 'xTick', []);
                end
            end
            
            plot([f.time(it) f.time(it)], ylim, 'k--');
            
        end
        
    %%        

        %%
        %save_picture(gcf, ['./Video/PSD_M=', num2str(unique(mu.arr)), '_', num2str(it, '%03.0f')], 'jpg', 'pdf', 'epsc');
        frame = getframe(f1);
        %mov = addframe(mov, frame);
        writeVideo(mov,frame);
    end

    %mov = close(mov);
    close(mov);

    %save_picture(gcf, ['./Plots/PSD_M=', num2str(unique(mu.arr))], 'jpg', 'pdf', 'epsc');

end







