function [varargout] = load_4d(filename, varargin)
    %%
    % LOAD_4D loads a 4D array in VERB 4D format
    %
    % [varargout] = load_4d(filename, varargin)
    %
    % Input:
    %   filename - file names
    %   optional arguments, varargin:
    %     nopermute - do not apply permute
    %     permute - do apply permute
    %     nozone - do not search for a 'ZONE' keyword and start reading data right after VARIABLES line
    % 
    % Output:
    %   data columns in file loaded as variables

    % Author: Dmitriy Subbotin
    % Email: subbotin@ucla.edu
    % Last change: 20014-01-01 by Dmitriy Subbotin
    % The work was supported by:
    %   LANL grant 12-LR-235337, PI Yuri Shprits
    %   NASA grant NNX09AF51G, PI Yuri Shprits
    
    nozone = false;
    dopermute = true;

    comments={};

    if (nargin > 1)
        for it = 1:1:nargin-1
            if strcmp(varargin(it), 'nopermute')
                dopermute = false;
            elseif strcmp(varargin(it), 'permute')
                dopermute = true;
    %            permute_mask = varargin(it+1);
            elseif strcmp(varargin(it), 'nozone')
                nozone = true;
            else
                varargin(it)
                error('Unknown parameter')
            end
        end
    end
    %    if (~(strcmp(varargin(1), 'dosqueeze')))
    %        dosqueeze = false;
    %    end;
    %end;


    fid = fopen(filename);

    % comments
    f_names = fgetl(fid);
    while (length(f_names) == 0 || f_names(1) == '#')
        comments = [comments, {f_names}];
        f_names = fgetl(fid);
    end;

    % var names
    f_names = regexpi(f_names, '"(.*?)"', 'match');
    n_of_vars = size(f_names,2);
    for it = 1:n_of_vars
    %%%    vars(it).name = strrep(f_names(it){1}, '"', ''); %%%
        vars(it).name = strrep(f_names{it}, '"', ''); %%%
        vars(it).comments = comments;
    end

    tmp = fgetl(fid);
    while (~nozone && ((length(tmp) < 4) || (strcmpi(tmp([1:4]), 'ZONE') ~= 1)))
        tmp = fgetl(fid);
    end;

    t = regexpi(tmp, 'T="(.*)"', 'tokens');      
    if (length(t) > 0) time = str2num(t{1}{1}); end;

    t = regexpi(tmp, 'I=\D*([0-9]+)', 'tokens');
    if (length(t) > 0) arr_size(4) = str2num(t{1}{1}); else arr_size(4) = 1; end;
    t = regexpi(tmp, 'J=\D*([0-9]+)', 'tokens');
    if (length(t) > 0) arr_size(3) = str2num(t{1}{1}); else arr_size(3) = 1; end;       
    t = regexpi(tmp, 'K=\D*([0-9]+)', 'tokens');
    if (length(t) > 0) arr_size(2) = str2num(t{1}{1}); else arr_size(2) = 1; end;       
    t = regexpi(tmp, 'L=\D*([0-9]+)', 'tokens');
    if (length(t) > 0) arr_size(1) = str2num(t{1}{1}); else arr_size(1) = 1; end;       

    total_size = arr_size(1) * arr_size(2) * arr_size(3) * arr_size(4);
    f_arr = fscanf(fid, '%f', [n_of_vars, total_size]);

    for var_number = 1:n_of_vars
        vars(var_number).time = time;
        if (var_number <= 4)
            vars(var_number).size = arr_size(var_number);
        end
        vars(var_number).size1 = arr_size(1);
        vars(var_number).size2 = arr_size(2);
        vars(var_number).size3 = arr_size(3);
        vars(var_number).size4 = arr_size(4);
        vars(var_number).arr(:, :, :, :) = reshape(f_arr(var_number, :), [arr_size(4), arr_size(3), arr_size(2), arr_size(1)]); 
    end

    if (dopermute)
        for it = 1:n_of_vars
            vars(it).arr = permute(vars(it).arr, [4, 3, 2, 1]);
            %var_tmp = vars(it).size1;
            %vars(it).size1 = vars(it).size3;
            %vars(it).size3 = var_tmp;
        end
    end

    nout = max(nargout,1);
    for k=1:nout
        try 
            varargout(k) = {vars(k)};
        catch 
            err = lasterror;
            if (err.identifier == 'MATLAB:badsubscript')
                err.message = ['Only ', num2str(k-1), ' variables in the file (', num2str(nout), ' requested).'];
                rethrow(err);
            else 
                rethrow(lasterror)
            end
        end
    end

    fclose (fid);

end
